#!/usr/bin/perl

use warnings;
use strict;

use Data::Dumper;

use SystemConfiguration;

if(@ARGV != 3 || $ARGV[0] !~ /\.xml$/ || $ARGV[2] !~ /\.h$/) {
	print STDERR "Usage: $0 <config.xml> <os.config> <SPICOM_distributer.h>\n";
	exit 1;
}

my $cfg = SystemConfiguration->new($ARGV[0], $ARGV[1]);

## Here comes the actual generation code
open (CPP, ">$ARGV[2]") || die "cannot open $ARGV[2]\n";

print CPP <<eofjs;
	spiDeviceData_t *msg;

	bool msgReceived = false;
	do {
		msgReceived = false;
eofjs

foreach my $msgP ($cfg->getMessagePorts()) {
	if ( $msgP->{name} =~ /^SPICom/ ) {
		my $name = $msgP->{name};
		$name =~ s/^SPICom//;

		print CPP <<eofjs;
		// ${name}
		msg = ${name}SPICom_receiver.receiveMessage();

		if ( msg != NULL ) {
			msgReceived = true;
			spiDeviceData_t tmpmsg = *msg;
			${name}SPICom_receiver.releaseMessage(msg);

			spiDeviceData_t* answer = SPICom${name}_sender.allocateMessage();
			if ( answer != 0x0 ) {
				*answer = tmpmsg;
				sendReceive(&tmpmsg, answer);
				SPICom${name}_sender.sendMessage(answer);
			}
		}
eofjs
	}
}

print CPP <<eofjs;
	} while ( msgReceived );
eofjs

close(CPP);
