#!/usr/bin/perl

use strict;
use warnings;

my $c = 0;

if(@ARGV and $ARGV[0] =~ /^-(c|java)$/) {
	$c = $1 eq 'c';	
} else {
	print STDERR "Usage: $0 (-c | -java) [numsets] [startvalue] <copterlogfile\n";
	exit 1;
}

my @data = ( );

my $formatline = <STDIN>;
my @format = split(';', $formatline);

while(my $line = <STDIN>) {
	my @fields = split(';',$line);
	die unless @fields == @format;
	my $res = { };

	for(my $i=0; $i<=$#fields; $i++) {
		$res->{$format[$i]} = $fields[$i];
	}

	push @data, $res;
}

my @ckeys = (qw(Pitch Roll Yaw Throttle), 'AngularSpeed X', 'AngularSpeed Y', 'AngularSpeed Z', 'Acceleration X', 'Acceleration Y');
my $jfields = join('', map {	my $id = $_; $id =~ s/\s/_/g; "\t\tpublic final float $id;\n" } @ckeys);
my $jparam = join(', ', map {	my $id = $_; $id =~ s/\s/_/g; "float $id" } @ckeys);
my $jfieldsinit = join('', map {	my $id = $_; $id =~ s/\s/_/g; "\t\t\tthis.$id = $id;\n" } @ckeys);

unless($c) {
	print <<EOH;
package flightcontrol.attitudecontroller;

class TestData {
	static final class ACData {
$jfields

		ACData($jparam) {
$jfieldsinit
		}
	}

EOH
}


if($c) {
	for(my $i=0; $i<=$#ckeys; $i++) {
		my $cident = $ckeys[$i];
		$cident =~ s/\s/_/g;
		print "#define $cident $i\n"; 
	}
}

print "\n";

my $numsets = @data;
my $curset = 0;
my $numsets2print = (@ARGV>=2 && $ARGV[1] < $#data) ? ($ARGV[1] - 1) : $#data;
my $startvalue = (@ARGV>=3 && $ARGV[2] >0 && ($ARGV[2] + $numsets2print) < $#data) ? $ARGV[2] : 0;

if($c) {
	print "#define NUM_INPUTSETS ".($numsets2print+1)."\n";
	print "float inputdata[NUM_INPUTSETS][".scalar(@ckeys)."] = {\n";
} else {
	print "\tpublic static final ACData[] inputdata = new ACData[".($numsets2print+1)."];\n";
}

my $currentPart = 0;
my $partIndex = 0;

foreach my $i ($startvalue..($startvalue+$numsets2print)) {
	my $iset = $data[$i];
	if($c) {
		print "\t{ " . join (', ', map { float2String($iset->{$_}) } @ckeys) . " },\n";
	} else {
		if($partIndex == 0) {
			print "\t\tprivate static void initPart$currentPart() {\n";
		}

		my $idx = $currentPart * 1000 + $partIndex;
		print "\t\t\tinputdata[$idx] = new ACData(" . join(', ', map { float2String($iset->{$_}) } @ckeys) . " );\n";

		$partIndex++; $idx++;
		if($partIndex == 1000 or $i==($startvalue+$numsets2print)) {
			$currentPart++;
			$partIndex = 0;
			print "\t\t}\n";
		}
	}
}

unless ($c) {
	print "\t\tstatic {\n";
	for my $i (0..($currentPart-1)) {
		print "\t\t\tinitPart$i();\n";
	}
	print "\t\t}\n}";
} else {
	print "};\n";
}

print STDERR scalar(@data) . " input data sets\n";

sub float2String {
	my $f = shift;
	$f .= '.0' if $f !~ /\./;
	return $f.'f';
}

