#include <stdlib.h>

#include "program.h"

prog_elem_t* NewProgElem() {
	prog_elem_t* elem = (prog_elem_t*) malloc(sizeof(prog_elem_t));
	elem->type    = start;
	elem->n       = 0;
	elem->rounds  = 0;
	elem->next    = NULL;
	elem->prior   = NULL;
	
	return elem;
}

program_t* NewProgram()
{
	program_t* program = (program_t*) malloc(sizeof(program_t));
	prog_elem_t* elem = NewProgElem();
	
	elem->type = start;
	
	program->firstElem   = elem;
	program->lastElem    = elem;
	program->errorPos    = -1;
	program->currentCoil = 0;
	
	return program;
}

void DeleteProgram(program_t* prog) {
	prog_elem_t* toDel;
	while(prog->firstElem != NULL) {
		toDel = prog->firstElem;
		prog->firstElem = prog->firstElem->next;
		free(toDel);
	}
	
	free(prog);
}

void AddProgElem(program_t* prog, prog_elem_t* progElem) {
	prog->lastElem->next = progElem;
	progElem->prior = prog->lastElem;
	prog->lastElem = progElem;
}

void UpContinuous(program_t* prog, int n) {
	prog_elem_t* elem = NewProgElem();
	
	elem->type   = up_continuous;
	elem->n      = n;
	
	AddProgElem(prog, elem);
}


void DownContinuous(program_t* prog, int n) {
	prog_elem_t* elem = NewProgElem();
	
	elem->type   = down_continuous;
	elem->n      = n;
	
	AddProgElem(prog, elem);
}


void UpStepwise(program_t* prog, int n) {
	prog_elem_t* elem = NewProgElem();
	
	elem->type   = up_stepwise;
	elem->n      = n;
	
	AddProgElem(prog, elem);
}


void DownStepwise(program_t* prog, int n) {
	prog_elem_t* elem = NewProgElem();
	
	elem->type   = down_stepwise;
	elem->n      = n;
	
	AddProgElem(prog, elem);
}


void OscContinuous(program_t* prog, int n, int rounds) {
	prog_elem_t* elem = NewProgElem();
	
	elem->type   = osc_continuous;
	elem->n      = n;
	elem->rounds = rounds;
	
	AddProgElem(prog, elem);
}


void OscStepwise(program_t* prog, int n, int rounds) {
	prog_elem_t* elem = NewProgElem();
	
	elem->type   = osc_stepwise;
	elem->n      = n;
	elem->rounds = rounds;
	
	AddProgElem(prog, elem);
}


void Hold(program_t* prog, int n) {
	prog_elem_t* elem = NewProgElem();
	
	elem->type   = hold;
	elem->n      = n;
	
	AddProgElem(prog, elem);
}
