#
# MODULE: compile
#
# find and compile Sources

# find sources
SRC ?= $(shell find $(SRC_DIR) -follow -name '*.cpp' ) \
       $(shell find $(SRC_DIR) -follow -name '*.c' )
ASM ?= $(shell find $(SRC_DIR) -follow -name '*.s' ) \
       $(shell find $(SRC_DIR) -follow -name '*.S' )
NASM_ASM ?= $(shell find $(SRC_DIR) -follow -name '*.asm' )
NASM16_ASM ?= $(shell find $(SRC_DIR) -follow -name '*.asm16' )

AH  ?= $(shell find $(SRC_DIR) -follow -name '*.ah' )
#Check, if extra (application-specific) aspects have to be considered
ifdef EXTRA_AH_DIR
	AH  += $(shell find $(EXTRA_AH_DIR) -follow -name '*.ah' )
endif

OUT_DIR ?= $(LIB_DIR)

# generate object files
OBJ  = $(patsubst $(SRC_DIR)/%.c,$(OUT_DIR)/%.o,$(patsubst $(SRC_DIR)/%.cpp,$(OUT_DIR)/%.o,$(SRC))) 
OBJ += $(patsubst $(SRC_DIR)/%.S,$(OUT_DIR)/%.o,$(patsubst $(SRC_DIR)/%.s,$(OUT_DIR)/%.o,$(ASM)))
OBJ += $(patsubst $(SRC_DIR)/%.asm,$(OUT_DIR)/%.o,$(patsubst $(SRC_DIR)/%.asm,$(OUT_DIR)/%.o,$(NASM_ASM)))
OBJ += $(patsubst $(SRC_DIR)/%.asm16,$(OUT_DIR)/%.bin16,$(patsubst $(SRC_DIR)/%.asm16,$(OUT_DIR)/%.bin16,$(NASM16_ASM)))

# add clean 
MODULES_CLEAN += compile

# add all 
MODULES_ALL += compile

# 'clean' target for 'compile' MODULE
clean_compile: 
	@find $(OUT_DIR) -name "*.o" -delete 

# 'all' target for 'compile' MODULE
all_compile: $(OBJ)

test_compile:
	@echo SRC=$(SRC)
	@echo ASM=$(ASM)
	@echo OUT_DIR=$(OUT_DIR)
	@echo SRC_DIR=$(SRC_DIR)
	@echo OBJ=$(OBJ)
	@echo CXXFLAGS= $(CXXFLAGS)
	@echo ROOT_DIR= $(ROOT_DIR)
	
# pretty print pathes
printpath=$(1:$(ROOT_DIR)/%=%)


# Compile a C File in the '$(SRC_DIR)' directory
$(OUT_DIR)/%.o: $(SRC_DIR)/%.c 
	@echo "($(XX))    $(call printpath,$@)"
	@if [ ! -e $(@D) ];then mkdir -p $(@D);fi 
	@$($(XX)) -xc -c $< -o $@ 

# Compile a C++ File in the '$(SRC_DIR)' directory
$(OUT_DIR)/%.o: $(SRC_DIR)/%.cpp $(AH)
	@echo "($(XX))    $(call printpath,$@) $(CXXFLAGS_EXTRA)"
#	@echo "            + active : $(EXTRA_AH_DIR)"
#	@echo "            + active aspects: $(AH)"
	@if [ ! -e $(@D) ];then mkdir -p $(@D);fi
	@$($(XX)) -c $< -o $@ 

# Compile a Assembler File in the '$(SRC_DIR)' directory
$(OUT_DIR)/%.o: $(SRC_DIR)/%.s
	@echo "(AS)      $(call printpath,$@)"
	@if [ ! -e $(@D) ];then mkdir -p $(@D);fi 
	@$(AS) -c $< -o $@ 

# Compile a preprocessed Assembler File in the '$(SRC_DIR)' directory
$(OUT_DIR)/%.o: $(SRC_DIR)/%.S
	@echo "(AS)      $(call printpath,$@)"
	@if [ ! -e $(@D) ];then mkdir -p $(@D);fi 
	@$(AS) -c $< -o $@ 

# Compile a NASM-Assembler File in the '$(SRC_DIR)' directory
$(OUT_DIR)/%.o: $(SRC_DIR)/%.asm
	@echo "(NASM)    $(call printpath,$@)"
	@if [ ! -e $(@D) ];then mkdir -p $(@D);fi
	@$(NASM) -g $(NASMFLAGS) -Ox $< -o $@

# Compile a 16-Bit NASM-Assembler File and create flat binary
$(OUT_DIR)/%.bin16: $(SRC_DIR)/%.asm16
	@echo "(NASM)    $(call printpath,$@)"
	@if [ ! -e $(@D) ];then mkdir -p $(@D);fi
	@$(NASM) -f bin $< -o $@


.PRECIOUS: $(LIB_DIR)/%.o
