// $Id: AS.h 1276 2010-09-14 10:27:54Z mike $
// Created: 2007-08-02 siwahofe

#ifndef __AS_H__
#define __AS_H__

#include "autosar_types.h"
#include "os/krn/SchedImpl.h"
#include "os/krn/OSControl.h"

/**
 * The class AS
 * - provides the complete (configured) user interface to the OS
 * - has no state of its own
 * - is completely static
 */
class AS {
public:
	// task services

	static StatusType ActivateTask (TaskType taskid) {
		os::krn::SchedImpl::Inst ().activate (taskid);
		return E_OK;
	}
	static StatusType TerminateTask ();
	static StatusType ChainTask (TaskType taskid);
	static StatusType Schedule () {
		os::krn::SchedImpl::Inst ().reschedule ();
		return E_OK;
	}
	static StatusType GetTaskID (TaskRefType taskid) {
		taskid = os::krn::SchedImpl::Inst ().getRunning ();
		return E_OK;
	}
	static StatusType GetTaskState (TaskType taskid, TaskStateRefType state) {
		state = os::krn::SchedImpl::Inst ().getTaskState (taskid);
		return E_OK;
	}

	// ISR category 1 services
	// introduced if configured

	// ISR category 2 services
	// introduced if configured

	// resource services
	// introduced if configured
	
	// event services
	// introduced if configured
	
	// alarm services
	// introduced if configured
	
	// OS services
	
	static void StartOS (AppModeType appmode) {
		os::krn::OSControl::Inst ().startOS (appmode);
	}
	static void ShutdownOS (StatusType error) __attribute__((noreturn)) {
		os::krn::OSControl::Inst ().shutdownOS (error);
	}
	static AppModeType GetActiveApplicationMode () {
		return os::krn::OSControl::Inst ().getApplicationMode ();
	}
};

// call has to be inlined in order not to waste CSAs
// when terminating
inline StatusType CIAO_INLINE AS::TerminateTask () {
	os::krn::SchedImpl::Inst ().exitRunning ();
	// no return to call level in standard mode
	return E_OK;
}

// call has to be inlined in order not to waste CSAs
// when terminating
inline StatusType CIAO_INLINE AS::ChainTask (TaskType taskid) {
	os::krn::SchedImpl::Inst ().chain (taskid);
	// no return to call level in standard mode
	return E_OK;
}

#endif // __AS_H__
