// $Id: autosar_pointcuts.h 1281 2010-09-23 13:07:33Z mike $
// Created: 2007-06-20 siwahofe
#ifndef __AUTOSAR_POINTCUTS_H__
#define __AUTOSAR_POINTCUTS_H__

// every pointcut defined here has the prefix "as"

// return types
pointcut asReturningVoid () = "void ...::% (...)";
pointcut asReturningStatusType () = "StatusType ...::% (...)";

// user code
pointcut asTaskFunctions () = "% ...::functionTask% (...)";
pointcut asISR2Functions () = "% ...::functionISR% (...)";

// AUTOSAR services and groups
pointcut asActivateTask () = "% AS::ActivateTask (...)";
pointcut asTerminateTask () = "% AS::TerminateTask (...)";
pointcut asChainTask () = "% AS::ChainTask (...)";
pointcut asSchedule () = "% AS::Schedule (...)";
pointcut asGetTaskID () = "% AS::GetTaskID (...)";
pointcut asGetTaskState () = "% AS::GetTaskState (...)";
pointcut asTaskServices () =
	asActivateTask () ||
	asTerminateTask () ||
	asChainTask () ||
	asSchedule () ||
	asGetTaskID () ||
	asGetTaskState ();

pointcut asEnableAllInterrupts () = "% AS::EnableAllInterrupts (...)";
pointcut asDisableAllInterrupts () = "% AS::DisableAllInterrupts (...)";
pointcut asEnableInterrupt () = "% AS::EnableInterruptSource (...)";
pointcut asDisableInterrupt () = "% AS::DisableInterruptSource (...)";
pointcut asResumeAllInterrupts () = "% AS::ResumeAllInterrupts (...)";
pointcut asSuspendAllInterrupts () = "% AS::SuspendAllInterrupts (...)";
pointcut asResumeOSInterrupts () = "% AS::ResumeOSInterrupts (...)";
pointcut asSuspendOSInterrupts () = "% AS::SuspendOSInterrupts (...)";
pointcut asInterruptServices () =
	asEnableAllInterrupts () ||
	asDisableAllInterrupts () ||
	asResumeAllInterrupts () ||
	asSuspendAllInterrupts () ||
	asResumeOSInterrupts () ||
	asSuspendOSInterrupts ();

pointcut asGetResource () = "% AS::GetResource (...)";
pointcut asReleaseResource () = "% AS::ReleaseResource (...)";
pointcut asResourceServices () =
	asGetResource () ||
	asReleaseResource ();

pointcut asSetEvent () = "% AS::SetEvent (...)";
pointcut asClearEvent () = "% AS::ClearEvent (...)";
pointcut asGetEvent () = "% AS::GetEvent (...)";
pointcut asWaitEvent () = "% AS::WaitEvent (...)";
pointcut asEventServices () =
	asSetEvent () ||
	asClearEvent () ||
	asGetEvent () ||
	asWaitEvent ();

pointcut asGetAlarmBase () = "% AS::GetAlarmBase (...)";
pointcut asGetAlarm () = "% AS::GetAlarm (...)";
pointcut asSetRelAlarm () = "% AS::SetRelAlarm (...)";
pointcut asSetAbsAlarm () = "% AS::SetAbsAlarm (...)";
pointcut asCancelAlarm () = "% AS::CancelAlarm (...)";
pointcut asAlarmServices () =
	asGetAlarmBase () ||
	asGetAlarm () ||
	asSetRelAlarm () ||
	asSetAbsAlarm () ||
	asCancelAlarm ();

pointcut asGetActiveApplicationMode () = "% AS::GetActiveApplicationMode (...)";
pointcut asStartOS () = "% AS::StartOS (...)";
pointcut asShutdownOS () = "% AS::ShutdownOS (...)";
pointcut asOSServices () =
	asGetActiveApplicationMode () ||
	asStartOS () ||
	asShutdownOS ();

// general
pointcut asAllServices () =
	asTaskServices () ||
	asInterruptServices () ||
	asResourceServices () ||
	asEventServices () ||
	asAlarmServices () ||
	asOSServices ();

// AUTOSAR internal hooks

pointcut asInternalPreTaskHook () ="% os::krn::SchedImpl::internalPreTaskHook (...)";
pointcut asInternalPostTaskHook () ="% os::krn::SchedImpl::internalPostTaskHook (...)";
pointcut asInternalStartupHook () ="% os::krn::SchedImpl::internalStartupHook (...)";
pointcut asInternalShutdownHook () ="% os::krn::SchedImpl::internalShutdownHook (...)";
pointcut asInternalErrorHook () ="% os::krn::SchedImpl::internalErrorHook (...)";

//pointcut as () = "% ErrorHook% (...)" ||
//pointcut as () = "% PreTaskHook (...)" ||
//pointcut as () = "% PostTaskHook (...)" ||
//pointcut as () = "% StartupHook% (...)" ||
//pointcut as () = "% ShutdownHook% (...)";
//pointcut hookRoutines () =

//pointcut otherServices () =
//	"% AS::GetISRID (...)";
//
//pointcut allServices () =

// ComStack interfaces pointcuts
pointcut asCanIf_Transmit () = "% AS::CanIf_Transmit(...)";
pointcut asCanIf_SetControllerMode () = "% AS::CanIf_SetControllerMode(...)";
pointcut asCanIf_ReadTxNofifStatus () = "% AS::CanIf_ReadTxNotifStatus(...)";
pointcut asCanIf_ReadRxNofifStatus () = "% AS::CanIf_ReadRxNotifStatus(...)";
pointcut asCanIf_ReadRxPduData () = "% AS::CanIf_ReadRxPduData(...)";
pointcut asCom_MainFunctionTx () = "% AS::Com_MainFunctionTx(...)";
pointcut asCom_MainFunctionRx () = "% AS::Com_MainFunctionRx(...)";
pointcut asCom_SendSignal () = "% AS::Com_SendSignal(...)";
pointcut asCom_ReceiveSignal () = "% AS::Com_ReceiveSignal(...)";
pointcut asCom_TxIpduGroupStart () = "% AS::Com_TxIpduGroupStart (...)";
pointcut asCom_RxIpduGroupStart () = "% AS::Com_RxIpduGroupStart (...)";
pointcut asCom_InvalidateSignal () = "% AS::Com_InvalidateSignal (...)";
pointcut asCom_UpdateShadowSignal () = "% AS::Com_UpdateShadowSignal (...)";
pointcut asCom_InvalidateShadowSignal () = "% AS::Com_InvalidateShadowSignal (...)";
pointcut asCom_InvalidateSignalGroup () = "% AS::Com_InvalidateSignalGroup (...)";
pointcut asCom_SendSignalGroup () = "% AS::Com_SendSignalGroup (...)";
pointcut asCom_ReceiveSignalGroup () = "% AS::Com_ReceiveSignalGroup (...)";
pointcut asCom_ReceiveShadowSignal () = "% AS::Com_ReceiveShadowSignal (...)";
// FIXME check missing synchronization points of AsCom

pointcut kernelCallsToSynchronize () =
	asActivateTask () ||
	asTerminateTask () ||
	asSchedule () ||
	asShutdownOS () ||
	asGetResource () ||
	asReleaseResource () ||
	asSetEvent () ||
	asClearEvent () ||
	asWaitEvent () ||
	asGetAlarm () ||
	asSetRelAlarm () ||
	asSetAbsAlarm () ||
	asCancelAlarm () ||
	asCanIf_Transmit () ||
	asCanIf_SetControllerMode () ||
//	asCanIf_ReadTxNofifStatus () ||		For now don't have to be synced,
//	asCanIf_ReadRxNofifStatus () ||     because algo has been adopted
	asCanIf_ReadRxPduData () ||
	asCom_MainFunctionTx () ||
	asCom_MainFunctionRx () ||
	asCom_SendSignal () ||
	asCom_ReceiveSignal () ||
	asCom_RxIpduGroupStart () ||
	asCom_TxIpduGroupStart () ||
	asCom_InvalidateSignal () ||
	asCom_UpdateShadowSignal () ||
	asCom_InvalidateShadowSignal () ||
	asCom_InvalidateSignalGroup () ||
	asCom_SendSignalGroup () ||
	asCom_ReceiveSignalGroup () ||
	asCom_ReceiveShadowSignal ();

pointcut asConstServices() = asGetTaskID () ||
	asGetTaskState () ||
	asGetAlarmBase () ||
	asGetAlarm () ||
	asGetEvent () ||
	asGetActiveApplicationMode ();
 
#endif // __AUTOSAR_POINTCUTS_AH__
