// $Id: iPort.h 995 2009-08-11 14:26:31Z mike $

#ifndef __iPort_h__
#define __iPort_h__

/**
 * \file iPort.h
 * \brief abstract i/o port interface
 */
 
/*! \class IPort
 * \brief abstract i/o port interface
 *
 * This abstract class is the common interace of all kinds of port classes.
 * A port is used for I/O and can either be memory mapped or in a separate
 * I/O address space. As port can be of 8/16/32... bits width the interface
 * is defined as a template.
 *
 * Note that in CiAO it is not implicit that all port classes are derived
 * from the interface. It depends on the system configuration.
 */

namespace ciao {
    
    template <typename T> iPort {

    public:

	//! read from the port
	operator T () volatile = 0;
	
	//! write to the port
	iPort<T>& operator  = (T) = 0;

	//! clear bits with boolean 'and' operation
	iPort<T>& operator &= (T) = 0;

	//! set bits with boolean 'or' operation
	iPort<T>& operator |= (T) = 0;
    };

} // namespace ciao

#endif // __iPort_h__
