// $Id: triboard1796.h 1211 2010-07-16 13:05:25Z sijesche $
// Created:   2006-03-04 lohmann

/**
 * \file triboard1796.h
 * \brief TriCore-specific types, macros and intrinsics
 * 
 * Includes all required tricore-gcc-specific headers for hw-registers
 * Tricore-specific types are placed in the namespace hw::_tc. Macros and
 * intrinsics are, however, in the global namespace.
 * 
 */
 
 
#ifndef __TRIBOARD1796_H__
#define __TRIBOARD1796_H__

#include "util/intrinsics.h"

	
// We include the tricore-gcc hardware abstractions (register definitions)
// into the namespace hw::_tc. 


#include "tricore-abs-noabs-FIX.h"
#include <tc1796b/csfr.h>
#include <tc1796b/scu.h>


namespace hw { namespace _tc {
//! Layout of an TriCore upper context
struct  __attribute__((aligned(16)))  UpperContext {
	union {
		PCXI_t_nonv		pcxi_typed;
		void			*pcxi;
	};
	union {
		PSW_t_nonv		psw_typed;
		long			psw;
	};
	union {
		void			*a10;
		void			*sp;
	};
	union {
		void			*a11;
		void			*ra;
	};
	long				d8, d9, d10, d11;
	void				*a12, *a13, *a14, *a15;
	long				d12, d13, d14, d15;
};

//! Layout of an TriCore lower context
struct  __attribute__((aligned(16)))  LowerContext {
	union {
		PCXI_t_nonv		pcxi_typed;
		void			*pcxi;
	};
	union {
		void			*a11;
		void			*ra;
	};
	void 				*a2, *a3;
	long 				d0, d1, d2, d3;
	void 				*a4, *a5, *a6, *a7;
	long 				d4, d5, d6, d7;
};

union HalfContext {
	UpperContext uc;
	LowerContext lc;
	long regs[ 32 ];
};

/// Returns the absolute address (effective address) from a CXI link word
inline HalfContext* cx2ea( PCXI_t_nonv cx )  {
	unsigned int res = ( (cx.bits.PCXO << 6) ) | ( (cx.bits.PCXS << 28) );
	return (HalfContext*) res; 
}
	
/// Sets the offset and segement of a CXI link word to an effective address
/// All other parts of the CXI word are not modified
inline void setcxea( PCXI_t_nonv& cx, HalfContext* ea ) {
	cx.bits.PCXO = (long) ea >> 6;
	cx.bits.PCXS = (long) ea >> 12;
}

/// Builds a CXI link word	
inline PCXI_t_nonv ea2cx( HalfContext* ea, bool UL, bool PIE = false, unsigned char PCPN = 0 ) {
	PCXI_t_nonv cx;
	setcxea( cx, ea);
	cx.bits.UL 		= UL,
	cx.bits.PIE 	= PIE;
	cx.bits.PCPN 	= PCPN;
	return cx;
}

/**
 * \brief Adds a linked list of CSAs to the free list
 * tail.pcxi = $fcx
 * $fcx = head 
 */

inline void free_cx_list( PCXI_t_nonv head, PCXI_t_nonv tail ) {
	HalfContext* tail_ea = cx2ea( tail );
	_disable();
	tail_ea->lc.pcxi_typed.reg = _mfcr( $fcx );
//	_dsync();					// required here?
	_mtcr( $fcx, head.reg );
	_isync();
	_enable();		
}

/// unlocks (clears) the TriCore ENDINIT bit to modify ENDINIT protected registers
void unlock_wdtcon();

/// locks (sets) the TriCore ENDINIT bit
void lock_wdtcon();

/// introduces a reorder barrier (see also trigcc user's guide, p. 56)
//inline void _reorder_barrier () {
#define _reorder_barrier() __asm__ __volatile__( "": : : "memory")
//}

} } // namespace hw::_tc
#endif // __TRIBOARD1796_H__
