// $Id$
// Created:   Apr 11, 2006 lohmann

/**
 * \file CGU.cpp
 * \brief Implementation of the TC1796 Clock Generation Unit
 *
 */

#include "CGU.h"
#include "util/assert.h"
#include "hw/triboard1796.h"

namespace hw { namespace dev {

CGU CGU::theCGU_;

void CGU::init() {

	PLL_CLC_t_nonv pllclc;
	PLL_CLC_t_nonv pllclc_tmp;
	pllclc.reg = 0;
	pllclc_tmp.reg =0;
	
	// we use PLL-mode:
	// clockCPU = (N * OSCFREQ) / (P * K)
	// clockSYS = clockCPU / 2
	
	//damit die PLL sich einschwingen kann
	pllclc.bits.VCOBYP =1; //VCO erst mal von der Sysclock abkoppeln
	pllclc.bits.RESLD = 1; //Warten, bis PLL eingeschwungen ist
	
	pllclc.bits.NDIV 	= PLL_NDIV -1;
	pllclc.bits.KDIV 	= PLL_KDIV -1;
	pllclc.bits.PDIV 	= PLL_PDIV -1;
	pllclc.bits.SYSFS	= PLL_SYSDIV == 2 ? 0 : 1;
	pllclc.bits.VCOSEL	=  2;

	//auf pll warten
	PLL_CLC.reg = pllclc.reg;
	pllclc.bits.VCOBYP = 0;
	pllclc.bits.RESLD = 0;		

	hw::_tc::lock_wdtcon(); //kann dauern, daher watchdog abschalten	
	while (!pllclc_tmp.bits.LOCK) {
		pllclc_tmp.reg = PLL_CLC.reg;
	}
	hw::_tc::unlock_wdtcon(); //watchdog wieder an, damit reg beschreibbar wird
	
	PLL_CLC.reg = pllclc.reg;	
}

CGU::Frequency CGU::getRealCPUClockFreq() {
	PLL_CLC_t_nonv pllclc;
	pllclc.reg = PLL_CLC.reg;
	
	if( pllclc.bits.BYPPIN )
		// clock rate is directly derived from the oscillator
		return OSCFREQ;
	else {
		// clock rate is generated by PLL
		unsigned int k_div, n_div, p_div;
		n_div = 1 + pllclc.bits.NDIV;
		k_div = 1 + pllclc.bits.KDIV;
		p_div = 1 + pllclc.bits.PDIV;

		return OSCFREQ * n_div / (k_div * p_div);
	}
}



}} // namespace hw::dev
