// $Id: P0.h 995 2009-08-11 14:26:31Z mike $
// Created:   05.06.2006 lohmann

/**
 * @file P0.h
 * @brief Contains abstractions for the TC1796 General Purpose IO (GPIO) lines
 */
 
#ifndef HW_DEV_P0_H_
#define HW_DEV_P0_H_

#include "hw/dev/GPIO.h"

namespace hw { namespace dev {

/**
 * @brief TC1796 GPIO Port 0
 * 
 * Port 0 is a general-purpose 16-bit bi-directional port. It serves as GPIO lines without
 * secondary functions. The logic state of the sixteen Port 0 lines is, however, stored in the SCU
 * Software Configuration Latched Input Register SCU_SCLIR at hardware reset time
 * and can be read by software afterwards. This might be used for power-on configuration settings.
 * 
 * @note P0.5-P0.15 are connected to the red LEDs of our experimental board.
 */
class P0 : public PortBase< 0, 16, 0xffff > {
	static P0 theP0_;
	//! Performs start-time initializations. Only called by advice.
	void init() {
		unsigned long* p = &port().IOCR[0].reg;
		*p++ = cfDEV_P0_IOCR0;
		*p++ = cfDEV_P0_IOCR4;
		*p++ = cfDEV_P0_IOCR8;
		*p   = cfDEV_P0_IOCR12;
	}
  protected:
    P0(){}
  public:	
	//! Returns the one-and-only P0 instance
	static P0& Inst() {
		return theP0_;
	}
};

}} // namespace hw::dev
#endif /*HW_DEV_P0_H_*/
