// $Id: P4.h 1039 2009-10-30 14:17:24Z sijesche $
// Created:   05.06.2006 lohmann

/**
 * @file P4.h
 * @brief Contains abstractions for the TC1796 General Purpose IO (GPIO) lines
 */
 
#ifndef HW_DEV_P4_H_
#define HW_DEV_P4_H_

#include "hw/dev/GPIO.h"

namespace hw { namespace dev {

/**
 * @brief TC1796 GPIO Port 4
 * 
 * Port 4 is a general-purpose 16-bit bi-directional port. It serves as GPIO lines without
 * secondary functions. The logic state of the sixteen Port 4 lines is, however, stored in the SCU
 * Software Configuration Latched Input Register SCU_SCLIR at hardware reset time
 * and can be read by software afterwards. This might be used for power-on configuration settings.
 * 
 */
class P4 : public PortBase< 4, 16, 0xffff > {
	static P4 theP4_;
	//! Performs start-time initializations. Only called by advice.
	void init() {
		unsigned long* p = &port().IOCR[0].reg;
		*p++ = cfDEV_P4_IOCR0;
		*p++ = cfDEV_P4_IOCR4;
		*p++ = cfDEV_P4_IOCR8;
		*p   = cfDEV_P4_IOCR12;
	}
  protected:
    P4(){}
  public:	
	//! Returns the one-and-only P0 instance
	static P4& Inst() {
		return theP4_;
	}
};

}} // namespace hw::dev
#endif /*HW_DEV_P4_H_*/
