// $Id: P7.h 234 2006-06-07 05:01:08Z lohmann $
// Created:   05.06.2006 lohmann

/**
 * @file P7.h
 * @brief Contains abstractions for the TC1796 General Purpose IO (GPIO) lines
 */
 
#ifndef HW_DEV_P7_H_
#define HW_DEV_P7_H_

#include "hw/dev/GPIO.h"

namespace hw { namespace dev {

/**
 * @brief TC1796 GPIO Port 7
 *
 * Port 7 is an 8-bit bi-directional general-purpose I/O port that can be used
 * for the external trigger input lines REQ[7:4] or for the ADC0/ADC1 external
 * multiplexer control output lines. 
 */
class P7 : public PortBase< 7, 8, 0xff > {
	static P7 theP7_;
	//! Performs start-time initializations. Only called by advice.
	void init() {
		unsigned long* p = &port().IOCR[0].reg;
		*p++ = cfDEV_P7_IOCR0;
		*p   = cfDEV_P7_IOCR4;
	}
  protected:
    P7(){}
  public:	
	//! Returns the one-and-only P7 instance
	static P7& Inst() {
		return theP7_;
	}
};

}} // namespace hw::dev
#endif /*HW_DEV_P7_H_*/
