#ifndef __HW_DEV_IOPORT_H__
#define __HW_DEV_IOPORT_H__

#include "util/types.h"

namespace hw {

namespace dev {

class IOPort {

	UInt16 address;

public:
	IOPort(UInt16 addr = 0) :
		address(addr) {
	}

	void setAddress(UInt16 addr) {
		address = addr;
	}

	UInt16 getAddress() {
		return address;
	}

	IOPort offset(UInt16 offset) {
		return IOPort(address + offset);
	}

	void outb(UInt8 val) {
		asm volatile (
				"outb %0, %1"
				:
				: "a"(val), "d"(address)
		);
	}

	void outw(UInt16 val) {
		asm volatile (
				"outw %0, %1"
				:
				: "a"(val), "d"(address)
		);
	}

	void outl(UInt32 val) {
		asm volatile (
				"outl %0, %1"
				:
				: "a"(val), "d"(address)
		);
	}

	UInt8 inb() {
		UInt8 val;

		asm volatile ("xorl %%eax, %%eax\n"
				"inb %1, %0"
				: "=a"(val)
				: "d"(address)
		);

		return val;
	}

	UInt16 inw() {
		UInt16 val;

		asm volatile ("xorl %%eax, %%eax\n"
				"inw %1, %0"
				: "=a"(val)
				: "d"(address)
		);

		return val;
	}

	UInt32 inl() {
		UInt32 val;

		asm volatile (
				"inl %1, %0"
				: "=a"(val)
				: "d"(address)
		);

		return val;
	}
};

}

}

#endif // __HW_DEV_IOPORT_H__
