#include "stdinc.h"

#include "PCI.h"

namespace hw {
namespace dev {

PCI PCI::inst_;

void PCI::init() {

	config_address_.setAddress(IO_PORT_CONFIG_ADDRESS);
	config_data_.setAddress(IO_PORT_CONFIG_DATA);

	configspace_entries_ = 0;

	// Enumerate PCI-Host-Bridge
	int next_free_busnum = 1;

	for (int i = 0; i < 255; ++i) {
		enumerate_bus(i, next_free_busnum);
	}
}

void PCI::enumerate_bus(int bus, int& next_free_busnum) {
	for (unsigned unit = 0; unit < 32; ++unit) {
		for (unsigned function = 0; function < 8; ++function) {

			PCIConfigSpace cs;

			unsigned tmp = getConfigSpaceData(bus, unit, function, 0);
			cs.device = tmp >> 16;

			if (cs.device != 0xFFFF) { // PCI-Device present

				cs.bus = bus;
				cs.unit = unit;
				cs.function = function;

				cs.vendor = tmp & 0xFFFF;

				tmp = getConfigSpaceData(bus, unit, function, 2);
				cs.classcode = tmp >> 8;
				cs.revision = tmp & 0xFF;

				tmp = getConfigSpaceData(bus, unit, function, 3);
				cs.headertype = (tmp >> 16) & 0xFF;

				cs.bar0 = getConfigSpaceData(bus, unit, function, 4);
				cs.bar1 = getConfigSpaceData(bus, unit, function, 5);
				cs.bar2 = getConfigSpaceData(bus, unit, function, 6);
				cs.bar3 = getConfigSpaceData(bus, unit, function, 7);
				cs.bar4 = getConfigSpaceData(bus, unit, function, 8);
				cs.bar5 = getConfigSpaceData(bus, unit, function, 9);

				cs.rom_base = getConfigSpaceData(bus, unit, function, 12);

				tmp = getConfigSpaceData(bus, unit, function, 15);
				cs.interrupt_pin = (tmp >> 8) & 0xFF;
				cs.interrupt_line = tmp & 0xFF;

				if (configspace_entries_ < CONFIGSPACE_MAXSIZE) {
					configspace_[configspace_entries_++] = cs;
				}

				// TODO PCI-Bridge without headertype 0x80?
				//if (cs.headertype == 0x80 && (cs.classcode >> 8) == 0x0604  ) { // PCI-PCI-Bridge
				if ((cs.classcode >> 8) == 0x0604) { // PCI-PCI-Bridge

					// TODO: Funktion rekursiv auf alle PCI-PCI-Bridges

				}
			} else { // No PCI-Device found
				continue;
			}
		}
	}
}
}
}
