/** \author Christian Meier
 *  \version $Id: CanDriverImpl.cpp 995 2009-08-11 14:26:31Z mike $
 */

#include "hw/hal/CPU.h"

#include "CanDriverImpl.h"

namespace hw { namespace dev { namespace can { namespace multican {

CanDriverImpl CanDriverImpl::theMultican_;

void
CanDriverImpl::Init(const ConfigType * p_Config) {
	// First we initialize all Interrupt disable counters to zero
	for(UInt i = 0; i < cfHW_DEV_CAN_MULTICAN_NUM_CONTROLLERS; ++i) {
		_DisableInterruptCounter[i] = 0;
	}

	// Clock must be generated correctly
	ClockManager::Inst().Init(p_Config);

	// PendingHwObjManager must be initialized
	PendingHwObjManager::Inst().Init(p_Config);

	// In order to initialize ListManager, we must enable CCE (Configuration
	// Change Enable) bits for ALL nodes alias ControllerUnits
	Controller::EnableAllCCE();
	ListManager::Inst().Init(p_Config);
	Controller::DisableAllCCE();

	// HwObjects configuration
	HwObject::InitAll(p_Config->m_HwObjConfigArray,
					  p_Config->m_NumHwObjConfigs);

	// Controller configuration
	Controller::InitAll(p_Config->m_CtrlConfigArray,
						p_Config->m_NumCtrlConfigs);
}

void
CanDriverImpl::InitController(ControllerIdType ControllerId, ControllerConfigType const * p_Config) {
	Controller::Inst(ControllerId).InitController(p_Config);
}

ReturnType
CanDriverImpl::SetControllerMode(ControllerIdType ControllerId, StateTransitionType Transition) {
	Controller::Inst(ControllerId).SetControllerMode(Transition);
	if (_DisableInterruptCounter[ControllerId] == 0) { // req: CAN196 + CAN197
		// Only if DisableControllerInterrupts hasn't been called before,
		// or enough EnableControllerInterrupts have been called, we shall
		// perform interrupt initialization for this controller

		Controller::EnableInterrupts(ControllerId);
		return CAN_OK;
	}
	return CAN_NOT_OK; // FIXME Check whether we shall return NOT_OK or OK
}

void
CanDriverImpl::DisableControllerInterrupts(ControllerIdType ControllerId) {
	hw::hal::CPU::IEState State;
	hw::hal::CPU::disable(State);
	_DisableInterruptCounter[ControllerId]++;
	if (_DisableInterruptCounter[ControllerId] == 1) {
		Controller::DisableInterrupts(ControllerId);
	}
	hw::hal::CPU::enable(State);
}

void
CanDriverImpl::EnableControllerInterrupts(ControllerIdType ControllerId) {
	hw::hal::CPU::IEState State;
	hw::hal::CPU::disable(State);
	if (_DisableInterruptCounter[ControllerId] != 0) {		/* req: CAN208 */
		_DisableInterruptCounter[ControllerId]--;
		if (_DisableInterruptCounter[ControllerId] == 0) {
			Controller::EnableInterrupts(ControllerId);
		}
	}
	hw::hal::CPU::enable(State);
}

void
CanDriverImpl::MainFunction_BusOff(void) {
	// If Polling or mixed mode for busoff is configured, there are aspects that bind here.
}

}}}}

