/** \author Christian Meier
 *  \version $Id: ClockManager.h 995 2009-08-11 14:26:31Z mike $
 */

#ifndef HW_DEV_CAN_MULTICAN_CLOCK_MANAGER_H
#define HW_DEV_CAN_MULTICAN_CLOCK_MANAGER_H

#include "util/types.h"
#include "../cfAttribs.h"
#include "hw/dev/can/ConfigTypes.h"
#include "RegTypes.h"
#include "Addr.h"

namespace hw { namespace dev { namespace can { namespace multican {

class ClockManager {
public:
	static ClockManager & Inst() {
		return * (ClockManager *) (HW_DEV_CAN_MULTICAN_MODULE_BASE_ADDRESS
								  + HW_DEV_CAN_MULTICAN_CLOCK_MANAGER_OFFSET);
	}

	inline void
	Init(ConfigType const * p_Config) {
		_CLC.bits.DISR		= 0;		// Enable clock for Multican module
		
		// we must wait until clock has been enabled for the multican module
		for(UInt32 i = 0; (i < cfHW_DEV_CAN_MULTICAN_TIMEOUT_DURATION_FACTOR)
						  && (_CLC.bits.DISS != 0) ; ++i) { }

		_FDR.bits.STEP		= cfHW_DEV_CAN_MULTICAN_FDR_STEP;
		_FDR.bits.DM		= 1;		// we always use normal divider mode
	}

private:
	// Memory map for ClockManager
	CAN_CLC_t		_CLC;		//!< 0x0000 Can Clock Control Register
	uint32			_dummy0[2];		//!< 0x0004 Unused memory map
	CAN_FDR_t		_FDR;			//!< 0x000C Fractional Divider Register
}; // class hw::dev::can::multican::ClockManager

}}}}

#endif /* HW_DEV_CAN_MULTICAN_CLOCK_MANAGER_H */
