/** \author Christian Meier
 *  \version $Id: HwObject.h 995 2009-08-11 14:26:31Z mike $
 */

#ifndef HW_DEV_CAN_MULTICAN_HW_OBJECT_H
#define HW_DEV_CAN_MULTICAN_HW_OBJECT_H

#include "os/com/ComStack_Types.h"
#include "hw/dev/can/ConfigTypes.h"
#include "hw/dev/can/HwObjectIdType.h"

#include "RegTypes.h"
#include "Addr.h"

// Errata: MSGVAL, TXRQ TXEN0 TXEN1 stuff of: MultiCAN_TC.038, MultiCAN_TC.037
// 		   We should only use at most 127 of the 128 HwObjects: MultiCAN_AI.041

namespace hw { namespace dev { namespace can { namespace multican {

class HwObject {
public:
	static HwObject&
	Inst(HwObjectIdType Index) {
		return ((HwObject *)
					(HW_DEV_CAN_MULTICAN_MODULE_BASE_ADDRESS +
					HW_DEV_CAN_MULTICAN_HWOBJECTS_BASE_OFFSET))
				[Index];
	}

	static void
	InitAll(HwObjectConfigType const * HwObjectConfigs,
			     HwObjectIdType HwObjectMultiplicity) {
		HwObjectIdType i = 0;

		// We assume HwObjectMultiplicity <= 127
		for(; i < HwObjectMultiplicity; ++i) {
			Inst(i).Init(&HwObjectConfigs[i]);
		}
	}

	HwObjectIdType
	Id() const {
		return this - ((HwObject const *)
				(HW_DEV_CAN_MULTICAN_MODULE_BASE_ADDRESS +
				HW_DEV_CAN_MULTICAN_HWOBJECTS_BASE_OFFSET));
	}

	// This function initializes a HwObject to serve our HwObject scheme
	void
	Init(HwObjectConfigType const * p_Config);

	inline uint8
	GetNextHwObjHndl(void) const {
		return _MOSTAT.bits.PNEXT;
	}

	inline uint8
	GetPreviousHwObjHndl(void) const {
		return _MOSTAT.bits.PPREV;
	}

	inline uint8
	GetAssociatedListHndl(void) const {
		return _MOSTAT.bits.LIST;
	}

	inline uint8
	GetOwnHwObjHndl(void) const {
		// We have successfully "abused" the MPN field for the HwObject handle
		// This is correct, because we also use this scheme for the pending interrupts register notification
		return _MOIPR.bits.MPN;
	}

private:
	CAN_MOFCRm_t	_MOFCR;			//!< Function Control Register
	CAN_MOFGPRm_t	_MOFGPR;		//!< FIFO/ Gateway Pointer Register
	CAN_MOIPRm_t	_MOIPR;			//!< Interrupt Pointer Register
	CAN_MOAMRm_t	_MOAMR;			//!< Acceptance Mask Register
	CAN_MODATALm_t	_MODATAL;		//!< Data Register low
	CAN_MODATAHm_t	_MODATAH;		//!< Data Register high
	CAN_MOARm_t		_MOAR;			//!< Arbitration Register
	union {
		CAN_MOCTRm_new_t	_MOCTR;	//!< Write only Control Register
		CAN_MOSTATm_t	_MOSTAT;	//!< Read only Status Register
	};
};

}}}}

#endif /* HW_DEV_CAN_MULTICAN_HW_OBJECT_H */
