/** \author Christian Meier
 *  \version $Id: PendingHwObjManager.h 995 2009-08-11 14:26:31Z mike $
 */

#ifndef HW_DEV_CAN_MULTICAN_PENDING_HW_OBJ_MANAGER_H
#define HW_DEV_CAN_MULTICAN_PENDING_HW_OBJ_MANAGER_H

#include "os/com/ComStack_Types.h"
#include "hw/dev/can/ConfigTypes.h"
#include "hw/dev/can/HwObjectIdType.h"

#include "Cfg.h"
#include "RegTypes.h"
#include "Addr.h"

#define HW_DEV_CAN_MULTICAN_MASKS_NUM_ARRAY_ELEMS ((cfHW_DEV_CAN_MULTICAN_NUM_HW_OBJECTS + 31) / 32)

namespace hw { namespace dev { namespace can { namespace multican {

class PendingHwObjManager {
public:
	static PendingHwObjManager & Inst() {
		return * (PendingHwObjManager *)
					( HW_DEV_CAN_MULTICAN_MODULE_BASE_ADDRESS
					+ HW_DEV_CAN_MULTICAN_PENDING_HW_OBJECT_MANAGER_OFFSET );
	}

	class MaskType {
	friend class PendingHwObjManager;

	public:
		inline MaskType(void) {
			for(int i = 0; i < HW_DEV_CAN_MULTICAN_MASKS_NUM_ARRAY_ELEMS; ++i) {
				m_Val[i] = 0;
			}
		}

		inline void MarkHwObj(uint8 HwObjHndle) {
			uint8 ArrayIndex = HwObjHndle / 32;
			uint8 BitIndex   = HwObjHndle - (ArrayIndex * 32);

			m_Val[ArrayIndex] |= (1 << BitIndex);
		}

  private:
		uint32 m_Val[HW_DEV_CAN_MULTICAN_MASKS_NUM_ARRAY_ELEMS];
	};

	void
	Init(ConfigType const * p_Config);

	bool GetNextHwObjHndl(const MaskType * p_Mask, HwObjectIdType * p_HwObjHndl) {
		uint32 Index;
		for(int i = 0; i < HW_DEV_CAN_MULTICAN_MASKS_NUM_ARRAY_ELEMS; ++i) {
			_MSIMASK.bits.IM = p_Mask->m_Val[i];
			Index = _MSID[i].bits.INDEX;
			if(Index < 32) {
				*p_HwObjHndl = i * 32 + Index;
				return true;
			}
		}
		return false;
	}

	void AcknowledgeHwObj(HwObjectIdType HwObjHndl) {
		uint8 ArrayIndex = HwObjHndl / 32;
		uint8 BitIndex   = HwObjHndl - (ArrayIndex * 32);

		_MSPND[ArrayIndex].reg = ~ (1 << BitIndex); // Writing 0 clears, writing 1 has no effect
	}

private:
	CAN_MSPNDm_t	_MSPND[8];		//!< 0x0000 Message Pending Registers
	CAN_MSIDm_t		_MSID[8];		//!< 0x0020 Message Index Registers
	uint32			_dummy0[24];	//!< 0x0040 Outside responsibility
	CAN_MSIMASK_t	_MSIMASK;		//!< 0x00A0 Message Index Mask Register
	uint32			_dummy1[1];		//!< 0x00A4 Outside responsibility
	CAN_MCR_t		_MCR;			//!< 0x00A8 Module Control Register
};

}}}}

#endif /* HE_DEV_CAN_MULTICAN_PENDING_HW_OBJ_MANAGER_H */
