// $Id: SystemClock.h 995 2009-08-11 14:26:31Z mike $
// Created:   02.06.2006 lohmann

/**
 * @file SystemClock.h
 * @brief Contains the implementation of hw::hal::SystemTimer
 * 
 */


#ifndef __HW_HAL_SYSTEMCLOCK_H__
#define __HW_HAL_SYSTEMCLOCK_H__

#include "SystemClockImpl.h"


namespace hw { namespace hal {

/**
 * @brief Implements the hw::hal::SystemTimer on top the configured hw::hal::SystemClockImpl
 * 
 */
class SystemClock : public SystemClockImpl {
	static SystemClock theSystemClock_;
	//! Performs start-time initializations. Only called by advice.
	void init();
  protected:
    SystemClock(){}
  public:	
	//! Returns the one-and-only SystemClock instance
	static SystemClock& Inst() {
		return theSystemClock_;
	}

	//! Returns the number of days the system has been running.
	UInt16 days() const {
		return value() / (24 * 3600 * freq());
	}
	//! Returns the hour part (0-23) of the current day.
	UInt8 hour() const {
		return (value() / (3600 * freq())) % 24;
	}
	//! Returns the minute part (0-59) of the current hour.
	UInt8 min() const {
		return (value() / (60 * freq())) % 60;
	}
	//! Returns the second part (0-59) of the current minute.
	UInt8 sec() const {
		return (value() / freq()) % 60;
	}
	//! Returns the millisecond part (0-59) of the current second.
	UInt16 msec() const {
		return (value() / ( freq() / 1000) ) % 1000;
	}
};



}} // namespace hw::hal

#endif // __HW_HAL_SYSTEMCLOCK_H__
