// $Id: STMTimer0.cpp 1276 2010-09-14 10:27:54Z mike $
// Created:   22.05.2006 lohmann

/**
 * @file STMTimer.cpp
 * @brief Implementation of the TC1796 STMTimer0 and STMTimer1 classes
 */
 
#include "STMTimer0.h"
 
namespace hw { namespace hal {

using namespace hw::dev;

STMTimer0 STMTimer0::theSTMTimer0_;

void STMTimer0::init() {
	// nothing to do...

	// STM::init() already did everything
}

void STMTimer0::windupPeriodical( Unit period ) {
	STM::CMCON().bits.MSIZE0  = 31;
	STM::CMCON().bits.MSTART0 = 0;

	CPU::IEState ie;
	CPU::disable( ie );

	Unit snapshot = hw::dev::STM::TIM<0>();
	hw::dev::STM::CMP0() = snapshot + period;
	period_ = period;

	hw::dev::STM::ICR().bits.CMP0EN = 1;    // enable timer

	IRQ::enable();
	CPU::enable( ie );
}

void STMTimer0::windupAndStartSingle( Unit period, Unit decisionBase ) {
	STM::CMCON().bits.MSIZE0  = 31;
	STM::CMCON().bits.MSTART0 = 0;
	period_                   = 0;

	// windup and start
	CPU::IEState ie;
	CPU::disable( ie );

	//Unit snapshot = hw::dev::STM::TIM<0>();
	hw::dev::STM::CMP0() = /*snapshot +*/ period;

	hw::dev::STM::ICR().bits.CMP0EN = 1;    // enable timer

	// If period is very small, we cannot be sure that
	// irq has been triggered automatically.
	// In this case we manually trigger it.
	// This case also works flawlessly under overflow conditions.
	//if ((hw::dev::STM::TIM<0>() - snapshot) >= period) {
	if( period <= hw::dev::STM::TIM<0>() || hw::dev::STM::TIM<0>() < decisionBase ) {
		set ();
		IRQ::set();
	}

	IRQ::enable();
	CPU::enable( ie );
}

void STMTimer0::start() {
	CPU::IEState ie;
	CPU::disable( ie );
	hw::dev::STM::CMP0() = hw::dev::STM::TIM<0>() + period_;
	hw::dev::STM::ICR().bits.CMP0EN = 1; // enable timer
	IRQ::enable();
	CPU::enable( ie );
}
}} // namespace hw::hal

