/*
 * PageTable.h
 *
 *  Created on: Feb 16, 2010
 *      Author: bossel00
 */

#ifndef PAGETABLE_H_
#define PAGETABLE_H_

#include "stdinc.h"

namespace hw {
namespace hal {

template<typename PtrType, typename EntryType = UInt64> class BasicPageTable;

class PageTable {
public:
	enum Pagesize {
		PS_4k   = 4096,
		PS_2m   =   2 * 1024 * 1024,
		PS_4m   =   4 * 1024 * 1024,
		PS_1g   =   1 * 1024 * 1024 * 1024,
		PS_512g = 512 * 1024 * 1024 * 1024     // "virtual" 512GB pagesize for clearPage()
	};

private:

public:
	// to be adviced
	void init() {}

	// to be adviced
	bool setPage(void* virtual_addr, void* physical_addr, Pagesize ps, bool writeable = true, bool user = true, bool executable = true) {
		return true;
	}

	// to be adviced
	void clearPage(void* virtual_addr, Pagesize ps) {}

	// TODO: invalidatePage nach hw::hal::Paging verschieben,
	//       da das Verändern von zur Zeit nicht benutzten Seitentabellen natürlich kein invlpg braucht!
	void invalidatePage(const void* virtual_addr) {
		asm volatile("invlpg %0;" : : "m"(*static_cast<const unsigned*>(virtual_addr)));
	}

	// returns the address of the first pagetable structure (PML4E, PDPTE or PDE)
	// can be used for initialisation of CR3
	// to be adviced
	void* getFirstPagingStructure() { return 0; }
};

// class BasicPageTable
// template for a PDE, PTE, PDPE, PML4E, etc.
template<typename PtrType, typename EntryType>
class BasicPageTable {
	EntryType entry_;

	enum {
		PRESENT = 0,
		WRITEABLE = 1,
		USER = 2,
		PAGESIZE = 7,
		NX = 63
	};

public:

	void init(PtrType next_table, bool present, bool writeable, bool user, bool pagesize, bool nx = false) {
		// TODO: Compiler-Warnung vermeiden? (Im 32-Bit Modus wird nx links rausgeschoben, was aber so gewollt ist!)
		entry_ = ((EntryType) next_table & 0x000FFFFFFFFFF000ULL) | (static_cast<EntryType>(present) << PRESENT)
				                                                | (static_cast<EntryType>(writeable) << WRITEABLE)
				                                                | (static_cast<EntryType>(user) << USER)
				                                                | (static_cast<EntryType>(pagesize) << PAGESIZE)
				                                                | (static_cast<EntryType>(nx) << NX);
	}

	BasicPageTable() {}

	BasicPageTable(PtrType next_table, bool present, bool writeable, bool user, bool pagesize, bool nx = false) {
		init(next_table, present, writeable, user, pagesize, nx);
	}

	PtrType getPtr() { return (PtrType) (entry_ & 0x000FFFFFFFFFF000ULL); }
	bool  isPresent() { return entry_ & (1ULL << PRESENT); }
	bool  isWriteable() { return entry_ & (1ULL << WRITEABLE); }
	bool  isUser() { return entry_ & (1ULL << USER); }
	bool  isPagesize() { return entry_ & (1ULL << PAGESIZE); }
	bool  isNX() { return entry_ & (1ULL << NX); }

	void  setPtr(PtrType addr) {
		entry_ &= 0xFFF0000000000FFFULL;
		entry_ |= ((unsigned long) addr) & 0x000FFFFFFFFFF000ULL;
	}

	void  setPresent(bool val)   { if (val) {entry_ |= (1ULL<<PRESENT);}   else {entry_ &= ~(1ULL<<PRESENT);} }
	void  setWriteable(bool val) { if (val) {entry_ |= (1ULL<<WRITEABLE);} else {entry_ &= ~(1ULL<<WRITEABLE);} }
	void  setUser(bool val)      { if (val) {entry_ |= (1ULL<<USER);}      else {entry_ &= ~(1ULL<<USER);} }
	void  setPagesize(bool val)  { if (val) {entry_ |= (1ULL<<PAGESIZE);}  else {entry_ &= ~(1ULL<<PAGESIZE);} }
	void  setNX(bool val)        { if (val) {entry_ |= (1ULL<<NX);}        else {entry_ &= ~(1ULL<<NX);} }

	EntryType getEntry() { return entry_; }
};

}
}

#endif /* PAGETABLE_H_ */
