#ifndef __HW_HAL_CPU_H__
#define __HW_HAL_CPU_H__

#include "stdinc.h"

#include "hw/dev/PIC.h"

namespace hw { namespace hal {

CIAO_STD class CPU {

  static bool insideAST; // represents IRQL 2

  public:


	//! Type to store an IE (interrupts enabled) state.
 	//CIAO_STD typedef ::hw::dev::PIC_IRQ IEState;
	  CIAO_STD typedef bool IEState;

	//! Returns status of the IE (interrupt enable) flag.
	CIAO_STD inline static bool IE(){
		return ::hw::dev::PIC::Inst().getIRQMask().mask;
	}

	//! Disable IRQs globally.
	CIAO_STD inline static void disable() {
		asm volatile("cli");
	}

	//! Disable IRQs globally, returns the pervious IE state in /a oldIE.
	CIAO_STD inline static void disable( IEState& oldIE ) {
		//oldIE = ::hw::dev::PIC::Inst().getIRQMask();
		oldIE = getIRQL();
		disable();
	}

	//! enable IRQs globally
	CIAO_STD inline static void enable() {
		asm volatile("sti");
	}

	//! Restores the IE state passed in /a oldIE.
	CIAO_STD inline static void enable( IEState oldIE ) {
		//::hw::dev::PIC::Inst().setIRQMask(oldIE);
		setIRQL(oldIE);
		//if (oldIE.mask) enable();
	}

	//! returns current CPU IRQL
	CIAO_EXT inline static UInt8 getIRQL() {

		if(insideAST == true) 
			return 2;

		unsigned long val;

#if defined(__x86_64__)
		asm volatile("pushf\n"
				     "popq %0\n"
				     "andq $0x200, %0\n"
					: "=a"(val)
					:
					: "memory");
#else
		asm volatile("pushf\n"
				     "popl %0\n"
				     "andl $0x200, %0\n"
					: "=a"(val)
					:
					: "memory");
#endif

		return val ? 0 : 1;
	}

	//! returns IRQL of a pending IRQ (PIRQ < IRQL) or 0
	CIAO_EXT inline static UInt8 getPIRQL() {
		//return getICR().bits.PIPN;

		// TODO: Was tun?
		return 0;
	}

	//! returns IRQL of the preceeding context
	CIAO_EXT inline static UInt8 getPreviousIRQL() {
		//hw::_tc::PCXI_t_nonv myPCXI;
		//myPCXI.reg = _mfcr( $pcxi );
		//return myPCXI.bits.PCPN;

		// TODO: Was tun?
		return 1;
	}

	CIAO_EXT inline static void setAST(bool ast) {
		insideAST = ast;
	}

	//! modifies the current CPU IRQL
	CIAO_EXT inline static void setIRQL( UInt8 irql) {
		if (irql)
			disable();
		else
			enable();
	}

};

} } // namespace hw::hal

#endif // __HW_HAL_CPU_H__
