/**
 * \file InputDeviceKeyboard.h
 * \brief Implements a low-level interface for an AT-Keyboard
 *
 */

#ifndef __HW_HAL_INPUTDEVICEKEYBOARD__
#define __HW_HAL_INPUTDEVICEKEYBOARD__

#include "util/ringbuffer.h"

namespace hw { namespace hal {

class InputDeviceKeyboard {
public:
	static const int  DEFAULT_NEWLINE_SEQUENCE_LENGTH = 1;
	static const char DEFAULT_NEWLINE_SEQUENCE[DEFAULT_NEWLINE_SEQUENCE_LENGTH];

private:
	static InputDeviceKeyboard inst_;

	static const int BUFFERSIZE = 16;
	Ringbuffer buffer_;

	bool echo_;

  public:
	  void init() { echo_ = true; buffer_.reset(); }
	  void addCharacter(int character) { buffer_.put(character); }

  public:  // Userinterface
  	static InputDeviceKeyboard& Inst() {
  		return inst_;
  	}

  	// enables/disables echo
  	// TODO: Implementieren
  	void echo(bool on) { echo_ = on; }

  	// returns the next character without removing it from buffer
  	int peekCharacter();

  	// returns the next character
  	int getCharacter();

  	// returns # of characters in buffer
  	int charactersAvailable() const;

};

}} // namespace hw::hal



#endif // __HW_HAL_INPUTDEVICEKEYBOARD__
