/*
 * InputDeviceSTDIN.cpp
 *
 *  Created on: 30.03.2010
 *      Author: bossel00
 */

#include "InputDeviceSTDIN.h"

// TODO: Funktionen aus termios.h und unistd.h unter hw::guest kapseln
#include <termios.h>
#include <unistd.h>

#include "hw/guest/syscall.h"

namespace hw {
namespace hal {

const char InputDeviceSTDIN::DEFAULT_NEWLINE_SEQUENCE[InputDeviceSTDIN::DEFAULT_NEWLINE_SEQUENCE_LENGTH] = { '\n' };

InputDeviceSTDIN InputDeviceSTDIN::inst_;

void InputDeviceSTDIN::init() {
	termios param;
	tcgetattr(STDIN_FILENO, &param);
	param.c_lflag &= ~(ICANON | ECHO);       // set non-canonical mode
	tcsetattr(STDIN_FILENO, TCSANOW, &param);

	echo(true);

	buffer_ok = false;
}

void InputDeviceSTDIN::fini() {
	termios param;
	tcgetattr(STDIN_FILENO, &param);
	param.c_lflag |= ICANON | ECHO; // return to canonical mode
	tcsetattr(STDIN_FILENO, TCSANOW, &param);
}

// returns the next character without removing it from buffer
int InputDeviceSTDIN::peekCharacter() {
	if (buffer_ok) {
		return buffer;
	}
	else {
		::read(STDIN_FILENO, &buffer, 1);
		buffer_ok = true;
		return buffer;
	}
}

// returns the next character
int InputDeviceSTDIN::getCharacter() {
	if (buffer_ok) {
		buffer_ok = false;
		return buffer;
	}
	else {
		::read(STDIN_FILENO, &buffer, 1);
		return buffer;
	}
}

// returns # of characters in buffer
int InputDeviceSTDIN::charactersAvailable() const {
	// TODO: Was tun?
	return buffer_ok;
}

void InputDeviceSTDIN::echo(bool on) {
	termios param;
	tcgetattr(STDIN_FILENO, &param);

	if (on) {
		param.c_lflag |= ECHO;       // set non-canonical mode
	}
	else {
		param.c_lflag &= ~(ECHO);       // set non-canonical mode
	}


	tcsetattr(STDIN_FILENO, TCSANOW, &param);
}

}
}
