/**
 * \file OutputDeviceSTDOUT.h
 * \brief Implements a low-level interface to stdout on Linux guest
 *
 */

#ifndef __HW_HAL_OUTPUTDEVICESTDOUT__
#define __HW_HAL_OUTPUTDEVICESTDOUT__

#include "stdinc.h"

namespace hw { namespace hal {

class OutputDeviceSTDOUT {
  public:
	// ANSI Escape Codes
	struct Attribute {
		UInt16 foreground : 4;
		UInt16 background : 4;
		UInt16 intensity : 1;
		UInt16 cursor : 1;

		Attribute(UInt8 f = 8, UInt8 b = 8, bool i = 0, bool cursor = 1) : foreground(f), background(b), intensity(i), cursor(i) {}

		Attribute(int attribute) : foreground(attribute & 15), background((attribute >> 4) & 15),
									intensity((attribute >> 8) & 1), cursor(attribute >> 9) {}

		operator int() {
			return foreground | (background << 4) | (intensity << 8) | (cursor << 9);
		}
	};

	static const int DEFAULT_ATTRIBUTE = 0x288;  // default foreground and background with cursor

  private:
	void writeStdout(const void* data, unsigned long size);
	void sendAttributeEscapeCodes();
	static OutputDeviceSTDOUT theOutputDevice_;

	Attribute attribute_;

  public:

	void init() {
		resetAttribute();
		//clearScreen();
	}

  	static OutputDeviceSTDOUT& Inst() {
  		return theOutputDevice_;
  	}

  	void clearScreen();

  	void setCursorPosition(int x, int y);

  	void putCharacter(char c);
  	void putString(const char* str, unsigned long size);

  	void setAttribute(const Attribute& attribute);

  	void resetAttribute(); // reset to default

  private:
	OutputDeviceSTDOUT() {}
};

}} // namespace hw::hal



#endif // __HW_HAL_OUTPUTDEVICESTDOUT__
