/*
 * Paging.h
 *
 *  Created on: 27.11.2009
 *      Author: bossel00
 */

#ifndef PAGING_H_
#define PAGING_H_

#include "PageTable.h"

//#include "util/mempool.h"

namespace hw {

extern "C" void paging_set_pagetable(void* pagetable);
extern "C" void enable_paging(void* pagetable);
extern "C" void disable_paging();
extern "C" void enable_pae();
extern "C" void disable_pae();

namespace hal {

inline void paging_set_pagetable(void* pagetable) { hw::paging_set_pagetable(pagetable); }
inline void enable_paging(void* pagetable)        { hw::enable_paging(pagetable); }
inline void disable_paging()                      { hw::disable_paging(); }
inline void enable_pae()                          { hw::enable_pae(); }
inline void disable_pae()                         { hw::disable_pae(); }

class Paging {
public:
	//typedef hw::hal::PageTable::Pagesize Pagesize;

private:
	static Paging inst_;
	//paging::LM_2MB::PML4E* pml4_;

	//util::Mempool<4096, 1024> pagepool_;

	PageTable pagetable_;

public:
	static Paging& Inst() { return inst_; }

	void init();

	void enable();
	void disable();  // TODO: Testen

	void switchto_system_pagetable() { paging_set_pagetable(pagetable_.getFirstPagingStructure()); }

	//paging::LM_2MB::PML4E* getPML4() { return pml4_; }

	//! new pagetable entry: virtual_addr -> physical_addr
	bool setPage(void* virtual_addr, void* physical_addr, PageTable::Pagesize ps, bool writeable = true, bool user = true, bool executable = true);

	bool setPage(unsigned long virtual_addr, unsigned long physical_addr, PageTable::Pagesize ps, bool writeable = true, bool user = true, bool executable = true) {
		return setPage((void*)virtual_addr, (void*)physical_addr, ps, writeable, user, executable);
	}

	bool setPageDirect(void* address, PageTable::Pagesize ps, bool writeable = true, bool user = true, bool executable = true) {
		//unsigned long base = ((unsigned long) address) & ~0x1FFFFFUL;
		return setPage(address, address, ps, writeable, user, executable);
	}

	bool setPageDirect(unsigned long address, PageTable::Pagesize ps, bool writeable = true, bool user = true, bool executable = true) {
		return setPageDirect((void*) address, ps, writeable, user, executable);
	}

	void clearPage(void* virtual_addr, PageTable::Pagesize ps);

	void invalidatePage(const void* virtual_addr) {
		asm volatile("invlpg %0;" : : "m"(*static_cast<const unsigned*>(virtual_addr)));
	}
};

}
}

#endif /* PAGING_H_ */
