#ifndef HW_HAL_CAN_TX_LPDU_TYPE_H
#define HW_HAL_CAN_TX_LPDU_TYPE_H

#include "os/com/ComStack_Types.h"
#include "hw/hal/can/Cfg.h"

#include "TxLpduIdType.h"
#include "TxLpduConfigType.h"

namespace hw { namespace hal { namespace can {

class TxLpduType : private TxLpduConfigType {
public:
	inline void
	Init(TxLpduConfigType const * TxLpduConfig) {
		*(TxLpduConfigType *)this = *TxLpduConfig;
	}

	inline TxLpduIdType
	Id() const {
		return this - TxLpdus_;
	}

	StatusType
	Transmit(const os::com::PduInfoType * PduInfoPtr) {
		hw::dev::can::PduType pdu;
		pdu.Sdu_		= PduInfoPtr->SduDataPtr_;
		pdu.Id_			= CanId_;
		pdu.SwPduHandle_ = Id();
		pdu.Length_		= PduInfoPtr->SduLength_;

		hw::dev::can::ReturnType res = Write_Delegator(&pdu);

		if(res == hw::dev::can::CAN_OK) {
			return E_OK;
		} else {
			return E_NOT_OK;
		}
	}

private:
	hw::dev::can::ReturnType
	Write_Delegator(hw::dev::can::PduType const * Pdu);

public:
	static void
	InitAll(TxLpduConfigType const * ConfigArray, uint8 TxLpduConfigMultiplicity) {
		TxLpduIdType i = 0;

		// We assume ControllerMultiplicity <= cfHW_HAL_CAN_CONTROLLER_MULTIPLICITY
		for(; i < TxLpduConfigMultiplicity; ++i) {
			TxLpdus_[i].Init(&ConfigArray[i]);
		}

		TxLpduMultiplicity_ = i;

		for(; i < cfHW_HAL_CAN_TX_LPDU_MULTIPLICITY; ++i) {
			// FIXME Deinit or similar
		}
	}

	static TxLpduType &
	Inst(TxLpduIdType TxLpduId) {
		return TxLpdus_[TxLpduId];
	}

	static TxLpduIdType
	Multiplicity() {
		return TxLpduMultiplicity_;
	}

private:
	static TxLpduType TxLpdus_[cfHW_HAL_CAN_TX_LPDU_MULTIPLICITY];
	static TxLpduIdStorageType TxLpduMultiplicity_;
};

}}}

#endif

