// created: 08.01.2008 sirupfis

#include "PCP.h"

namespace hw { namespace irq {

void PCP::init() {

// PCP_CS register
//
// EN			0	rw 	PCP Enable
//							0 PCP is disabled for operation (default)
//							1 PCP is enabled for operation
// RST 		1 	rwh 	PCP Reset Request
//							0 No PCP software reset operation is requested
//							1 A PCP software reset is requested. Halt any
//							operating channel. Reset all control registers to
//							default values. Reset PCP state to default value.
//							RST is always read as 0, but is written with 1 in order
//							to initiate a reset.
//	RS 		2 	rh 	PCP Run/Stop Status Flag
//							0 PCP is stopped or idle (default)
//							1 PCP is currently running
//	0 			3 	r 		Reserved; read as 0; should be written with 0.
//	RCB 		4 	rw 	Channel Start Mode Control
//							0 Channel resume operation mode selected;
//							channel start PC is taken from restored context
//							1 Channel restart operation mode selected;
//							channel start PC is derived from the requested
//							channel number (= priority number of service request)
//	EIE 		5 	rw 	Endinit Enable
//							0 Writes to PCP_CS are disabled if Endinitprotection
//							is enabled.
//							1 Writes to PCP_CS are enabled if Endinitprotection
//							is enabled.
//	CS 	[7:6] rw 	Context Size Selection
//							00B Use Full Context for all channels
//							01B Use Small Context for all channels
//							10B Use Minimum Context for all channels
//							11B Reserved
//	PPE 	8 		rw 	PRAM Partitioning Enable
//							0 PRAM is not partitioned
//							1 PRAM is partitioned
//	PPS 	[15:9] rw 	PRAM Partition Size
//							0 Default, only allowed with PPE = 0
//							1 CSA contains 3 context save regions
//							…
//							n CSA contains 1 + 2 × n context save regions
//	CWE 	16 	rw 	Channel Watchdog Enable
//							0 Disable Channel Watchdog
//							1 Enable Channel Watchdog
//	CWT 	[23:17] rw 	Channel Watchdog Threshold
//							0 Reserved
//							1 Threshold = 16 instructions
//							…
//							n Threshold = 16 × ‘n’ instructions
//	ESR 	[31:24] rw 	Error Service Request Number
//							SRPN for interrupt to CPU on an error condition.
//							00H No interrupt request posted (default)
//							value n Post n as SRNP interrupt to CPU on an error
//							condition (n not equal 00H)

//   PCP_CS = 0x02000000; // PCP disabled, Channel resume operation mode selected
   							// on pcp error jump to handler priority 2
   							
   PCP_CS = 0x02000001; /* PCP enabled, Channel restart operation mode selected */
   							

// PCP_ICR register
//
//	CPPN 	[7:0] rh		Current PCP Priority Number
//							This field indicates the current priority level of the PCP
//							and is automatically updated by hardware on entry into
//							an interrupt service routine.
//	IE 	8 		rh 	Reserved
//	PIPN	[23:16] rh 	Pending Interrupt Priority Number
//							This read-only field is updated by the PICU at the end
//							of each arbitration process, and indicates the priority
//							number of a pending request. PIPN is set to 00H when
//							no request is pending and at the beginning of a new
//							arbitration process.
//	PARBCYC [25:24] rw Number of Arbitration Cycles Control
//							This bit field controls the number of arbitration cycles
//							used to determine the request with the highest priority.
//							It follows the same coding scheme as described for the
//							CPU interrupt arbitration.
//							00B Four arbitration cycles (default)
//							01B Three arbitration cycles
//							10B Two arbitration cycles
//							11B One arbitration cycle
//	PONECYC 26 	rw 	Clocks per Arbitration Cycle Control
//							This bit determines the number of clocks per arbitration
//							cycle.
//							0 Two clocks per arbitration cycle (default)
//							1 One clock per arbitration cycle
//	0 	[15:9],	r 		Reserved; read as 0; should be written with 0.
//		[31:27]			

   PCP_ICR = 0x00000000; // 4 arb. cycles (default), 2 clocks per arbitration 

// PCP_SRCn (n = 4-8) register
//
//	SRPN 	[7:0] rh 	PCP Node n Service Request Priority Number
//							This number is automatically set by the PCP if it
//							needs to place a service request on interrupt bus 0
//							(CPU interrupt arbitration bus) or 1 (PCP interrupt
//							arbitration bus). Default after reset is 00H.
//	TOS 	[11:10] rw 	PCP Node n Type-of-Service State/Control
//							TOS value depends on the interrupt mapping that
//							has been selected (see Page 11-113). This bit field
//							must be written at PCP configuration time and
//							should not subsequently modified while the PCP is
//							operating.
//	SRE 	12 	r 		PCP Node n Service Request Enable
//							Always read as 1 (enabled).
//	SRR 	13 	rh 	PCP Node n Service Request Flag
//							0 No service requested (default).
//							1 Valid active service requested.
//	0 		[9:8],	r 	Reserved; read as 0; should be written with 0.
//			[15:14],
//			[31:16]

   PCP_SRC4 = 0x00000000;



}

void PCP::setPrio(os::krn::Task::Pri prio) {

	PCP_PRIO	= prio;

}

void PCP::setTaskInt(os::krn::Task::Pri prio, os::krn::Task::Pri task, os::krn::Task& pTask, os::krn::Task::Id No) {

	*((unsigned int volatile *)(PCP_TTASK_ADD + prio * 12)) = task;
	*((unsigned int volatile *)(PCP_TTASK_ADD + prio * 12 + 4)) = (unsigned int volatile) &pTask + 0x18400000;
	*((unsigned int volatile *)(PCP_TTASK_ADD + prio * 12 + 8)) = (unsigned int volatile) No;
}


}} // namespace hw::irq


