/** \author Christian Meier
 *  \version $Id: ComStack_Types.h 995 2009-08-11 14:26:31Z mike $
 *
 *  This file defines some commonly used types of the Communication Stack.
 */

#ifndef OS_COM_COMSTACK_TYPES_H
#define OS_COM_COMSTACK_TYPES_H

// Just a simple workaround for now
#include "stddef.h"

#include "ciaoConfig.h"
#include "util/types.h"
#include "as/autosar_types.h"

typedef UInt8 uint8;
typedef UInt16 uint16;
typedef UInt32 uint32;


namespace os {
	namespace com {

		typedef uint8 NetworkHandleType;

		// FIXME: cfOS_COM_PDU_ID_MAX (+ 1)
		typedef uint_t_value<cfOS_COM_PDU_ID_MAX>::T PduIdType;

		// FIXME: cfOS_COM_PDU_LENGTH_MAX (+ 1)
		typedef uint_t_value<cfOS_COM_PDU_LENGTH_MAX>::T PduLengthType;

		struct PduInfoType {
			uint8 * SduDataPtr_;
			PduLengthType SduLength_;
		};

		enum BufReq_ReturnType {
			BUFREQ_OK,
			BUFREQ_E_NOT_OK,
			BUFREQ_E_BUSY,
			BUFREQ_E_OVFL
		};

		enum NotifResultType {
			NTFRSLT_OK						= 0x00,
			NTFRSLT_E_NOT_OK				= 0x01,
			NTFRSLT_E_TIMEOUT_A				= 0x02,
			NTFRSLT_E_TIMEOUT_BS			= 0x03,
			NTFRSLT_E_TIMEOUT_CR			= 0x04,
			NTFRSLT_E_WRONG_SN				= 0x05,
			NTFRSLT_E_INVALID_FS			= 0x06,
			NTFRSLT_E_UNEXP_PDU				= 0x07,
			NTFRSLT_E_WFT_OVRN				= 0x08,
			NTFRSLT_E_NO_BUFFER				= 0x09,
			NTFRSLT_E_CANCELATION_OK		= 0x0A,
			NTFRSLT_E_CANCELATION_NOT_OK	= 0x0B
		};

		enum BusTrcvErrorType {
			BUSTRCV_OK						= 0x00,
			BUSTRCV_E_ERROR					= 0x01
		};
	};
};

namespace hw { namespace hal { namespace can {

enum DlcMode {
	DLC_UNBOUND = 255
};

enum ChannelModeType {
  OFFLINE = 0,
  ONLINE,
  TX_ONLINE,
  TX_OFFLINE_ACTIVE,
  TX_OFFLINE_ACTIVE_RX_ONLINE,
  RX_ONLINE
};

enum ChannelModeRequestType {
  SET_OFFLNE = 0,
  SET_ONLINE,
  SET_RX_OFFLINE,
  SET_RX_ONLINE,
  SET_TX_OFFLINE,
  SET_TX_ONLINE,
  SET_TX_OFFLINE_ACTIVE
};

enum ControllerModeType {
	CANIF_CS_UNINIT = 0,
	CANIF_CS_STOPPED,
	CANIF_CS_STARTED,
	CANIF_CS_SLEEP
};

enum NotifStatusType {
	CANIF_NO_NOTIFICATION = 0,
	CANIF_TX_RX_NOTIFICATION
};

}}}


namespace os { namespace com { namespace ascom {

typedef Bool (*IpduCalloutType) (PduIdType, UInt8 const *);

/*enum ReturnType {
	E_OK,
	COM_SERVICE_NOT_AVAILABLE,
	COM_TIMEOUT
};
*/

enum EndianessType {
	BIG			= -1,
	LITTLE	= 1
};

enum IsSignedType {
	UNSIGNED = 0,
	SIGNED	 = 1
};

enum TransferPropertyType {
	PENDING   = 0,
	TRIGGERED = 1
};

enum InterpretationType {
	OPAQUE	= 0,
	INTEGER	= 1
};

enum NatureType {
	SIGNAL = 0,
	SIGNAL_GROUP = 1,
	GROUP_SIGNAL = 2
};

enum ServiceIdType {
	// FIXME
};

enum Com_StatusType {
	COM_UNINIT = 0,
	COM_INIT
};

enum TransmissionMode {
	NONE = 0,
	DIRECT = 1,
	PERIODIC = 2,
	MIXED = 3
};

}}}


#endif /* OS_COM_COMSTACK_TYPES_H  */
