/** \author Christian Meier
 *  \version $Id: FilterAlgo.h 954 2009-03-24 13:40:21Z sichmeie $
 *
 *  This file defines all the filter internal algorithms that are based on templates to support
 *  all types of signals.
 */

#ifndef OS_COM_ASCOM_FILTER_ALGO_H
#define OS_COM_ASCOM_FILTER_ALGO_H

#include "os/com/ComStack_Types.h"

namespace os {namespace com { namespace ascom {

namespace filter {

template <typename ONE_EVERY_N_COUNTER_TYPE>
inline Bool
OneEveryN(ONE_EVERY_N_COUNTER_TYPE & Occurrence, ONE_EVERY_N_COUNTER_TYPE Period) {
	if(Occurrence == 0) {
		Occurrence = Period;
		return true;
	} else {
		--Occurrence;
		return false;
	}
}

/*
template <typename INDEX_TYPE>
Bool
MaskedNewDiffersMaskedOld(UInt8 const * NewValue, UInt8 const * OldValue, UInt8 const * Mask, INDEX_TYPE Length) {
	for(INDEX_TYPE i = 0; i < Length; ++i) {
		if((NewValue[i] & Mask[i]) != (OldValue[i] & Mask[i])) return true;
	}
	return false;
}
*/

template <typename T>
inline Bool
MaskedNewDiffersMaskedOld(T NewValue, T OldValue, T Mask) {
	return (NewValue & Mask) != (OldValue & Mask);
}

/*
template <typename INDEX_TYPE>
Bool
MaskedNewEqualsX(UInt8 const * NewValue, UInt8 * Mask, UInt8 * X, INDEX_TYPE Length) {
	for(INDEX_TYPE i = 0; i < Length; ++i) {
		if((NewValue[i] & Mask[i]) != X[i]) return false;
	}
	return true;
}
*/

template <typename T>
inline Bool
MaskedNewEqualsX(T NewValue, T X, T Mask) {
	return (NewValue & Mask) == X;
}

/*
template <typename INDEX_TYPE>
Bool
MaskedNewDiffersX(UInt8 const * NewValue, UInt8 * Mask, UInt8 * X, INDEX_TYPE Length) {
	for(INDEX_TYPE i = 0; i < Length; ++i) {
		if((NewValue[i] & Mask[i]) != X[i]) return true;
	}
	return false;
}
*/

template <typename T>
inline Bool
MaskedNewDiffersX(T NewValue, T X, T Mask) {
	return (NewValue & Mask) != X;
}

template <typename T>
inline Bool
NewIsWithin(T NewValue, T Min, T Max) {
	return (NewValue >= Min) && (NewValue <= Max);
}

template <typename T>
inline Bool
NewIsOutside(T NewValue, T Min, T Max) {
	return (NewValue < Min) || (NewValue > Max);
}

}

}}}

#endif
