#include "os/irq/IMutex.h"
#include "hw/hal/CPU.h"

#include "hw/hal/Continuation.h"
#include "stdio.h"

namespace os { namespace irq {

void IMutex::leave() {
	hw::hal::Continuation *running=hw::hal::Continuation::getActive();

	printf("%p leaves %p\r\n",running,this);	
	
	owner_=0; 
	
	hw::hal::CPU::disable();
	
	//TODO: 
	/* optimization?
	 * (master_ == this) if there is no master and
	 * 
	 * if (waiting_==running->master_)
	 * 	   running->master_=running;
	 *     running->saveAndSwitchto(waiting_);
	 * 
	 * -better only, if jz is cheaper then je
	 * -requires initialization of master*/
	 
	if (waiting_)	
		//CRITICAL: "waiting" value
		/* higher prioritized interrupt enters this monitor
		 * => waiting_ is set to 0
		 * => we are reactivated by our interruptor... */
		if (waiting_==running->master_)
			running->saveAndSwitchto(waiting_);	

	//TODO: "else"?
		hw::hal::CPU::enable();
}

void IMutex::enter() {
	hw::hal::Continuation *running=hw::hal::Continuation::getActive();
	
	printf("%p enters %p\r\n",running,this);
	
	hw::hal::CPU::disable();
		
	while (owner_) {
		waiting_=running;	
		owner_->master_=running;		
		//CRITICAL: "owner" value
		/* higher prioritized interrupt could try to enter this monitor
		 * => switch to owner
		 * => owner leaves, switches to the higher interrupt
		 * => interrupt re-enters, leaves, owner_ is set to 0
		 * => we are reactivated by our interruptor... */		 
		running->saveAndSwitchto(owner_);

		hw::hal::CPU::disable();
	}
	
	owner_=running;
	waiting_=0;

	printf("%p proceed %p\r\n",running,this);	
	
	hw::hal::CPU::enable();
}

//hw::hal::Continuation *IMutex::highest_;

}} //os::irq;
