// $Id: SchedImpl.cpp 1276 2010-09-14 10:27:54Z mike $
// Created:   27.06.2006 lohmann


#include "SchedImpl.h"

namespace os { namespace krn {

// the one and only instance
SchedImpl SchedImpl::theSchedImpl_;

// me: running -> waiting
void SchedImpl::block( Task::Id me ){
	setWaiting( me );
	Task::Id next 	= schedule( true );

	if( next != me ) {
		dispatch( me, next );	
	}
}

void SchedImpl::reschedule() {
	Task::Id me 	= getRunning();
	Task::Id next 	= schedule( false );
	
	if( me != next ) {
		setReady( me );
		dispatch( me, next );
	}	
}

void SchedImpl::yield() {
	Task::Id me 	= getRunning();
	Task::Id next 	= takeNext();
	
	if( me != next ) {
		setReady( me );
		dispatch( me, next );
	}	
}

void SchedImpl::superDispatch_impl( SchedImpl* obj, Task::Id me, Task::Id next ) {
	obj->setInterrupted( me, next==Task::INVALID ? 0 : 1 ); // clear/set interrupted state
	TCB* me_tcb = obj->getTCB( me );

	if (next == Task::INVALID) {
		// was exit () -> discard context
		obj->setSuspended( me );
		next = obj->getNextTaskAfterExit( me );
		obj->setRunning( next );
		if( ! obj->isInterrupted( next )) {
			me_tcb->start( obj->allocStack( next ), obj->allocTCB( next ) );
		} else {
			me_tcb->switchto( obj->getTCB( next ));
		}	
	} else {
		// was real dispatch () -> save context
		obj->setRunning( next );
		if( ! obj->isInterrupted( next )) {
			me_tcb->saveAndStart( obj->allocStack( next ), obj->allocTCB( next ) );
		} else {
			me_tcb->saveAndSwitchto( obj->getTCB( next ) );
		}
	}
}

Task::Id SchedImpl::getNextTaskAfterExit( Task::Id me ) {
	return schedule( true );
}

}} // namespace os::krn
