// $Id: SchedImpl.h 1205 2010-07-09 15:36:27Z sijesche $
// Created:   27.06.2006 lohmann

#ifndef __OS_SCHEDIMPL_TASKIDLE_H__
#define __OS_SCHEDIMPL_TASKIDLE_H__

#include "SchedStrategy.h"

namespace os { namespace krn {

class Application;

// class SchedImpl is the os-internal interface to the scheduler

class SchedImpl : public SchedStrategy {
  protected:
    static SchedImpl theSchedImpl_;

  	SchedImpl() {};
  public:
  	static SchedImpl& Inst() {
  		return theSchedImpl_;
  	}

	// whom: terminated -> ready
	void activate( Task::Id whom );

	// me: running -> waiting
	void block( Task::Id me );

	// whom: waiting -> ready
	void awake( Task::Id whom );

	// me: running -> terminated
	void exit( Task::Id me );

	// schedule task with higher priority
	void reschedule();

	// schedule task with higher or equal priority
	void yield();

	// start scheduling
	void start();

	void chain( Task::Id whom );

	void exitRunning();

	Task::State getTaskState( Task::Id whom );

  private:
  	void dispatch( Task::Id me, Task::Id next );
	static void superDispatch_impl( SchedImpl* obj, Task::Id me, Task::Id next );
	Task::Id getNextTaskAfterExit( Task::Id me );
};

// whom: terminated -> ready
inline void SchedImpl::activate( Task::Id whom ) {
	setActive( whom );
}

// whom: waiting -> ready
inline void SchedImpl::awake( Task::Id whom ){
	setReady( whom );
}

inline void SchedImpl::exitRunning() {
	exit( getRunning () );
}

inline Task::State SchedImpl::getTaskState( Task::Id whom ) {
	return Task::getState( whom );
}

inline void SchedImpl::start(){
	Task::Id first = schedule( true );
	setRunning( first );
	TCB* first_tcb = allocTCB( first );
	first_tcb->go( allocStack( first ) );
}

// me: running -> terminated
inline void CIAO_INLINE SchedImpl::exit( Task::Id me ) {
	hw::JUMP3( superDispatch_impl, this, me, Task::INVALID );
}

inline void CIAO_INLINE SchedImpl::chain( Task::Id whom ) {
	setActive( whom );
	exit( getRunning() );
}

inline void SchedImpl::dispatch( Task::Id me, Task::Id next ) {
	hw::JUMP3( superDispatch_impl, this, me, next );
}

}} // namespace os::krn



#endif // __OS_SCHEDIMPL_TASKIDLE_H__
