// $Id: Task.h 1149 2010-03-09 13:24:34Z mike $
// Created:   27.06.2006 lohmann

#ifndef __OS_TASK_H__
#define __OS_TASK_H__

#include "stdinc.h"
#include "util/types.h"
#include "ciaoConfig.h"

#define cfOS_THREADFUNC_ATTRIBUTES cfHAL_THREADFUNC_ATTRIBUTES

namespace os { namespace krn {

	struct Task {
		enum DOXY_SYMBOL(Consts) {
			PRI_MAX					= cfOS_MAX_PRIORITY,
			PER_PRI_MAX				= cfOS_MAX_NUMBER_OF_TASKS_PER_PRIORITY,
			MAX						= cfOS_NUMBER_OF_TASKS
		};

		typedef TaskType Id; // set to int so the pcp can handle this correctly
		static const Id INVALID = INVALID_TASK;

		typedef uint_t_value< PRI_MAX  >::T	Pri;
		typedef TaskStateType State;
		typedef TaskFunc Func;

		Pri		pri_;
		volatile State	state_;
		Func	func_;
		Ptr		stack_;
		Bool	interrupted_; // set if control flow is not yet finished

		static Task& Inst( Id me );

		static Pri getPri( Id of ) {
			return Inst( of ).pri_;
		}
		static void setPri( Id of, Pri val ) {
			Inst( of ).pri_ = val;
		}
		static State getState( Id of ) {
			return Inst( of ).state_;
		}
		static void setState( Id of, State val ) {
			Inst( of ).state_ = val;
		}
		static Func getFunc( Id of ) {
			return Inst( of ).func_;
		}
		static void setFunc( Id of, Func val ) {
			Inst( of ).func_ = val;
		}
		static Ptr getStack( Id of ) {
			return Inst( of ).stack_;
		}
		static void setStack( Id of, Ptr val ) {
			Inst( of ).stack_ = val;
		}
		static Bool getInterrupted( Id of ) {
			return Inst( of ).interrupted_;
		}
		static void setInterrupted( Id of, Bool val ) {
			Inst( of ).interrupted_ = val;
		}
	};


	extern Task theTasks [];

	inline Task& Task::Inst( Task::Id me ) {
		return theTasks[ me ];
	}

}} // namespace os::krn

#endif // __OS_TASK_H__
