#ifndef CONTEXT_H_
#define CONTEXT_H_

//#include "hw/hal/TCBUser.h"
#include "MPU.h"
#include "Context_Manifestation.ah"
#include "os/krn/Application.h"
#include "util/compiler.h"

//avoid include cycle
namespace hw { namespace hal {
	class TCBUser;
}}

namespace os { namespace mp {

/* The class os::mp::Context is an abstraction for those parts of the
 * current execution context, which are relevant for memory protection */

struct Context : public ContextAccess {
	
	typedef ContextAccess Super;

	// this function is called upon each context switch
	static void load() CIAO_NOINLINE {
		// TODO behaviour should be introduced by aspect
		if (!isTrustedApp())
			load_hook();	
	}

	static void load_hook() CIAO_NOINLINE {}

	/**
	 * Determines whether the execution is currently in trusted context.
	 * We do not check the OS object of the current application since
	 * this function may be called from an ISR, where the current application
	 * is not available. Instead, we directly evaluate the MPU state.
	 */
	static bool isTrustedMPU() {
		return os::mp::MPU::isTrusted();
	}

	static bool isTrustedApp() {
		return os::krn::Application::isTrusted(getApplication()); 
	}

	static os::krn::ApplicationType getApplication() {
		return Super::getInitialApplication();
	}
	
	static Ptr getStackUpper() {
		return Super::getInitialStackUpper();
	}
	
	/* to be adviced by aspects */
	static void setApplication(os::krn::ApplicationType app) {}

	/* to be adviced by aspects */
	static void setStackUpper(Ptr stack) {}
	
	static void exportStack_begin(Ptr &stack_old, Ptr stack) {
		// protect the stack used up to now from manipulation by the
		// non-trusted function  

		// save old upper stack bound
		stack_old = getStackUpper();
	
		/* save new upper stackbound into the context variable 
		 *    (could of course also be done right before a context switch,
		 *     in the save()-method) */
		setStackUpper( stack );

		// update the stack protection register
		os::mp::MPU::setStackUpper( stack );
	}
	 
	static void exportStack_end(Ptr stack_old) {
		// exportStack_begin^(-1)

		// first update the context variable
		// setStackUpper( stack_old );
		// -> not always necessary -> done by advice

		// then the stack protection register
		os::mp::MPU::setStackUpper( stack_old );
	}
};

}}

#endif /*CONTEXT_H_*/
