#include "TP_common_data.h"
#include "TP_common_queueelement.h"
#include "TP_common_definitions.h"
#include "TP_common_deadlinecause.h"
#include "hw/hal/Timer1.h"
#include "stdio.h"

namespace os { namespace tp {

TP_Deadlines TP_Deadlines::theTPDeadlines_;

void TP_Deadlines::Init(){
	queue_.Init();
}

void TP_Deadlines::dealWithDeadline(QueueElement e){
	/* timing protection aspects should bind here 
	 * use something like:
	 *	#include "TP_common_data.h" //for os::TP_Deadlines
	 *	#include "TP_common_queueelement.h" //for os::tp::QueueElement
	 *	pointcut DeadlineHandling(os::tp::QueueElement tmp) = 
	 *		execution ("void os::tp::TP_Deadlines::dealWithDeadline(os::tp::QueueElement)") && args(tmp);
	 *	advice DeadlineHandling(tmp) : before (os::tp::QueueElement tmp){
	 *		if (tmp.type == os::tp::<yourDeadlineTypeHere>){
	 *			...
	 *		}
	 *	}
	 */
}


QueueElement* TP_Deadlines::windupNextDeadline(){
	//deal with missed deadlines
	QueueElement* e = queue_.getFirst();
	QueueElement tmp;
	while ((e != NULL) && (hw::hal::Timer1::Inst().getTimer1() > e->time)){
		tmp = *e;
		Inst().removeDeadline(e);
		Inst().dealWithDeadline(tmp);

		e = queue_.getFirst();
	}

	if (e == NULL){ 
	} else { 
		hw::hal::Timer1::Inst().windupAndStart(e->time);
	}

	return queue_.getFirst();
}

QueueElement* TP_Deadlines::addDeadline(UInt32 when, DeadlineType what, DeadlineCause who){
	QueueElement* t = queue_.insert(when);
	if (t == NULL){
		return NULL;
	}

	t->type = what;
	t->who = who;

	return t;
}

void TP_Deadlines::removeDeadline(QueueElement* whom){
	queue_.remove(whom);
}

QueueElement* TP_Deadlines::getCurrentDeadline(){
	return queue_.getFirst();
}

}} //namespaces
