#ifndef BITMANIP_I386_H
#define BITMANIP_I386_H

#include "util/metaprog.h"
#include "util/types.h"

namespace bitmanip {

namespace impl {

template <typename INT_TYPE>
struct highest1BitAlgo<INT_TYPE, false, true> {
	static inline UInt
	Do (register INT_TYPE Value) {
		register UInt32 Tmp = Value;
		register UInt Res;
		asm ("bsr %1, %0\n\t" : "=r"(Res) : "r"(Tmp) );
		return Res;
	}
};

template <typename INT_TYPE>
struct highest0BitAlgo<INT_TYPE, false, true> {
	static inline UInt
	Do (register INT_TYPE Value) {
		return highest1BitAlgo<UInt32>::Do ((UInt32)(INT_TYPE)~Value);
	}
};

template <typename INT_TYPE>
struct lowest1BitAlgo<INT_TYPE, false, true> {
	static inline UInt
	Do (register INT_TYPE Value) {
		register UInt32 Tmp = Value;
		register UInt Res;
		asm ("bsf %1, %0\n\t" : "=r"(Res) : "r"(Tmp) );
		return Res;
	}
};

template <typename INT_TYPE>
struct lowest0BitAlgo<INT_TYPE, false, true> {
	static inline UInt
	Do (register INT_TYPE Value) {
		return lowest1BitAlgo<UInt32>::Do (~Value);
	}
};

template <typename INT_TYPE>
struct twosComplementAlgo<INT_TYPE, true> {
	static INT_TYPE
	Do (register INT_TYPE Value) {
		// i386 has two's complement arithmetic, so ...
		return -Value;
	}
};

/*
// This specializations create a per method 1 byte larger code than the general implementation :(
template<typename INT_TYPE>
struct testBitAlgo<INT_TYPE, false, true> {
	static inline Bool
	Do (register INT_TYPE Value, register UInt BitIndex) {
		register Bool Res = false;
		register UInt32 Tmp = Value;
		asm ("bt %2, %1\n\t"
				 "setb %b0 \n\t": "=a"(Res) : "r"(Tmp), "r"(BitIndex) );
		return Res;
	}
};
*/

template<typename INT_TYPE>
struct clearBitAlgo<INT_TYPE, false, true> {
	static inline INT_TYPE
	Do (register INT_TYPE Value, register UInt BitIndex) {
		register UInt32 Tmp = Value;
		asm ("btr %2, %0\n\t" : "=r"(Tmp) : "0"(Tmp), "r"(BitIndex) );
		return (INT_TYPE) Tmp;
	}
};

template<typname INT_TYPE>
struct setBitAlgo<INT_TYPE, false, true> {
	static inline INT_TYPE
	Do (register INT_TYPE Value, register UInt BitIndex) {
		register UInt32 Tmp = Value;
		asm("bts %2, %0\n\t" : "=r"(Tmp) : "0"(Tmp), "r"(BitIndex) );
		return (INT_TYPE) Tmp;
	}
};

} // namespace impl

} // namespace bitmanip

#endif
