#ifndef UTIL_BITSET_SINGLE_ELEM_IMPL_H
#define UTIL_BITSET_SINGLE_ELEM_IMPL_H

#include "util/metaprog.h"
#include "util/bitmanip.h"

#define BITSET_SINGLE_INLINE inline

template<class ELEM_TYPE>
class BitsetSingleElemImplBase {
private:
	typedef ELEM_TYPE ElemType;
	typedef BitsetSingleElemImplBase<ELEM_TYPE> BitsetType;
	enum { BITS_PER_ELEM = 8*sizeof(ElemType) };

public:
//	typedef SingleElemIndexIteratorOnes<ELEM_TYPE,BitsetType> IndexIteratorOnes;
	typedef typename uint_t_value_opt<BITS_PER_ELEM + 1>::T BitIndexType;
//	typedef unsigned int BitIndexType;

	inline
	BitsetSingleElemImplBase() {
		RawSet_ = 0;
	}

	BITSET_SINGLE_INLINE void
	Set(BitIndexType BitIndex) {
		RawSet_ = bitmanip::setBit(RawSet_, BitIndex);
	}

	BITSET_SINGLE_INLINE void
	Insert(BitIndexType BitIndex, UInt NewState) {
		// Only lowest bit of NewState will be used
		// the rest is ignored.
		RawSet_ = bitmanip::insertBit(RawSet_, NewState, BitIndex);
	}

	BITSET_SINGLE_INLINE void
	Clear(BitIndexType BitIndex) {
		RawSet_= bitmanip::clearBit(RawSet_, BitIndex);
	}

	BITSET_SINGLE_INLINE Bool
	Test(BitIndexType BitIndex) const {
		return bitmanip::testBit(RawSet_, BitIndex);
	}

	BITSET_SINGLE_INLINE Bool
	TestAndClear(BitIndexType BitIndex) {
		return bitmanip::testAndClearBit(&RawSet_, BitIndex);
	}

	BITSET_SINGLE_INLINE BitIndexType
	Lowest1Bit() const {
		return bitmanip::lowest1Bit(RawSet_);
	}

	BITSET_SINGLE_INLINE BitIndexType
	Lowest0Bit() const {
		return bitmanip::lowest0Bit(RawSet_);
	}

	BITSET_SINGLE_INLINE BitIndexType
	Highest1Bit() const {
		return bitmanip::highest1Bit(RawSet_);
	}

	BITSET_SINGLE_INLINE BitIndexType
	Highest0Bit() const {
		return bitmanip::highest0Bit(RawSet_);
	}

	BITSET_SINGLE_INLINE Bool
	AllCleared() const {
		return (RawSet_ == (ElemType)0);
	}

	BITSET_SINGLE_INLINE void
	ClearAll() {
		RawSet_ = (ElemType)0;
	}

	class IndexIteratorOnes {
	public:
		inline IndexIteratorOnes()
			: MyBitset_(0), MyMask_(0) {
		}

		inline IndexIteratorOnes(BitsetType * MyBitset)
			: MyBitset_(MyBitset), MyMask_(metaprog::TwoPowI_minus1<8*sizeof(ELEM_TYPE)>::RES) {
		}

		__attribute__((noinline)) Bool
		GetNext(BitIndexType& FoundIndex) {
			register ElemType MaskedElem = MyBitset_->RawSet_ & MyMask_;
			if(MaskedElem) {
				FoundIndex = bitmanip::lowest1Bit(MaskedElem);
				MyMask_ = (metaprog::TwoPowI_minus1<8*sizeof(ElemType)>::RES - 1) << FoundIndex;
				return 1;
			} else {
				return 0;
			}
			return FoundIndex;
	//			register BitIndexType FoundIndex = bitmanip::lowest1Bit(MyBitset_->RawSet_ & MyMask_);
	//			register ElemType TwoPowI = 1<<FoundIndex;
	//			MyMask_ = ~(ElemType)(TwoPowI - 1 + TwoPowI);
	//			return FoundIndex;
		}

		inline Bool
		HasNext() {
			return MyBitset_->RawSet_ & MyMask_;
		}

	private:
		BitsetType *	MyBitset_;
		ElemType			MyMask_;
	};

	inline IndexIteratorOnes GetIndexIteratorOnes() {
		return IndexIteratorOnes(this);
	}
protected:
	ElemType RawSet_;
};

template<class ELEM_TYPE, int BIT_LENGTH>
class BitsetSingleElemImpl : public BitsetSingleElemImplBase<ELEM_TYPE> {
private:
	typedef BitsetSingleElemImpl<ELEM_TYPE,BIT_LENGTH> BitsetImplBase;
	typedef ELEM_TYPE ElemType;

	enum { LAST_ELEM_BIT_LENGTH = ((BIT_LENGTH-1) % (8*sizeof(ELEM_TYPE))) + 1};

	static const ElemType SetAllElem = metaprog::TwoPowI_minus1<LAST_ELEM_BIT_LENGTH>::RES;

public:
	typedef typename BitsetImplBase::BitIndexType BitIndexType;

	BITSET_SINGLE_INLINE Bool
	AllSet() const {
		return (this->RawSet_ == SetAllElem);
	}

	BITSET_SINGLE_INLINE void
	SetAll() {
		this->RawSet_ = SetAllElem;
	}
/*
	template<class FUNCTOR, BitIndexType I=BIT_LENGTH-1>
	struct ForEachSetBitInner {
		static inline void Do(BitsetImpl & Bitset) {
			ForEachSetBitInner<FUNCTOR, I-1>::Do(Bitset);
			if(Bitset.IsSet(I)) FUNCTOR::Action(Bitset, I);
		}
	};

	template<class FUNCTOR>
	struct ForEachSetBitInner<FUNCTOR,0> {
		static inline void Do(BitsetImpl & Bitset) {
			if(Bitset.IsSet(0)) FUNCTOR::Action(Bitset, 0);
		}
	};

	template<class FUNCTOR>
	void ForEachSetBit() {
		ForEachSetBitInner<FUNCTOR>::Do(*this);
	}
*/
};

#endif
