#ifndef __Chain_h__
#define __Chain_h__

#include "util/container/ChainBase.h"

// forward declaration of ChainIterator
template<class T>
class ChainIterator;

template<class T>
class Chain :public ChainBase{

private:
	T _value;

public:	
	typedef ChainBase Base;
	
	typedef T value_t;	
	
	typedef ChainIterator< Chain<T> > iterator_t;
	
	//! a short (STL conforming) form of iterator type. 
	typedef iterator_t iterator;
	
	//! constructor
	/*!  
	 * \param value is the value of the new Chain object
	 */
	 Chain(T value):_value(value){}
	 	
 	//! constructor
 	/*!  
	 * \param value is the value of the new Chain object
	 * \param chain is another Chain object which will be linked 
	 *  to the new Chain object
	 */
 	Chain(Chain* chain,T value):Base(chain),_value(value){}
 	
 	//! Get the next Chain element
	/*!
	 * \return pointer to the next Chain object
	 */
 	Chain* getNext(){
 		return (Chain*) Base::getNext();
 	}

	//! Get the value of this Chain element
	/*!
	 * \return reference to the value of this Chain element
	 */
	value_t& value(){
			return _value;
	}
	
	//! Set the value of this Chain element
	/*!
	 * \param value is the new value of this Chain element
	 */
	void value(const value_t& value){
		  _value=value;
	}
	
 	//! Find last element in chain
	Chain* findLast(){
		return (Chain<T>*) ChainBase::findLast() ;
	}
 		
	//! Get an iterator which references this chain element
	iterator_t begin(){
		return this;
	}
	
	//! Get an iterator which references this chain element
	iterator_t last(){
		this->findLast;
	}
	

	//! Get an iterator which marks the end of the \ref Chain
	iterator_t end(){
		return 0;
	}
	
};

template<class T>
class ChainIterator  {

	//protected:
	public: //should be protected; set to 'public' for testing purposes
		 
		typedef T chain_t;
		
		typedef typename chain_t::Base chainBase_t;
		
		typedef typename chain_t::value_t value_t;
		
		//! a pointer to the current \ref Chain
		chainBase_t* _currentChain;

	public:

		//public:
		//! Constructor
		/*!
		 * \param chain is a pointer to a \ref Chain object
		 */
		ChainIterator(chainBase_t* chain):_currentChain(chain){}

		//! Increment iterator
		/*!
		 * Set the \ref _currentChain to the next \ref Chain item 
		 * 
		 * \return an iterator
		 */
		ChainIterator operator++ () {
			_currentChain=_currentChain->getNext();
			return *this;
		}

		//! Compare two iterators
		/*!
		 * \return \a true if iterators are equal
		 */
		bool operator==(const ChainIterator i) const{
			return _currentChain == i._currentChain;
		}

		//! Compare two iterators
		/*!
		 * \return \a true if iterators are not equal
		 */
		bool operator!=(const ChainIterator i) const{
			return _currentChain != i._currentChain;
		}

		//! Get the value
		/*!
		 * \return the value of the \ref Chain item the 
		 * iterator is pointing at 
		 */
		value_t& operator* () const{
			return (static_cast< T* >(_currentChain))->value();
		}

		//! insert a chain element after the current position of the iterator 
		/*!
		 * \param chain is pointer to a chain element
		 */
		ChainIterator insert(chainBase_t* chain);
		
				
};

template<class T>
ChainIterator<T> ChainIterator<T>::insert( typename T::Base* chain){
			if(chain != 0 && _currentChain != 0){
				chain->setNext(_currentChain->getNext());
				_currentChain->setNext(chain);
			}
			return _currentChain;
}


#endif //__Chain_h__ 

