// $Id: DlChain.h 995 2009-08-11 14:26:31Z mike $

/**
 * \file DlChain.h
 * \brief Double Linked Chain implementation
 */
 
#ifndef __DlChain_h__
#define __DlChain_h__

#include "util/container/DlChainBase.h"
#include "util/container/Chain.h"

/// forward declaration of ChainIterator
template<class T>
class DlChainIterator;

template<class T>
class DlChain :public DlChainBase{
public:
	
	typedef T value_t;
	
	typedef DlChainBase Base;
	
	typedef DlChainIterator< DlChain<T> > iterator_t;
	
	/// a short (STL conforming) form of iterator type. 
	typedef iterator_t iterator;
	
	
private:
	value_t _value;

public:
	//! constructor
	/*!  
	 * \param value is the value of the new Chain object
	 */
	 DlChain(value_t value):Base(),_value(value){}
	 	
 	//! constructor
 	/*!  
	 * \param value is the value of the new Chain object
	 * \param next is another Chain object which will be linked 
	 *  behind the new Chain object
	 */
 	DlChain(DlChain* next,T value):Base(next),_value(value){}
 	
 	//! constructor
 	/*!  
	 * \param value is the value of the new Chain object
	 * \param prev is another Chain object which will be linked 
	 *  before the new Chain object
	 * \param next is another Chain object which will be linked 
	 *  behind the new Chain object
	 */
 	DlChain(DlChain* prev, DlChain* next, T value):Base(prev,next),_value(value){}
 	
 	//! Get the next Chain element
	/*!
	 * \return pointer to the next Chain object
	 */
 	DlChain* getNext(){
 		return (DlChain*) Base::getNext();
 	}
 	
 	//! Set the next Chain element
	/*!
	 * \param chain is a pointer to the next \ref DlChainBase object
	 */
 	void setNext(DlChain* chain){
 		Base::setNext(chain);
 	}
 	
 	//! Get the previous chain element
	/*!
	 * \return Pointer to the previous chain object
	 */
 	DlChain* getPrev(){
 		return (DlChain*) Base::getPrev();
 	}
 	
 	//! Set the previous Chain element
	/*!
	 * \param chain is a pointer to the next \ref DlChainBase object
	 */
 	void setPrev(DlChain* chain){
 		Base::setPrev(chain);
 	}
 	
 	//! Get the value of this Chain element
	/*!
	 * \return reference to the value of this Chain element
	 */
 	value_t& value(){
 		return _value;
 	}
 	
 	//! Set the value of this Chain element
	/*!
	 * \param value is the new value of this Chain element
	 */
 	void value(const value_t& v){
 		_value=v;
 	}
 	
	//! Find last element in chain
	DlChain* findLast(){
		return (DlChain<T>*) Base::findLast() ;
	}
	
	//! Find first element in chain
	DlChain* findFirst(){
		return (DlChain<T>*) Base::findFirst() ;
	}
	
	//! Get an iterator which marks the end of the \ref Chain
	iterator_t end(){
		return 0;
	}
	
	//! Get an iterator which references this chain element
	iterator_t begin(){
		return this;
	}
	
	
};

/**
 * \brief Iterator for DlChain
 */
template<class T>
class DlChainIterator  : public  ChainIterator<T>{
		

	public:
		
		typedef ChainIterator<T> Base;
		
		typedef T chain_t;
		
		typedef typename chain_t::Base chainBase_t;
		
		typedef typename chain_t::value_t value_t;

		//public:
		//! Constructor
		/*!
		 * \param chain is a pointer to a \ref Chain object
		 */
		DlChainIterator(DlChainBase* chain):Base(chain){}

		//! Increment iterator
		/*!
		 * Set the \ref _currentChain to the next \ref Chain item 
		 * 
		 * \return an iterator
		 */
		DlChainIterator operator++ () {
			this->_currentChain=this->_currentChain->getNext();
			return *this;
		}
		
		//! Increment iterator
		/*!
		 * Set the \ref _currentChain to the next \ref Chain item 
		 * 
		 * \return an iterator
		 */
		DlChainIterator operator-- () {
			this->_currentChain=this->_currentChain->getPrev();
			return *this;
		}


		//! insert a chain element after the current position of the iterator 
		/*!
		 * \param chain is pointer to a chain element
		 */
		DlChainIterator insert(DlChainBase* chain){
			return Base::insert(chain);
		}
		
		
};


#endif //__Chain_h__ 

