#ifndef __DlChainBase_h__
#define __DlChainBase_h__

#include "util/container/ChainBase.h"

//! A double linked Chain
class DlChainBase: public ChainBase{

public:
	typedef ChainBase Base;
	
protected:
	//! Pointer to the previous element
	DlChainBase* _prev;
	
	
public:
	
	//! Default constructor
	DlChainBase():Base(),_prev(0){}
	
	//! Constructor
	/*! 
	 * \param next is a pointer to a next \ref ChainBase element
	 */
	DlChainBase(DlChainBase* next):Base(next),_prev(0){
		if(next != 0){
			next->_prev=this;
		}
	}
	
	//! Constructor
	/*! 
	 * \param next is a pointer to a next \ref ChainBase element
	 * \param prev is a pointer to a previous \ref ChainBase element
	 */
	DlChainBase(DlChainBase* prev, DlChainBase* next):Base(next),_prev(prev){
		if(next != 0){
			next->_prev=this;
		}
		if(prev != 0){
			prev->_next=this;
		}

	}
	
	//! Copy constructor
	/*!
	 * \param chain is another \ref DlChainBase object
	 */
	DlChainBase(DlChainBase& chain):Base(chain),_prev(chain._prev){}
	
	// Default destructor
	~DlChainBase(){}
	
	
 	//! Get the next Chain element
	/*!
	 * \return pointer to the next Chain object
	 */
	DlChainBase* getNext() const{
		return (DlChainBase*) this->_next;
	}
	
	//! Get the previous chain element
	/*!
	 * \return Pointer to the previous chain object
	 */
	DlChainBase* getPrev() const{
		return (DlChainBase*) this->_prev;
	}
	
		
	//! Set the next Chain element
	/*!
	 * \param chain is a pointer to the next \ref DlChainBase object
	 */
 	void setNext(DlChainBase *chain){
 		this->_next=chain;
 		if(chain != 0){
 			chain->_prev=this;
 		}
	}
 	
 	//! Set the previous Chain element
	/*!
	 * \param chain is a pointer to the next \ref DlChainBase object
	 */
 	void setPrev(DlChainBase *chain){
 		this->_prev=chain;
 		if(chain != 0){
 			chain->_next=static_cast<ChainBase*>(this);
 		}
 	}
 	
	
	//! Find last element in double linked Chain
	/*!
	 * \return Pointer to the last element in double linked chain
	 */
	DlChainBase* findLast(){
		return (DlChainBase*) Base::findLast();
	}
		
	//! Find first element in double linked Chain
	/*!
	 * \return Pointer to the first element in double link chain
	 */
	DlChainBase* findFirst();
	
};

#endif //__DlChainBase_h__
