#ifndef __List_h__
#define __List_h__

#include "util/container/Chain.h"

//! A List
/*! Container with constant insertion and deletion time.
 *  This container can be instantiated with custom types and/or 
 *  allocators
 */
template<class T,class C=Chain<T> >
class List{
	public:
		
		//! Type definition for instatiated type
		typedef T value_t;
		
		//! Type definition for pointer to intantiated type
		typedef T* pointer_t;
		
		//! Type of the node which is managed by this container
		typedef C node_t;
		
		//! Type of the iterator 
		typedef typename node_t::iterator_t iterator_t;
	
		//! a short (STL conforming) form of iterator type. 
		typedef iterator_t iterator;
	
	protected:
		//! Pointer to the first list element
		node_t* _front;
	
	public:
	
	
		//! Constructor
		List():_front(0){}
		
		
		//! Constructor
		/*!
		 * \param elem will be the first element in the List
		 */ 
		List(node_t* elem):_front(elem){}
		
		//! Copy Constructor 
		/*!
		 * \param list is another list object
		 */
		List(List& list):_front(list._front){}
		
		//! Add a \ref node_t element to the beginning of this List
		/*! 
		 * \param new_node is a pointer to a \ref node_t 
		 */
		void push_front(node_t* new_node){
			new_node->setNext(this->_front);
			this->_front=new_node;
		}
		
		//! Remove a \ref node_t element from the beginning of this List
		node_t* pop_front(){
			node_t* ret = _front;
			if(_front != 0){
				_front=_front->getNext();
			}
			return ret;
		}
		
		//! 
		node_t* front() const{
			return _front;
		}
		
		//! Test if this list is empty
		bool empty() const{
			return (_front == 0);
		}
		
		//! Get an iterator which marks the start of the list
		iterator_t begin(){
			return _front;
		}
		
		//! Get an iterator which marks the last element of the list
		iterator_t last(){
				if( _front == 0){
					return 0;
				}else{
					return _front->findLast();
				}
		}
		
		//! Get an iterator which marks the end of the list
		iterator_t end(){
			return 0;
		}
		
		
		
};

#endif // __List_h__

