#ifndef __SizeArray_h__
#define __SizeArray_h__

//#include "mem/Allocator.h"

#include "util/container/Array.h"
#include "util/container/StandardIterator.h"
#include "util/types.h"


//! A InitSizeArray
/*! Container with random access
 */
template<class T,int S>
class SizeArray: public Array<T,S>{

// public typedefs and enums
public:
	//!
	typedef Array<T,S> Base;

	//! Type definition of instatiated type
	typedef T value_t;
	
	//! Type definition of pointer to intantiated type
	typedef T* pointer_t;
	
	//! Type definition of an index type
	typedef typename index_t_value<S>::T index_t;

	//! Type definition of an iterator type
	typedef StandardIterator<T> iterator_t;
	
	//! a short (STL conforming) form of iterator type. 
	typedef iterator_t iterator;
	
	//! size of SizeArray	
	enum{CAPACITY=S};
	
// protected variables	
protected:

		//! SizeArray of content types
		UInt8 _content[ CAPACITY * sizeof(value_t) ];
		
		//!
		index_t _pos;
		
	
//public methods
public:
		//! Constructor 
		SizeArray():_pos(0){}
		
		//! Copy constructor
		SizeArray(SizeArray& array):_pos(array.pos),Base(array){}
		
		//! Desturctor
		~SizeArray(){}
		
		
		void push_back(const value_t& elem){
			if(_pos<CAPACITY){
				++_pos;
				new(static_cast<void*>(&(_content[_pos*sizeof(value_t)]))) value_t(elem);
			}
		}
		void pop_back(){
			if(_pos>0){
				--_pos;
			}
		}
		
		value_t& back(){
			return _content[_pos*sizeof(value_t)];
		}
		
		size_t size(){
			return _pos;
		}
		//! Get content of Container
		/*!
		 * \param i is the position of the stored Object
		 */
		value_t& operator[](int i) {
			return (value_t&) _content[i*(sizeof(value_t))];
			
		}
			
		//! Get an iterator which marks the start of the vector
		iterator_t begin(){
			return (pointer_t) _content;
		}
		
		//! Get an iterator which marks the end of the vector
		iterator_t end(){
			return (pointer_t) &(_content[CAPACITY*(sizeof(value_t))]);
		}
};

#endif // __SizeArray_h__
