#ifndef __StandardIterator_h__
#define __StandardIterator_h__
#include "util/types.h"

template<class T>
class StandardIterator {

public:
	//! Type definition of instatiated type
	typedef T value_t;
	//! Type definition of pointer to intantiated type
	typedef T* pointer_t;
	//! Type definition of an index type
	typedef UIntPtr index_t;

	
private:
	//! Pointer to the current position of the iterator in \ref _content
	pointer_t _currentPos;
	
public:
	//! constructor
	/*!
	 * \param pos is a pointer of an array
	 */
	StandardIterator(const pointer_t pos):_currentPos(pos){}
	
	//! copy constructor
	/*!
	 * \param iter is a reference to a StandardIterator
	 * 
	 * remark: code is getting bigger with a copy constructor
	 *         don't know why
	 */
	//StandardIterator(const StandardIterator& iter):_currentPos(iter._currentPos){}
	
	//! increment iterator
	/*!
	 *  
	 * \return an iterator
	 */ 
	StandardIterator operator++ () {
		++_currentPos;
		return _currentPos;
	}
	
	//! decrement iterator
	/*!
	 *  
	 * \return an iterator
	 */ 
	StandardIterator operator-- () {
		--_currentPos;
		return _currentPos;
	}
	
	//! Compare two iterators
	/*!
	 * \return \a true if iterators are equal
	 */
	bool operator==(const StandardIterator& i) const{
		return _currentPos == i._currentPos;
	}
	
	//! Compare two iterators
	/*!
	 * \return \a true if iterators are not equal
	 */
	bool operator!=(const StandardIterator& i) const{
		return _currentPos != i._currentPos;
	}
	
	//! Derefernece operator
	/*!
	 * \return the content of the position  the 
	 *  iterator is pointing at 
	 */
	value_t& operator* () const{
		return *_currentPos;
	}
	
	//! Get the i-th element starting from the current position
	/*! 
	 * \param i is 
	 * \return reference to element with i steps from the current
	 * position
	 */
	value_t& operator[] (index_t i) const{
			return _currentPos[i];
	}
		
		
};

#endif // __StandardIterator_h__
