#ifndef __alloc_Queue_h__
#define __alloc_Queue_h__

#include "util/container/allocating/List.h"
#include "mem/DefaultAllocator.h"

namespace allocating{

//! A Queue
/*! Container which allows efficent efficent access operations 
 * at the end and the beginning of the data. 
 */
template<class T,class A = DefaultAllocator,class C = Chain<T> >
class Queue :public List<T,A,C> {
	public:
		
		//! Type definition for base type
		typedef List<T,A,C> Base;
		
		//! Type definition for instatiated type
		typedef T value_t;
		
		//! Type definition for pointer to intantiated type
		typedef T* pointer_t;
		
		//! Type of the node which is managed by this container
		typedef typename Base::node_t node_t;
		
		//! Type of the iterator 
		typedef typename node_t::iterator_t iterator_t;
		
		//! Type of the allocator
		typedef typename Base::alloc_t alloc_t;
	
		//! Shorter name (STL conforming) of iterator type. 
		typedef iterator_t iterator;
	
	protected:
		//! Pointer to last queue element 
		node_t* _back;
	
	
	public:
	
	
		//! Constructor
		Queue():Base(),_back(0){}
		
		//! Constructor
		/*!
		 * \param allocator will be used to allocate memory for \
		 *        list elements
		 */ 
		Queue(alloc_t& allocator):Base(allocator),_back(0){}
		
		//! Constructor
		/*!
		 * \param elem will be the first element in the Queue
		 */ 
		
		//! Copy constructor
		Queue(Queue& queue):Base(static_cast<Base>(queue)),_back(queue._back){}
		
		//! Constructor
		/*!
		 * When using this constructor the tail of the Queue will be 
		 * adjusted.
		 * 
		 * \param list is a refenrence to list object
		 * 
		 */
		Queue(Base& list):Base(list){
			adjust_back();
		}
		
		bool push_front(const value_t& value){
			bool ret=Base::push_front(value);
			if(this->_back == 0){
				this->_back = this->_front;
			}
			return ret;
		}
		
		void pop_front(){
			Base::pop_front();
			if(this->_front == 0){
				this->_back = 0;
			}
		}
		
		
		bool push_back(const value_t& value){
			node_t* new_node = static_cast<node_t*>(Base::_alloc.allocate( sizeof(node_t) ));
			if ( new_node == 0 ){
				return false;
			}
			new(new_node) node_t(value);
			if(this->_front == 0){
				this->_front = this->_back = new_node;
			}else{
				(this->_back)->setNext(new_node);
				this->_back=new_node;
			}
			return true;
		}
		
		//! Removelist element from the beginning of this Queue
		/*! 
		 * \param value is reference to value of the new element
		 */
		void pop_back(){
			if(this->_back != 0){
				node_t* tmp = this->_back;
				if(this->_front == this->_back){
					this->_front = this->_back = 0;
				}else{
					this->_back =0;
					adjust_back();
				}
				Base::_alloc.free(static_cast<typename alloc_t::pointer_t >(tmp));
			}
			
		}
		
		
		//! Get the content last element of this Queue
		/*!
		 * \return Constant Reference to the value of last \ref node_t in this Queue
		 */
		value_t& back() const{
			return _back->value();
		}

		
		//! Adjust the \ref _back pointer to the last element 
		void adjust_back(){
			_back=Base::_front->findLast();
		}
		
		
		//! Get an iterator which marks the start of the list
		/*!
		 * \return an \ref iterator_t obeject instantiated with the last element
		 */
		iterator_t last(){
			return _back;
		}
		
}; //class Queue

}//namespace allocating

#endif // __alloc_Queue_h__
