// Testcase InterruptPCP appcode.cpp 
// sirupfis 14.02.2008

#include "appcode.h"
#include "as/AS.h"
#include "objects.h"
#include "stdio.h"

#include "hw/irq/PCP.h"
#include "hw/irq/DMA_SYSSRC2.h"
#include "hw/hal/STMTimer.h"
#include "hw/hal/CPU.h"


/////////////////////////////////////////////////////////////////


namespace hw { namespace hal {

using namespace hw::dev;


//#include "hw/hal/STMTimer.h"


/**
 * @brief typically used to implement hal::SystemTimer
 * 
 * @tc STM timers are build on top of the system timer module (STM). STMTimer0 is hard-wired
 * to use the CMP0 compare-and-match register and to use hw::irq::STM_SRC0 as CPU interrupt SRN.
 * 
 */
class STMTimer1 : public STMTimerBase< hw::dev::STM::IRQ_SRC1 > {
	static STMTimer1 theSTMTimer1_;

	//! Performs start-time initializations. Only called by advice.
	void init();

  protected:
  
	Unit	period_;

    STMTimer1(){}

  public:
  
  	typedef STMTimerBase< hw::dev::STM::IRQ_SRC1 > Super;
  	
	//! Returns the one-and-only Timer0 instance
	static STMTimer1& Inst() {
		return theSTMTimer1_;
	}
	
	
	/**
	 * @brief  Windup timer as periodical timer.
	 * 
	 * A periodical timer triggers an irq every \a timer units.
	 * 
	 */
	CIAO_STD void windupPeriodical( Unit period );
			
	/**
	 * @brief  Windup timer as single shot
	 * 
	 * A single shot timer triggers an ... TODO
	 */
	CIAO_STD void windupAndStartSingle( Unit period );


	CIAO_STD void start();
		
	CIAO_STD void stop() {
		hw::dev::STM::ICR().bits.CMP1EN = 0;
	}

	CIAO_STD static Bool running() {
		return hw::dev::STM::ICR().reg & STM_ICR_CMP1EN_MASK;
	}
					
	CIAO_STD static void clear() {
		hw::dev::STM::ISRR().bits.CMP1IRR = 1;
		_dsync();			// ensure bit has been cleared (or do we need _isync() here?)
		IRQ::clear();		// also clear (possibly already pending) SRN flag
	}	

	//! Returns true, iff service request is pending. 
	static Bool pending() {
		// test for pending internal IRQ should be enough, 
		// as SRN flag should always be in line with it.
		return hw::dev::STM::ICR().reg & STM_ICR_CMP1IR_MASK;
	}
	
	//! Explicitly set service request pending flag.
	static void set() {
		// set internal IRQ flag, which automatically sets the SRN flag, iff enabled
		hw::dev::STM::ISRR().bits.CMP1IRS = 1;
	}
	
	/**
	 * @brief Acknowledge timer event. To be called by the IRQ handler (prologue!).
	 * 
	 * @todo Maybe we should provide an aspect for this.
	 */ 
	CIAO_STD void CIAO_INLINE irqAck() {
		if( period_ != 0 )
			hw::dev::STM::CMP1() += period_;
		else
			stop();
	
		// internal pending bit has to be cleared explicitly,
		// SRN pending bit has already been cleared by hardware
		hw::dev::STM::ISRR().bits.CMP1IRR = 1;
	}


};




//#include "STMTimer.h"
//#include "CPU.h"
 

STMTimer1 STMTimer1::theSTMTimer1_;

void STMTimer1::init() {
	// nothing to do...
	
	// STM::init() already did everything
}

void STMTimer1::windupPeriodical( Unit period ) {
	stop();
	clear();
	hw::dev::STM::CMCON().bits.MSIZE1 	= 31;
	hw::dev::STM::CMCON().bits.MSTART1	= 0;
	period_						= period - 4;	// assume 4 cycles for calculations
}

void STMTimer1::windupAndStartSingle( Unit period ) {
	hw::dev::STM::
	CMCON()
	.bits.MSIZE1	= 31;
	hw::dev::STM::
	CMCON()
	.bits.MSTART1	= 0;
	period_ 					= 0;

	// windup and start
	CPU::IEState ie;
	CPU::disable( ie );
	hw::dev::STM::CMP1() = hw::dev::STM::TIM<1>() + period;
	hw::dev::STM::ICR().bits.CMP1EN = 1;	// enable timer
	CPU::enable( ie );	
}

void STMTimer1::start() {
	CPU::IEState ie;
	CPU::disable( ie );
	hw::dev::STM::CMP1() = hw::dev::STM::TIM<1>() + period_;
	hw::dev::STM::ICR().bits.CMP1EN = 1;	// enable timer
	CPU::enable( ie );	
}


}} // namespace hw::hal











///////////////////////////////////////////////////////////////////

// Register definitions - TODO: move to some header file ...
#define IGCR0	*((unsigned int volatile *) 0xF0000094)	/* Interrupt Gating Register 0 */
#define EICR0	*((unsigned int volatile *) 0xF0000080)	/* External Input Channel Register 0 */

//HAS to be overridden! (standard-main() only does StartOS)

int CIAO_WEAK main () {

	// configure ECU to generate interrupt on DMA_SYSSRC2 after
	// reciving singal on P1.0 and to generate interrupt on
	// DMA_SYSSRC3 after reciving singal on P1.1
	IGCR0 = 0xa002a001;
	EICR0 = 0x1a000a00;

	hw::irq::PCP::setTaskInt((os::krn::Task::Pri) 4, (os::krn::Task::Pri) 4, os::krn::Task::Inst(Task1), Task1);
	hw::irq::DMA_SYSSRC2::setTOSPCP ();
	// enable SRN DMA_SYSSRC2
	hw::irq::DMA_SYSSRC2::enable ();
	
	unsigned usec = 50 * 1000;
	hw::irq::PCP::setTaskInt((os::krn::Task::Pri) 5, (os::krn::Task::Pri) 5, os::krn::Task::Inst(Task2), Task2);
	hw::hal::STMTimer0::Inst().windupPeriodical( hw::hal::STMTimer0::usec2units( usec ) );
	hw::hal::STMTimer0::IRQ::setTOSPCP ();
	hw::hal::STMTimer0::Inst().start();
	hw::hal::STMTimer0::IRQ::enable();

	usec = 100 * 1000;
	hw::irq::PCP::setTaskInt((os::krn::Task::Pri) 7, (os::krn::Task::Pri) 7, os::krn::Task::Inst(Task3), Task3);
	hw::hal::STMTimer1::Inst().windupPeriodical( hw::hal::STMTimer1::usec2units( usec ) );
	hw::hal::STMTimer1::IRQ::setTOSPCP ();
	hw::hal::STMTimer1::Inst().start();
	hw::hal::STMTimer1::IRQ::enable();

	// enable interrupts for cpu
	hw::hal::CPU::enable();
	
	// start OS
	os::krn::OSControl::Inst ().startOS (OSDEFAULTAPPMODE);
	return 0;
}


void Alpha::functionTaskTask0 (void) {
//TASKDEF (Alpha,Task0) {
	while (true)
	{
		asm ( " " );
		for (int i = 0; i < 80000; i++)
		{
			asm ( "nop" );
		}
		asm ( " " );
		
		//triggerTask2();
		
		asm ( " " );
		for (int i = 0; i < 80000; i++)
		{
			asm ( "nop" );
		}
		asm ( " " );

		// suspend task
		//AS::ActivateTask (Task1);
   	//os::krn::SchedImpl::Inst ().block (Task0);
		AS::TerminateTask();
   }
}

void Alpha::functionTaskTask1 (void) {
	UInt32 ticks = hw::hal::STMTimer0::usec2units( 10 );
	int count;

		count = 0;
		UInt32 next =	hw::dev::STM::TIM<0>() + ticks;
	
		while (count < 900)
		{
			if ((hw::dev::STM::TIM<0>() < (next - ticks)) || (hw::dev::STM::TIM<0>() > (next + ticks)))
			{
				next = hw::dev::STM::TIM<0>() + ticks;
				count++;
			}
			else if (hw::dev::STM::TIM<0>() > next)
			{
				next = next + ticks;
				count++;
			}
		}
		AS::TerminateTask();
}

void Alpha::functionTaskTask2 (void) {
//TASKDEF (Alpha,Task2) {
	UInt32 ticks = hw::hal::STMTimer0::usec2units( 10 );
	int count;

//	while (true)
//	{
//		hw::hal::CPU::enable();
		
		hw::hal::STMTimer0::Inst().irqAck();
		hw::hal::CPU::enable();

		count = 0;
		UInt32 next =	hw::dev::STM::TIM<0>() + ticks;
	
		while (count < 2500)
		{
			if ((hw::dev::STM::TIM<0>() < (next - ticks)) || (hw::dev::STM::TIM<0>() > (next + ticks)))
			{
				next = hw::dev::STM::TIM<0>() + ticks;
				count++;
			}
			else if (hw::dev::STM::TIM<0>() > next)
			{
				next = next + ticks;
				count++;
			}
		}

		// suspend task
		AS::TerminateTask();

//   }
}

void Alpha::functionTaskTask3 (void) {
//TASKDEF (Alpha,Task3) {

	UInt32 ticks = hw::hal::STMTimer0::usec2units( 10 );
	int count;

		hw::hal::STMTimer1::Inst().irqAck();
		hw::hal::CPU::enable();

		count = 0;
		UInt32 next =	hw::dev::STM::TIM<0>() + ticks;
	
		while (count < 1000)
		{
			if ((hw::dev::STM::TIM<0>() < (next - ticks)) || (hw::dev::STM::TIM<0>() > (next + ticks)))
			{
				next = hw::dev::STM::TIM<0>() + ticks;
				count++;
			}
			else if (hw::dev::STM::TIM<0>() > next)
			{
				next = next + ticks;
				count++;
			}
		}

		// suspend task
   	//os::krn::SchedImpl::Inst ().exitRunning ();
   	//AS::ActivateTask(Task1);
		AS::TerminateTask();
}

