#include "hw/hal/CPU.h"
#include "hw/hal/SystemTimer.h"
#include "hw/hal/SystemClock.h"
#include "hw/hal/Out0.h"
#include "hw/hal/In0.h"
#include "hw/hal/In1.h"
#include "hw/hal/DebugPort.h"
#include "stdio.h"


enum Mode { normal, reenable_In0, reenable_In1 };

unsigned	wait	= 500;
unsigned 	usec 	= 5000 * 1000;
Mode 		mode 	= normal;

using namespace hw::hal;


// The really invoked handler
void cfHAL_IRQHANDLER_ATTRIBUTES hw::hal::SystemTimer::IRQ::HWHandler() {

	//direct handler omitting CiAO's interrupt management

	// acknowledge the irq
	hw::hal::SystemTimer::Inst().irqAck();

	if( mode != normal ){
		switch( mode ) {
		
		case reenable_In0:
			In0::IRQ::clear();
			In0::IRQ::enable();
			printf( "SystemTimer::IRQ -  Reenabled In0::IRQ.\r\n" );
			break;
		case reenable_In1:
			In1::IRQ::clear();
			In1::IRQ::enable();
			printf( "SystemTimer::IRQ - Reenabled In1::IRQ.\r\n" );
			break;
		}

		printf( "SystemTimer::IRQ - Winding up the repetitive timer with %d msec.\r\n", usec / 1000 );
		mode = normal;
		SystemTimer::Inst().windupPeriodical( SystemTimer::usec2units( usec ) );
		SystemTimer::Inst().start();
	} else {
		printf( "SystemTimer::IRQ - The system has been running for %02d:%02d:%02d.%03d\r\n"
		        "                   I will toggle the LED\r\n", 
			SystemClock::Inst().hour(),
			SystemClock::Inst().min(),
			SystemClock::Inst().sec(),
			SystemClock::Inst().msec()
		);
		
		Out0::Inst().toggle();
	}
}


#ifndef __puma

// The really invoked handler
void cfHAL_IRQHANDLER_ATTRIBUTES hw::hal::In0::IRQ::HWHandler() {
	//direct handler omitting CiAO's interrupt management
	printf("In0::IRQ - ON button pressed.\r\n");
	Out0::Inst().set();
	printf("In0::IRQ - using SystemTimer to debounce (%d usec).\r\n", wait );
	SystemTimer::Inst().stop();
	SystemTimer::Inst().windupAndStartSingle( SystemTimer::usec2units( wait ) );
	In0::IRQ::clear();
	In0::IRQ::disable();
	mode = reenable_In0;
	
}


// The really invoked handler
void cfHAL_IRQHANDLER_ATTRIBUTES hw::hal::In1::IRQ::HWHandler() {

	//direct handler omitting CiAO's interrupt management
	printf("In1::IRQ - OFF button pressed.\r\n");
	Out0::Inst().clear();

}

#endif


int main() {


	printf("HELLO - I am the CiAO system, running example MultiTest1\r\n");
	printf("   Have fun! :-)\r\n\r\n" );

	printf( "Winding up the repetitive timer with %d msec.\r\n", usec / 1000 );
	SystemTimer::Inst().windupPeriodical( SystemTimer::usec2units( usec ) );
	SystemTimer::Inst().start();
	SystemTimer::IRQ::enable();

	In0::IRQ::enable();
	In1::IRQ::enable();
	
	CPU::enable();
	
	hw::hal::Out0::Inst().clear();

	

	while( true )
		;

	return 0;		
}
