/** \author Christian Meier
 *  \version $Id: appcode.cpp 961 2009-03-26 12:56:36Z sichmeie $
 *
 *  This test directly injects manually created RxIpdus into ascom rx part.
 *  The associated signals are tested for equality with injected data.
 *
 *  This test should be run with different combinations of
 *  cfOS_COM_ASCOM_RX_SIGNAL_*_*_BIT_INDEX to test different extraction algorithms
 *  as well as "Bitlength inside IPDU may differ signal bitlength" feature disabled
 *  and enabled, ...
 */

#include "appcode.h"
#include "as/AS.h"
#include "util/types.h"
#include "objects.h"
#include "stdio.h"
#include "string.h"

#include "AsCom_Cfg.h"
#include "os/com/ComStack_Types.h"

// only needed for direct injection into AsCom
#include "os/com/ascom/rx/RxIpduType.h"

UInt8 RxSignalU8 = 0;
UInt16 RxSignalU16 = 0;
UInt32 RxSignalU32 = 0;

Int8 RxSignal8 = 0;
Int16 RxSignal16 = 0;
Int32 RxSignal32 = 0;

UInt8 RxSignalU8Comp = 0;
UInt16 RxSignalU16Comp = 0;
UInt32 RxSignalU32Comp = 0;

Int8 RxSignal8Comp = 0;
Int16 RxSignal16Comp = 0;
Int32 RxSignal32Comp = 0;

UInt8 SduBuffer[2][8];

TASKDEF(Alpha,Task0) {
	AS::Com_RxIpduGroupStart(0, true);

	while(true) {
		// We directly inject some data into ascom
		memcpy(&SduBuffer[0][0], &RxSignalU8Comp, 1);
		memcpy(&SduBuffer[0][1], &RxSignalU16Comp, 2);
		memcpy(&SduBuffer[0][3], &RxSignalU32Comp, 4);
		os::com::ascom::RxIpduType::Inst (0).RxIndication (SduBuffer[0]);

		memcpy(&SduBuffer[1][0], &RxSignal8Comp, 1);
		memcpy(&SduBuffer[1][1], &RxSignal16Comp, 2);
		memcpy(&SduBuffer[1][3], &RxSignal32Comp, 4);
		os::com::ascom::RxIpduType::Inst (1).RxIndication (SduBuffer[1]);

		// We can now read the data from ascom
		AS::Com_ReceiveSignal (0, &RxSignalU8);
		AS::Com_ReceiveSignal (1, &RxSignalU16);
		AS::Com_ReceiveSignal (2, &RxSignalU32);
		AS::Com_ReceiveSignal (3, &RxSignal8);
		AS::Com_ReceiveSignal (4, &RxSignal16);
		AS::Com_ReceiveSignal (5, &RxSignal32);

		// the signal must always represent the injected data
		if ((RxSignalU8 != RxSignalU8Comp)
				|| (RxSignalU16 != RxSignalU16Comp)
				|| (RxSignalU32 != RxSignalU32Comp)
				|| (RxSignal8   != RxSignal8Comp)
				|| (RxSignal16  != RxSignal16Comp)
				|| (RxSignal32  != RxSignal32Comp)) {
			printf("Data not received correctly!");
		}

		RxSignalU8Comp  += 1;
		RxSignalU16Comp += 2;
		RxSignalU32Comp += 3;
		RxSignal8Comp 	+= 4;
		RxSignal16Comp	+= 5;
		RxSignal32Comp	+= 6;
	}

	AS::TerminateTask();
}
