/** \author Christian Meier
 *  \version $Id: appcode.cpp 963 2009-03-26 17:39:58Z sichmeie $
 *
 *  This test sends some byte aligned data which will be caught in TxIpduCallout.
 *  In this test case the transmission are done by a cyclic task to get used to alarms.
 */

#include "appcode.h"
#include "as/AS.h"
#include "util/types.h"
#include "objects.h"
#include "stdio.h"

#include "os/com/ComStack_Types.h"

#include "AsCom_Cfg.h"

namespace os { namespace krn {

	Alarm theAlarms [] = {
		{{OSMAXALLOWEDVALUE_SYSTEMCOUNTER, OSTICKSPERBASE_SYSTEMCOUNTER, OSMINCYCLE_SYSTEMCOUNTER}, 0, 0, PeriodicTask0, Alarm::INVALID, Alarm::INVALID}
	};
}}

TASKDEF(Alpha,InitTask) {
	hw::hal::SystemTimer::IRQ::enable ();
	AS::Com_TxIpduGroupStart(ASCOM_TX_IPDU_GROUP_TEST, true);

	AS::SetRelAlarm (PeriodicAlarm0, 1000000000U / OSTICKDURATION, 1000000000U / OSTICKDURATION);

	AS::TerminateTask();
}

UInt8 TxSignal8 = 0;
UInt16 TxSignal16 = 0;
UInt32 TxSignal32 = 0;

UInt8 TxSignal8Comp = 0;
UInt16 TxSignal16Comp = 0;
UInt32 TxSignal32Comp = 0;

UInt32 TxCounter = 0;
UInt32 TxCounterNew = 0;

TASKDEF(Alpha,PeriodicTask0) {
	// Pending signal must not lead to a transmission
	TxSignal8++;
	AS::Com_SendSignal(ASCOM_TX_SIGNAL_SMALL,  &TxSignal8);

	// Pending signal must not lead to a transmission
	TxSignal16++;
	AS::Com_SendSignal(ASCOM_TX_SIGNAL_MEDIUM, &TxSignal16);

	// Triggering signal must lead to a transmission
	// as corresponding Ipdu is configured to direct transmission mode
	TxSignal32++;
	AS::Com_SendSignal(ASCOM_TX_SIGNAL_LARGE,  &TxSignal32);

	if (TxCounterNew != (TxCounter + 1)) {
		printf("DefaultTxIpduCallout hasn't been called correctly!\r\n");
	}
	TxCounter = TxCounterNew;

	AS::TerminateTask();
}

Bool DefaultTxIpduCallout(os::com::PduIdType PduId, UInt8 const * SduPtr) {
	TxCounterNew = TxCounter + 1;

	memcpy (&TxSignal8Comp, SduPtr, 1);
	memcpy (&TxSignal16Comp, &SduPtr[1], 2);
	memcpy (&TxSignal32Comp, &SduPtr[3], 4);

	if ((TxSignal8Comp != TxSignal8)
		 || (TxSignal16Comp != TxSignal16)
		 || (TxSignal32Comp != TxSignal32)) {
		printf("Tx Signals not correctly transmitted!\r\n");
	}

	return true;
}
