/** \author Christian Meier
 *  \version $Id: appcode.cpp 960 2009-03-25 21:45:04Z sichmeie $
 *
 *  This test is good for testing tx/rx Ipdu callouts normal tx/rx signals as well as
 *  tx/rx signal groups.
 */

#include "appcode.h"
#include "as/AS.h"
#include "util/types.h"
#include "objects.h"
#include "stdio.h"

#include "AsCom_Cfg.h"
#include "os/com/ComStack_Types.h"

// Only neccessary for direct reinjection of sent pdus into ascom
#include "os/com/ascom/rx/RxIpduType.h"

UInt8 RxSignal8 = 0;
UInt8 TxSignal8 = 0;

UInt16 RxSignal16 = 0;
UInt16 TxSignal16 = 0;

UInt32 RxSignal32 = 0;
UInt32 TxSignal32 = 0;

UInt8 RxGroupSignal_0_0 = 0;
UInt8 TxGroupSignal_0_0 = 0;

UInt8 RxGroupSignal_0_1 = 0;
UInt8 TxGroupSignal_0_1 = 0;

UInt16 RxGroupSignal_1_0 = 0;
UInt16 TxGroupSignal_1_0 = 0;

UInt16 RxGroupSignal_1_1 = 0;
UInt16 TxGroupSignal_1_1 = 0;

UInt8 SduBuffer[8];

volatile UInt8 RxCounter = 0;
volatile UInt8 RxCounterNew = 0;

void
checkTxConsistency () {
	os::com::ascom::TxIpduType::Inst (1).TriggerTransmit (SduBuffer);
	if((SduBuffer[0] != SduBuffer[1])
			|| (SduBuffer[2] != SduBuffer[4])
			|| (SduBuffer[3] != SduBuffer[5])) {
		printf ("Tx Signal Group inconsistency!\r\n");
	}
}

void
checkRxConsistency (int descriptor) {
	if ((RxGroupSignal_0_0 != RxGroupSignal_0_1)
			|| (RxGroupSignal_1_0 != RxGroupSignal_1_1)) {
		printf ("%d :Rx Signal Group inconsistency!\r\n", descriptor);
	}
}

void
ReceiveShadowSignals () {
	AS::ReceiveShadowSignal (ASCOM_RX_GROUP_SIGNAL_0_0, &RxGroupSignal_0_0);
	AS::ReceiveShadowSignal (ASCOM_RX_GROUP_SIGNAL_0_1, &RxGroupSignal_0_1);
	AS::ReceiveShadowSignal (ASCOM_RX_GROUP_SIGNAL_1_0, &RxGroupSignal_1_0);
	AS::ReceiveShadowSignal (ASCOM_RX_GROUP_SIGNAL_1_1, &RxGroupSignal_1_1);
}

void Alpha::functionTaskTask0(void) {
	AS::Com_TxIpduGroupStart (0, true);
	AS::Com_RxIpduGroupStart (0, true);

	while(true) {
		TxSignal8++;
		TxSignal16++;
		TxSignal32++;

		AS::Com_SendSignal(ASCOM_TX_SIGNAL_SMALL,  &TxSignal8);
		AS::Com_SendSignal(ASCOM_TX_SIGNAL_MEDIUM, &TxSignal16);
		AS::Com_SendSignal(ASCOM_TX_SIGNAL_LARGE,  &TxSignal32);

		do {
			AS::Com_ReceiveSignal(ASCOM_RX_SIGNAL_SMALL, &RxSignal8);
		} while(RxSignal8 != TxSignal8);

		do {
			AS::Com_ReceiveSignal(ASCOM_RX_SIGNAL_MEDIUM, &RxSignal16);
		} while(RxSignal16 != TxSignal16);

		do {
			AS::Com_ReceiveSignal(ASCOM_RX_SIGNAL_LARGE, &RxSignal32);
		} while(RxSignal32 != TxSignal32);

//		printf("Received: 8bit = %d, 16bit = %d, 32bit = %d\r\n", RxSignal8, RxSignal16, RxSignal32);

		if(TxSignal32 == 70000) {
			break;
		}

		TxGroupSignal_0_0++;
		TxGroupSignal_0_1++;
		TxGroupSignal_1_0 += 2;
		TxGroupSignal_1_1 += 2;

		// Test for signal group consistency
		AS::Com_UpdateShadowSignal (ASCOM_TX_GROUP_SIGNAL_0_0, &TxGroupSignal_0_0);
		checkTxConsistency ();

		AS::Com_UpdateShadowSignal (ASCOM_TX_GROUP_SIGNAL_1_0, &TxGroupSignal_1_0);
		checkTxConsistency ();

		AS::Com_UpdateShadowSignal (ASCOM_TX_GROUP_SIGNAL_0_1, &TxGroupSignal_0_1);
		checkTxConsistency ();

		AS::Com_UpdateShadowSignal (ASCOM_TX_GROUP_SIGNAL_1_1, &TxGroupSignal_1_1);
		checkTxConsistency ();

		AS::Com_SendSignalGroup (ASCOM_TX_SIGNAL_GROUP_0);
		checkTxConsistency ();

		AS::Com_SendSignalGroup (ASCOM_TX_SIGNAL_GROUP_1);
		checkTxConsistency ();

		// rx part of signal groups

		// as we directly reinject the TxIpdu into rx part the check should always succeed immediately.
		// this is just as marker what had to be done for testing with for example can interface
		while (RxCounter == RxCounterNew);
		RxCounter = RxCounterNew;

		checkRxConsistency (0);

		AS::Com_ReceiveSignalGroup (ASCOM_RX_SIGNAL_GROUP_0);
		ReceiveShadowSignals ();
		checkRxConsistency (1);

		AS::Com_ReceiveSignalGroup (ASCOM_RX_SIGNAL_GROUP_1);
		ReceiveShadowSignals ();
		checkRxConsistency (2);

		if ((RxGroupSignal_0_0 != TxGroupSignal_0_0)
				|| (RxGroupSignal_0_1 != TxGroupSignal_0_1)
				|| (RxGroupSignal_1_0 != TxGroupSignal_1_0)
				|| (RxGroupSignal_1_1 != TxGroupSignal_1_1)) {
			printf("Data not correctly received!\r\n");
		}
	}

	AS::TerminateTask();
}

Bool
DefaultTxIpduCallout (os::com::PduIdType PduId, UInt8 const * SduPtr) {
	// We make a shortcut for testing here, TxIpdus will be directly reinjected with same id.
	// as this is called in sync to transmit events and we test from inside one task we don't
	// have to worry about direct reinjection here.
	// BUT: stack should be large enough.
	os::com::ascom::RxIpduType::Inst (PduId).RxIndication (SduPtr);
	return false;
}

Bool
DefaultRxIpduCallout (os::com::PduIdType PduId, UInt8 const * SduPtr) {
	RxCounterNew++;
	return true;
}
