#include "stdio.h"
#include "hw/hal/DebugPort.h"
#include "hw/hal/CPU.h"
#include "os/krn/SchedImpl.h"
#include "os/krn/EnterLeave.h"
#include "os/dev/std/Serial0.h"
#include "hw/hal/AST0.h"

/* use a second task with lower priority
 * which should be preempted by echo thread*/
#define __IDLE_TASK__

#ifdef __IDLE_TASK__
#define __THREAD_NUM__ 2
#else
#define __THREAD_NUM__ 1
#endif

#define theSched os::krn::SchedImpl::Inst()
#define theSerial0 os::dev::std::Serial0::Inst()


void task0() {

	printf( "task0 has been started, enters the kernel\r\n" );

	while (true) {
		printf("idle thread\r\n");
	}
}
void task1() {
	UInt8 ch;
	//HACK: Must not enable interrupts before the running continuation is known
	printf("Echo task started.\r\n");
	
	//hw::irq::ASC1_REC::enable();
	//hw::irq::ASC1_TBUF::enable();
	
	while (true) {
		os::krn::enterKernel();
		ch=theSerial0.getc();
		os::krn::leaveKernel();

		os::krn::enterKernel();
		theSerial0.putc(ch);
		os::krn::leaveKernel();
	}
}

namespace os { namespace krn {
	const int STACK = 64;
	UInt32 stack0[ STACK ];
	UInt32 stack1[ STACK ];
		
	Task myTasks[] = {
#ifdef __IDLE_TASK__
		 {0, Task::READY | Task::TERMINATED, task0, &stack0[ STACK ] },	
#endif		 
		 {1, Task::READY | Task::TERMINATED, task1, &stack1[ STACK ] }	
	};
}}


int main() {
	theSched.setTaskTable( os::krn::myTasks, __THREAD_NUM__ );
	
	os::krn::enterKernel();
	
#ifdef __IDLE_TASK__
	/*enable preemption
	 *only effects the 2 tasks case*/
	hw::hal::AST0::enable();
#endif
	
	//start scheduling
	hw::hal::CPU::enable();
	theSched.start();
	
	//should never come here
	_debug();
	return 0;
}
