// $Id: main.cpp 1211 2010-07-16 13:05:25Z sijesche $
// Created: 2007-07-30 siwahofe

#include "stdio.h"
#include "hw/hal/DebugPort.h"
#include "hw/hal/CPU.h"
#include "os/krn/ASSched.h"
#include "os/krn/Task.h"
#include "os/krn/ISR.h"
#include "os/krn/EnterLeave.h"
#include "ciaoConfig.h"
#include "os/krn/Resource.h"
#include "os/krn/Alarm.h"
#include "os/krn/Application.h"
#include "hw/hal/Out0.h"
#include "hw/hal/In0.h"
#include "hw/hal/In1.h"
#include "hw/hal/init.h"
#include "hw/irq/DMA_SYSSRC2.h"
#include "hw/irq/DMA_SYSSRC3.h"
//#include "hw/irq/STM_SRC0.h"
#include "os/krn/AppMode.h"
#include "hw/hal/SystemTimer.h"
#include "autosar.h"

#include "hw/dev/STM.h"

// generated by the OIL generator {

const TaskType Task0 = 0;
const TaskType Task1 = 1;
const TaskType Task2 = 2;
const TaskType Task3 = 3;
const ISRType ISRIn1 = 0;
const ResourceType CommonResource = 0;
const EventMaskType SyncEvent = 1 << 0;
const AlarmType WakeupAlarm = 0;
const ApplicationType App0 = 0;
const ApplicationType App1 = 1;

const TickType OSMAXALLOWEDVALUE_SYSTEMCOUNTER = 500000000;
const TickType OSTICKSPERBASE_SYSTEMCOUNTER = 4711;
const TickType OSMINCYCLE_SYSTEMCOUNTER = 20000000;
const TickType OSMAXALLOWEDVALUE = OSMAXALLOWEDVALUE_SYSTEMCOUNTER;
const TickType OSTICKSPERBASE = OSTICKSPERBASE_SYSTEMCOUNTER;
const TickType OSMINCYCLE = OSMINCYCLE_SYSTEMCOUNTER;
const int OSTICKDURATION = 1000000000 / hw::dev::STM::FREQ; // nanoseconds

void functionTaskTask0 (void);
void functionTaskTask1 (void);
void functionTaskTask2 (void);
void functionTaskTask3 (void);
void functionISRISRIn1 (void);
void StartupHook_App1 (void);
void ShutdownHook_App0 (StatusType status);
void ErrorHook_App0 (StatusType status);

namespace os { namespace krn {

	const unsigned int STACKSIZE = 128;
	const unsigned int STACKSIZE2 = 128; // 128 or 16
	UInt32 stack0 [STACKSIZE2];
	UInt32 stack1 [STACKSIZE];
	UInt32 stack2 [STACKSIZE];
	UInt32 stack3 [STACKSIZE];
//	UInt32 stack4 [STACKSIZE];

	// stack pointer points to last used element
	Task theTasks [] = {
		// priority, state, function, stack, interrupted, application, original priority, stack size, current activations, max. activations, current events, events waited for, preemptable
		{2, SUSPENDED, functionTaskTask0, & stack0 [STACKSIZE2], 0, 0, 2, (unsigned) sizeof stack0, 0, 5, 0, 0, 1}, // Task0
		{3, SUSPENDED, functionTaskTask1, & stack1 [STACKSIZE], 0, 1, 3, (unsigned) sizeof stack1, 0, 3, 0, 0, 1}, // Task1
		{5, SUSPENDED, functionTaskTask2, & stack2 [STACKSIZE], 0, 0, 5, (unsigned) sizeof stack2, 0, 1, 0, 0, 0}, // Task2
		{4, SUSPENDED, functionTaskTask3, & stack3 [STACKSIZE], 0, 1, 4, (unsigned) sizeof stack3, 0, 1, 0, 0, 1} // Task3
	};

	ISR theISRs [] = {
		// function, register address, application
		{functionISRISRIn1, (ISR::RegisterAddress) DMA_SYSSRC3_ADDR, 0} // ISRIn1
	};

	Resource theResources [] = {
		// ceiling priority
		{3} // CommonResource
	};

	AppMode theAppModes [] = {
		// autostarted tasks
		{(1 << Task0)} // OSDEFAULTAPPMODE
	};

	Alarm theAlarms [] = {
		// alarm base (maxallowedvalue, ticksperbase, mincycle), armed, absolute time, cycle time, activate task (/set event), task, event
		{{OSMAXALLOWEDVALUE_SYSTEMCOUNTER, OSTICKSPERBASE_SYSTEMCOUNTER, OSMINCYCLE_SYSTEMCOUNTER}, 0, 0, 0, 1, Task2, 0}
	};

	Application theApplications [] = {
		// startup hook, shutdown hook, error hook
		{0, ShutdownHook_App0, ErrorHook_App0}, // App0
		{StartupHook_App1, 0, 0} // App1
	};

}} // os::krn

// ISRIn1
Bool hw::irq::DMA_SYSSRC3::HWHandler () {
	os::krn::ASSched::Inst ().dispatchISR (ISRIn1);
	return true;
}

// can be overwritten by user specific main (),
// which must call StartOS () at the end
int CIAO_WEAK main () {
	hw::hal::In0::IRQ::enable ();
	hw::hal::In1::IRQ::enable ();
	hw::irq::ASC0_REC::enable ();
	hw::hal::SystemTimer::IRQ::enable ();
	StartOS (OSDEFAULTAPPMODE);

	return 0;
}

// debug: manually trigger interrupt
#include <tc1796b/dma.h>

Bool hw::irq::ASC0_REC::HWHandler () {
	char c;
	do {
		c = hw::dev::ASC0::Inst ().receive ();
	}
	while (c == 'p');

	if (c == '0') {
		//CPU_SRC2.bits.SRPN 	= pri;
		DMA_SYSSRC2.bits.SETR	= 1; 
	} else if (c == '1') {
		//CPU_SRC2.bits.SRPN 	= pri;
		DMA_SYSSRC3.bits.SETR	= 1; 
	}

	return true;
}

// }

// user code
#include "autosar.h"

DeclareTask (Task0);
DeclareTask (Task1);
DeclareTask (Task2);
DeclareTask (Task3);
DeclareResource (CommonResource);
DeclareEvent (SyncEvent);

// ISR 1
Bool hw::irq::DMA_SYSSRC2::HWHandler () {
	printf ("IRQ 1!\r\n");
	ISRType id = GetISRID ();
	printf ("My ISRID is: %i.\r\n", id);
	return true;
}

// ISR 2
ISR (ISRIn1) {
	printf ("XXXXXXXXXXXXXXXXXXXXXXXXX\r\n");
	printf ("IRQ 2 activates Task2!\r\n");
	ActivateTask (Task2);
	ISRType id = GetISRID ();
	printf ("My ISRID is: %i.\r\n", id);
	printf ("Calling ProtectionHook manually.\r\n");
	os::krn::OS::Inst ().internalProtectionHook (E_OK);
	ApplicationType app = GetApplicationID ();
	printf ("My AppID is: %i.\r\n", app);
}

TASK (Task0) {
	printf ("Preemptable task starting...\r\n");
	printf ("Getting resource.\r\n");
	GetResource (CommonResource);
	printf ("Activating other task with higher priority.\r\n");
	ActivateTask (Task1);
	printf ("Doing more stuff.\r\n");
	//printf ("Releasing resource.\r\n");
	//ReleaseResource (CommonResource);
	printf ("Doing more stuff, extended.\r\n");
	//printf ("Preemptable task ending...\r\n");
	//TerminateTask ();
	printf ("Preemptable task ending without TerminateTask ()...\r\n");
}

TASK (Task1) {
	printf ("\tPreemptable task starting...\r\n");
	printf ("\tDoing things.\r\n");
	printf ("\tFreq: %i.\r\n", hw::dev::STM::FREQ);
	printf ("\tCurrent interrupt suspension counter: %i.\r\n", os::krn::OS::Inst ().getInterruptSuspensionCounter ());
	printf ("\tSuspending interrupts once.\r\n");
	SuspendAllInterrupts ();
	printf ("\tCurrent interrupt suspension counter: %i.\r\n", os::krn::OS::Inst ().getInterruptSuspensionCounter ());
	printf ("\tResuming interrupts three times.\r\n");
	ResumeAllInterrupts ();
	ResumeAllInterrupts ();
	ResumeAllInterrupts ();
	printf ("\tCurrent interrupt suspension counter: %i.\r\n", os::krn::OS::Inst ().getInterruptSuspensionCounter ());
	printf ("\tCurrent OS interrupt suspension counter: %i.\r\n", os::krn::OS::Inst ().getInterruptOSSuspensionCounter ());
	printf ("\tResuming OS interrupts three times.\r\n");
	ResumeOSInterrupts ();
	ResumeOSInterrupts ();
	ResumeOSInterrupts ();
	printf ("\tCurrent OS interrupt suspension counter: %i.\r\n", os::krn::OS::Inst ().getInterruptOSSuspensionCounter ());

	printf ("\tDisabling interrupts.\r\n");
	DisableAllInterrupts ();
	printf ("\tTrying to activate Task2.\r\n");
	StatusType ret = ActivateTask (Task2);
	printf ("\tReturn value was: %i.\r\n", ret);
	printf ("\tEnabling interrupts.\r\n");
	EnableAllInterrupts ();

	printf ("\tSetting alarm with invalid value.\r\n");
	StatusType ret2 = SetRelAlarm (WakeupAlarm, 1000000000, 160000000);
	//// 2 billion nanoseconds = 2 seconds
	//StatusType ret2 = SetRelAlarm (WakeupAlarm, 2000000000 / OSTICKDURATION, 2000000000 / OSTICKDURATION);
	printf ("\tReturn value was: %i.\r\n", ret2);

	printf ("\tEndless loop.\r\n");
	for (;;);

	printf ("\tPreemptable task ending...\r\n");
	TerminateTask ();
}

TASK (Task2) {
	printf ("\t\tTask2 says hello and goodbye...\r\n");
	TerminateTask ();
}

TASK (Task3) {
	TerminateTask ();
}

void ErrorHook (StatusType error) {
	printf ("! ErrorHook called with error %i\r\n", error);
}

void ErrorHook_App0 (StatusType error) {
	printf ("! ErrorHook_App0 called with error %i\r\n", error);
}

void PreTaskHook () {
	TaskType current;

	GetTaskID (current);

	printf ("! PreTaskHook: new task: %i\r\n", current);
}

void PostTaskHook () {
	TaskType current;

	GetTaskID (current);

	printf ("! PostTaskHook: old task: %i\r\n", current);
}

void StartupHook () {
	printf ("! Doing some StartupHook stuff\r\n");
}

void StartupHook_App1 () {
	printf ("! Doing some StartupHook_App1 stuff\r\n");
}

void ShutdownHook (StatusType error) {
	printf ("! Doing some ShutdownHook stuff, error was: %i\r\n", error);
}

void ShutdownHook_App0 (StatusType error) {
	printf ("! Doing some ShutdownHook_App0 stuff, error was: %i\r\n", error);
}

ProtectionReturnType ProtectionHook (StatusType fatalerror) {
	printf ("! Doing some ProtectionHook stuff, error was: %i\r\n", fatalerror);
	return PRO_SHUTDOWN;
}
