// $Id: main.cpp 483 2007-07-27 08:18:39Z siwahofe $
// Created: 2007-08-04 siwahofe

#include "as/autosar_types.h"

#include "os/krn/Task.h"
#include "os/krn/Alarm.h"
#include "os/krn/AppMode.h"

#include "hw/dev/STM.h"
#include "hw/hal/SystemTimer.h"

#include "os/krn/OSControl.h"

// generated by the OIL generator {

const TaskType Task0 = 0;
const TaskType Task1 = 1;
const AlarmType SystemAlarm = 0;
const AlarmType SystemAlarm2 = 1;

const TickType OSMAXALLOWEDVALUE_SYSTEMCOUNTER = 500000000;
const TickType OSTICKSPERBASE_SYSTEMCOUNTER = 4711;
const TickType OSMINCYCLE_SYSTEMCOUNTER = 20000000;
const TickType OSMAXALLOWEDVALUE = OSMAXALLOWEDVALUE_SYSTEMCOUNTER;
const TickType OSTICKSPERBASE = OSTICKSPERBASE_SYSTEMCOUNTER;
const TickType OSMINCYCLE = OSMINCYCLE_SYSTEMCOUNTER;
const int OSTICKDURATION = 1000000000 / hw::dev::STM::FREQ; // nanoseconds

void functionTaskTask0 (void);
void functionTaskTask1 (void);

namespace os { namespace krn {

	const unsigned int STACKSIZE = 128;
	UInt32 stack0 [STACKSIZE];
	UInt32 stack1 [STACKSIZE];
	
	Task theTasks [] = {
		// priority, state, function, stack, interrupted
		{3, Task::SUSPENDED, functionTaskTask0, & stack0 [STACKSIZE], false}, // Task0
		{5, Task::SUSPENDED, functionTaskTask1, & stack1 [STACKSIZE], false} // Task1
	};

	Alarm theAlarms [] = {
		// *with* events configured
//		// alarm base (), armed, absolute time, cycle time, task, activate task (! set event), event
//		{{OSMAXALLOWEDVALUE_SYSTEMCOUNTER, OSTICKSPERBASE_SYSTEMCOUNTER, OSMINCYCLE_SYSTEMCOUNTER}, false, 0, 0, Task1, true, 0}, // SystemAlarm
//		{{OSMAXALLOWEDVALUE_SYSTEMCOUNTER, OSTICKSPERBASE_SYSTEMCOUNTER, OSMINCYCLE_SYSTEMCOUNTER}, false, 0, 0, Task1, true, 0} // SystemAlarm2
		// *without* events configured
		// alarm base (), armed, absolute time, cycle time, task
		{{OSMAXALLOWEDVALUE_SYSTEMCOUNTER, OSTICKSPERBASE_SYSTEMCOUNTER, OSMINCYCLE_SYSTEMCOUNTER}, false, 0, 0, Task1}, // SystemAlarm
		{{OSMAXALLOWEDVALUE_SYSTEMCOUNTER, OSTICKSPERBASE_SYSTEMCOUNTER, OSMINCYCLE_SYSTEMCOUNTER}, false, 0, 0, Task1} // SystemAlarm2
	};

	AppMode theAppModes [] = {
		// autostarted tasks
		{(1 << Task0)} // OSDEFAULTAPPMODE
	};
}} // os::krn

// can be overwritten by user specific main (),
// which must call StartOS () at the end
int CIAO_WEAK main () {
	hw::hal::SystemTimer::IRQ::enable ();
	os::krn::OSControl::Inst ().startOS (OSDEFAULTAPPMODE);

	return 0;
}

// user code
#include "stdio.h"
#include "as/autosar_macros.h"
#include "as/AS.h"

TASK (Task0) {
	StatusType ret;
	printf ("Task0 starting...\r\n");
	printf ("Task0 setting relative alarm in 2 seconds...\r\n");
	ret = AS::SetRelAlarm (SystemAlarm, 2000000000U / OSTICKDURATION, 0);
	printf ("Task0: return value %i.\r\n", ret);
	printf ("Task0 setting relative alarm in 3 seconds...\r\n");
	ret = AS::SetRelAlarm (SystemAlarm2, 3000000000U / OSTICKDURATION, 0);
	printf ("Task0: return value %i.\r\n", ret);
	printf ("Task0 ending...\r\n");
	AS::TerminateTask ();
}

TASK (Task1) {
	TaskType taskid;
	TaskStateType state;

	printf ("Task1 starting...\r\n");
	AS::GetTaskID (taskid);
	AS::GetTaskState (Task0, state);
	printf ("Task1: ID %i, state of Task0 %i.\r\n", taskid, state);
	printf ("Task1 ending...\r\n");
	AS::TerminateTask ();
}
