// $Id: main.cpp 483 2007-07-27 08:18:39Z siwahofe $
// Created: 2007-06-04 siwahofe

#include "stdio.h"
#include "hw/hal/DebugPort.h"
#include "hw/hal/CPU.h"
#include "os/krn/ASSched.h"
#include "os/krn/Task.h"
#include "os/krn/ISR.h"
#include "os/krn/EnterLeave.h"
#include "ciaoConfig.h"
#include "os/krn/Resource.h"
#include "os/krn/Alarm.h"
#include "os/krn/Application.h"
#include "hw/hal/Out0.h"
#include "hw/hal/In0.h"
#include "hw/hal/In1.h"
#include "hw/hal/init.h"
#include "hw/irq/DMA_SYSSRC2.h"
#include "hw/irq/DMA_SYSSRC3.h"
//#include "hw/irq/STM_SRC0.h"
#include "os/krn/AppMode.h"
#include "hw/hal/SystemTimer.h"
#include "autosar.h"

// generated by the OIL generator {

const TaskType Task0 = 0;
const TaskType Task1 = 1;
const TaskType Task2 = 2;
const TaskType Task3 = 3;
const ISRType ISRIn1 = 0;
const ResourceType CommonResource = 0;
const EventMaskType SyncEvent = 1 << 0;
const AlarmType WakeupAlarm = 0;
const ApplicationType App0 = 0;
const ApplicationType App1 = 1;

void functionTaskTask0 (void);
void functionTaskTask1 (void);
void functionTaskTask2 (void);
void functionTaskTask3 (void);
void functionISRISRIn1 (void);
void StartupHook_App1 (void);
void ShutdownHook_App0 (StatusType status);
void ErrorHook_App0 (StatusType status);

namespace os { namespace krn {

	const unsigned int STACKSIZE = 128;
	const unsigned int STACKSIZE2 = 128; // 128 or 16
	UInt32 stack0 [STACKSIZE2];
	UInt32 stack1 [STACKSIZE];
	UInt32 stack2 [STACKSIZE];
	UInt32 stack3 [STACKSIZE];
//	UInt32 stack4 [STACKSIZE];

	// stack pointer points to last used element
	Task theTasks [] = {
		// priority, state, function, stack, interrupted, application, original priority, stack size, current activations, max. activations, current events, events waited for, preemptable
		{2, SUSPENDED, functionTaskTask0, & stack0 [STACKSIZE2], 0, 0, 2, (unsigned) sizeof stack0, 0, 5, 0, 0, 1}, // Task0
		{3, SUSPENDED, functionTaskTask1, & stack1 [STACKSIZE], 0, 1, 3, (unsigned) sizeof stack1, 0, 3, 0, 0, 0}, // Task1
		{5, SUSPENDED, functionTaskTask2, & stack2 [STACKSIZE], 0, 0, 5, (unsigned) sizeof stack2, 0, 1, 0, 0, 0}, // Task2
		{4, SUSPENDED, functionTaskTask3, & stack3 [STACKSIZE], 0, 1, 4, (unsigned) sizeof stack3, 0, 1, 0, 0, 1} // Task3
	};

	ISR theISRs [] = {
		// function, register address, application
		{functionISRISRIn1, (ISR::RegisterAddress) DMA_SYSSRC3_ADDR, 0} // ISRIn1
	};

	Resource theResources [] = {
		// ceiling priority
		{3} // CommonResource
	};

	AppMode theAppModes [] = {
		// autostarted tasks
		{(1 << Task0)} // OSDEFAULTAPPMODE
	};

	Alarm theAlarms [] = {
		// alarm base, armed, absolute time, cycle time, activate task (/set event), task, event
		{{4711, 4711, 4711}, 0, 0, 0, 0, Task2, SyncEvent}
	};

	Application theApplications [] = {
		// startup hook, shutdown hook, error hook
		{0, ShutdownHook_App0, ErrorHook_App0}, // App0
		{StartupHook_App1, 0, 0} // App1
	};

}} // os::krn

// ISRIn1
Bool hw::irq::DMA_SYSSRC3::HWHandler () {
	os::krn::ASSched::Inst ().dispatchISR (ISRIn1);
	return true;
}

// can be overwritten by user specific main (),
// which must call StartOS () at the end
int CIAO_WEAK main () {
	hw::hal::In0::IRQ::enable ();
	hw::hal::In1::IRQ::enable ();
	hw::irq::ASC0_REC::enable ();
	hw::hal::SystemTimer::IRQ::enable ();
	StartOS (OSDEFAULTAPPMODE);

	return 0;
}

// debug: manually trigger interrupt
#include <tc1796b/dma.h>

Bool hw::irq::ASC0_REC::HWHandler () {
	char c;
	do {
		c = hw::dev::ASC0::Inst ().receive ();
	}
	while (c == 'p');

	if (c == '0') {
		//CPU_SRC2.bits.SRPN 	= pri;
		DMA_SYSSRC2.bits.SETR	= 1; 
	} else if (c == '1') {
		//CPU_SRC2.bits.SRPN 	= pri;
		DMA_SYSSRC3.bits.SETR	= 1; 
	}

	return true;
}

// }

// user code
#include "autosar.h"

DeclareTask (Task0);
DeclareTask (Task1);
DeclareTask (Task2);
DeclareTask (Task3);
DeclareResource (CommonResource);
DeclareEvent (SyncEvent);

// ISR 1
Bool hw::irq::DMA_SYSSRC2::HWHandler () {
	printf ("IRQ 1!\r\n");
	ISRType id = GetISRID ();
	printf ("My ISRID is: %i.\r\n", id);
	return true;
}

// ISR 2
ISR (ISRIn1) {
	printf ("XXXXXXXXXXXXXXXXXXXXXXXXX\r\n");
	printf ("IRQ 2 activates Task2!\r\n");
	ActivateTask (Task2);
	ISRType id = GetISRID ();
	printf ("My ISRID is: %i.\r\n", id);
	printf ("Calling ProtectionHook manually.\r\n");
	os::krn::OS::Inst ().internalProtectionHook (E_OK);
	ApplicationType app = GetApplicationID ();
	printf ("My AppID is: %i.\r\n", app);
}

//Bool hw::irq::STM_SRC0::HWHandler () {
//	printf ("IRQ from timer!\r\n");
//	printf ("IRQHandler sets SyncEvent for Task2\r\n");
//	SetEvent (Task2, SyncEvent);
//	hw::hal::SystemTimer::Inst ().irqAck ();
//	Schedule ();
//	return true;
//}

TASK (Task0) {
	printf ("Preemptable task starting...\r\n");
	//printf ("### Suspending OS interrupts.\r\n");
	//SuspendOSInterrupts ();
//	printf ("### Disabling ISRIn1 interrupts.\r\n");
//	DisableInterruptSource (ISRIn1);
	for (int i = 0; i < 100; i++) {
		printf (".");
	}
	ISRType id = GetISRID ();
	printf ("My ISRID is: %i.\r\n", id);
	ApplicationType app = GetApplicationID ();
	printf ("Activating an invalid task\r\n");
	ActivateTask (42);
	printf ("My AppID is: %i.\r\n", app);
	printf ("Activating Task2, causing rescheduling.\r\n");
	ActivateTask (Task2);
	printf ("\r\nPreemptable task ending...\r\n");
	ChainTask (Task1);
}

TASK (Task1) {
	printf ("\tNon-preemptable task starting...\r\n\t");
	for (int i = 0; i < 100; i++) {
		printf (".");
	}
	ISRType id = GetISRID ();
	printf ("My ISRID is: %i.\r\n", id);
	ApplicationType app = GetApplicationID ();
	printf ("My AppID is: %i.\r\n", app);
	//printf ("### Resuming OS interrupts.\r\n");
	//ResumeOSInterrupts ();
//	printf ("### Enabling ISRIn1 interrupts.\r\n");
//	EnableInterruptSource (ISRIn1);
	printf ("\r\n\tNon-preemptable task ending...\r\n");
	ChainTask (Task0);
}

TASK (Task2) {
	printf ("TASK2 TASK2 TASK2 TASK2!\r\n");
	printf ("Calling ProtectionHook manually.\r\n");
	os::krn::OS::Inst ().internalProtectionHook (E_OK);
	ApplicationType app = GetApplicationID ();
	printf ("TASK2: My AppID is: %i.\r\n", app);
	TerminateTask ();
}

/*
TASK (Task0) {
	int i;
	TaskType id;
	TaskStateType state_me;
	TaskStateType state_other;
	StatusType retvalue;

	printf ("Task0 has been started\r\n");
	for (i = 0; i < 5; i++) {
		printf( "Task0: loop %i\r\n", i);
	}

	printf ("Task0 sets WakeupAlarm\r\n");
	SetRelAlarm (WakeupAlarm, 160000000, 160000000);

	GetTaskID (id);
	printf ("My TaskID is: %i\r\n", id);
	if (id == INVALID_TASK) {
		printf ("My TaskID is invalid.\r\n");
	}

	printf ("Getting the common resource\r\n", id);
	GetResource (CommonResource);

	printf ("Activating the other task three times\r\n");
	retvalue = ActivateTask (Task1);
	retvalue = ActivateTask (Task1);
	retvalue = ActivateTask (Task1);
	printf ("Return value: %i\r\n", retvalue);
	printf ("Done activating the other task\r\n");

	printf ("Scheduling\r\n");
	Schedule ();

	printf ("Activating an invalid task\r\n");
	retvalue = ActivateTask (42);
	printf ("Return value: %i\r\n", retvalue);
	printf ("Done activating an invalid task\r\n");

	printf ("Releasing the common resource\r\n", id);
	ReleaseResource (CommonResource);

	printf ("Scheduling\r\n");
	Schedule ();

	GetTaskState (Task0, state_me);
	GetTaskState (Task1, state_other);
	printf ("My state is: %i, the one of the other task is: %i\r\n", state_me, state_other);

	//printf ("Task0 shuts down the OS\r\n");
	//ShutdownOS (E_OS_ACCESS);

	printf ("Task0 chains Task2\r\n");
	printf ("#####################################\r\n\r\n");
	retvalue = ChainTask (Task2);
	printf ("Return value: %i\r\n", retvalue);
	_debug ();
}

TASK (Task1) {
	static int run = 0;
	int i;
	TaskType id;
	TaskStateType state_me;
	TaskStateType state_other;
	printf ("    Task1 has been started (run %i)\r\n", run);
	run++;
	for (i = 0; i < 3; i++) {
		printf( "    Task1: loop %i\r\n", i);
	}
	GetTaskID (id);
	printf ("    My TaskID is: %i\r\n", id);
	if (id == INVALID_TASK) {
		printf ("    My TaskID is invalid.\r\n");
	}
	GetTaskState (Task1, state_me);
	GetTaskState (Task0, state_other);
	printf ("    My state is: %i, the one of the other task is: %i\r\n", state_me, state_other);

	printf ("    Task1 terminates\r\n");
	TerminateTask ();
	_debug ();
}

TASK (Task2) {
	printf ("Task2 has been started\r\n");

	printf ("Task2 activates Task3 (with lower priority)\r\n");
	ActivateTask (Task3);

	while (true) {
		printf ("Task2 waits for SyncEvent\r\n");
		WaitEvent (SyncEvent);
		printf ("Task2 got the SyncEvent\r\n");
		hw::hal::Out0::Inst ().toggle ();
		printf ("Task2 clears its event mask\r\n");
		ClearEvent (SyncEvent);
	}

	TerminateTask ();
	_debug ();
}
*/

TASK (Task3) {
	TaskStateType state_me;
	TaskStateType state_other;

	printf ("    Task3 has been started\r\n");

	while (true) {
		GetTaskState (Task3, state_me);
		GetTaskState (Task2, state_other);
		printf ("    My state is: %i, the one of the other task is: %i\r\n", state_me, state_other);

		//printf ("    Task3 schedules\r\n");
		//Schedule ();

		//while (hw::hal::In0::Inst ().get () == 0);
		printf ("    Task3 sets SyncEvent for Task2\r\n");
		SetEvent (Task2, SyncEvent);

		//printf ("    Task3 schedules\r\n");
		//Schedule ();
	}
	
	TerminateTask ();
	_debug ();
}

void ErrorHook (StatusType error) {
	StatusType retvalue;

	printf ("! ErrorHook called with error %i\r\n", error);

	printf ("! ErrorHook tries to activate an invalid task\r\n");
	retvalue = ActivateTask (42);
	printf ("! Return value: %i\r\n", retvalue);
	printf ("! ErrorHook is done activating an invalid task\r\n");
}

void ErrorHook_App0 (StatusType error) {
	printf ("! ErrorHook_App0 called with error %i\r\n", error);
}

void PreTaskHook () {
	TaskType current;

	GetTaskID (current);

	printf ("! PreTaskHook: new task: %i\r\n", current);
}

void PostTaskHook () {
	TaskType current;

	GetTaskID (current);

	printf ("! PostTaskHook: old task: %i\r\n", current);
}

void StartupHook () {
	printf ("! Doing some StartupHook stuff\r\n");
	printf ("! Illegally activating a task\r\n");
	StatusType retvalue = ActivateTask (Task1);
	printf ("! Return value: %i\r\n", retvalue);
}

void StartupHook_App1 () {
	printf ("! Doing some StartupHook_App1 stuff\r\n");
	printf ("! Illegally activating a task\r\n");
	StatusType retvalue = ActivateTask (Task1);
	printf ("! Return value: %i\r\n", retvalue);
}

void ShutdownHook (StatusType error) {
	printf ("! Doing some ShutdownHook stuff, error was: %i\r\n", error);
	printf ("! Illegally activating a task\r\n");
	StatusType retvalue = ActivateTask (Task1);
	printf ("! Return value: %i\r\n", retvalue);
}

void ShutdownHook_App0 (StatusType error) {
	printf ("! Doing some ShutdownHook_App0 stuff, error was: %i\r\n", error);
	printf ("! Illegally activating a task\r\n");
	StatusType retvalue = ActivateTask (Task1);
	printf ("! Return value: %i\r\n", retvalue);
}

ProtectionReturnType ProtectionHook (StatusType fatalerror) {
	StatusType ret = ActivateTask (Task2);
	printf ("! Doing some ProtectionHook stuff, error was: %i, result of illegal ActivateTask (): %i\r\n", fatalerror, ret);

	printf ("! Trying to ShutdownOS ()\r\n");
	ShutdownOS (E_OK);

	//return PRO_KILLTASKISR;
	return PRO_SHUTDOWN;
}
