#include <stdio.h>
#include <machine/wdtcon.h>
#include <machine/intrinsics.h>
#include <tc1796b/csfr/addr.h>
#include <tc1796b/port1-struct.h>
#include <tc1796b/scu.h>

#include "rs232.h"
#include "timer.h"
#include "leds.h"

#define TOGGLE_LED	(1<<15)

/* crystal on V100 is 16MHz */
/*#define CRYSTAL		16000000U*/
/* crystal on V101 is 20MHz */
#define CRYSTAL		20000000U

static PORT1_t *port1 = (PORT1_t*) P1_BASE;

//void _start() {
//	port1->IOCR12.bits.PC3 = 0x8;/* OUT_PPGPIO */
//	port1->OMR.reg |= TOGGLE_LED;
//	while(1);
//}

/************ MAIN Procedure ***************************************/

int main(void)
{
	unsigned int	K_Div, N_Div, P_Div;
	unsigned int	Hour, Min, Sec, OldSec;
	unsigned int	frequency;
	char			msg[50];
	PLL_CLC_t_nonv pllclc;
	unsigned int    active_led = 1;

	/* set speed to 128 MHz with 16Mhz Crystal or 150 MHz with 20MHz Crystal */
	pllclc.reg = 0;
	
#if (CRYSTAL == 16000000U)
	pllclc.bits.NDIV = 40-1;
	pllclc.bits.KDIV =  4-1;
	pllclc.bits.PDIV =  1-1;
	pllclc.bits.VCOSEL =  2;
#else
	pllclc.bits.NDIV = 30-1;
	pllclc.bits.KDIV =  4-1;
	pllclc.bits.PDIV =  1-1;
	pllclc.bits.VCOSEL =  2;
#endif
	
	/* setting endinit, password access to wdtcon0 */
	unlock_wdtcon();
	PLL_CLC.reg = pllclc.reg;
	lock_wdtcon();

	/* get frequency */
	pllclc.reg = PLL_CLC.reg;

	N_Div = pllclc.bits.NDIV + 1;
	K_Div = pllclc.bits.KDIV + 1;
	P_Div = pllclc.bits.PDIV + 1;

	if (pllclc.bits.BYPPIN)
		frequency = CRYSTAL;
	else
		frequency = CRYSTAL * N_Div / (K_Div * P_Div);

	/* initialize ASC0 dependant from frequency */
	if (!pllclc.bits.SYSFS)
		init_rs232 (frequency/2);
	else
		init_rs232 (frequency);

	/* set CPU- prio to 1 */
	_mtcr (ICR_ADDR & 0xffff, (_mfcr(ICR_ADDR & 0xffff) & 0xFFFFFF00) | 0x01);
	/* enable interrupts */
	_enable();

	/* send strings via ASC0 */
	message("\r\n\r\n\r\n");
	message("Hello World! (HIGHTEC version using GNU tools)\r\n\r\n");
	message("I am the TriBoard with TC1796\r\n");
	message("developed at\r\n");
	message("INFINEON Technologies AG in Munich\r\n");
	message("Department AI MC AE\r\n\r\n");
	message("Thomasiusplatz\r\n");
	message("D-81541 Munich\r\n");
	message("Tel.:+49-89-234-0\r\n");
	message("Fax.:+49-89-234-81785\r\n\r\n");
	message("If you have questions on this board or on the TriCore CPU,\r\n");
	message("see the manuals on the TriBoard CD.\r\n");
	message("This is the getting started demo.\r\n");
	message("Have fun working with me!\r\n\r\n");

	if (!pllclc.bits.SYSFS)
		frequency /= 2;
	sprintf(msg, "The Board is running at %d Hz\r\n\r\n", frequency);
	message(msg);

	Sec = 0;
	Min = 0;
	Hour = 0;
	OldSec = 59;

	/* initialize PORT1.15, 15 = 12+3 */
	port1->IOCR12.bits.PC3 = 0x8;/* OUT_PPGPIO */
	init_redleds();
	/* initialize timer dependant from frequency */
	init_timer (frequency);
	/* initialize now PORT1.3 to 1.0 just like 1.15 */
	port1->IOCR0.bits.PC0 = 0x20;
	port1->IOCR0.bits.PC1 = 0x20;
	port1->IOCR0.bits.PC2 = 0x20;
	port1->IOCR0.bits.PC3 = 0x20;
	/*	port1->OMR.reg = 0x000C0002; do some magic */

	while (1)
	{
		/* calculate the actual time */
		Sec = Ticks/1000;
		Sec = Sec%60;
		if (Sec == OldSec) continue;
		OldSec = Sec;
		Min = Ticks/60000;
		Min = Min%60;
		Hour = Ticks/3600000;
		Hour = Hour%24;
		/* send the new time via ASC0 */
		sprintf(msg, "\rrunning since: %02dh:%02dmin:%02dsec", Hour, Min, Sec);
		message(msg);
		/* toggle the LED */
		if (port1->IN.reg & TOGGLE_LED)
			port1->OMR.reg |= TOGGLE_LED<<16;
		else
			port1->OMR.reg |= TOGGLE_LED;
		light_leds(active_led);
		active_led = active_led << 1;
		if (active_led == (1 << 11))
		  active_led = 0x01;
	}

	return 0;
}
