/*====================================================================
* Project:
* Developed using:
* Function:  getting started sample with ASC0, interrupt and timer
*             functions for timer with TC1796
*====================================================================
* $Author: mike $
* $Revision: 1.8 $
* $Date: 2009-08-11 16:26:31 +0200 (Tue, 11 Aug 2009) $
*====================================================================
* $RCSfile: timer.c,v $
*====================================================================*/

#include <machine/wdtcon.h>
#include <machine/cint.h>
#include <machine/intrinsics.h>
#include <tc1796b/csfr/addr.h>
#include <tc1796b/stm.h>

#include "timer.h"

#define TICK_INTERRUPT		5

volatile unsigned int Ticks = 0;
static unsigned int   MSTim = 0;

/* timer interrupt routine */
void tick_irq(void)
{
	Ticks++;
	STM_CMP0.reg += MSTim;
}

void init_timer(unsigned int frequency)
{
	/* Get CPU revision to distinguish between StepA and StepB */
	unsigned char cpu_revision = _mfcr(CPU_ID_ADDR & 0xffff) & 0xFF;
	
	/* Install handler for timer interrupt. */
	_install_int_handler(TICK_INTERRUPT, (void (*) (int)) tick_irq, 0);

	/* initialize STM */
	/* first must set clock for STM and enable module */
	/* register is endinit protected */
	/* password access to wdtcon0 */
	unlock_wdtcon();
	/* setting clock to asc (enable) */
	STM_CLC.reg = 0x100;
	/* setting endinit, password access to wdtcon0 */
	lock_wdtcon();

	/* calculate value for compare */
	MSTim = frequency / 1000;
	STM_CMP0.reg = MSTim;
	STM_CMCON.bits.MSIZE0 = 24;

	if (cpu_revision >= 5)
		STM_ICR.reg = STM_ICR_CMP0EN_MASK;
	else
		/* StepA: Erratum STM_TC.002 --> select STMIR1 as interrupt output */
		STM_ICR.reg = STM_ICR_CMP0EN_MASK | STM_ICR_CMP0OS_MASK;
	
	/* enable service request node 0 of STM and assign it No./prio TICK_INTERRUPT */
	STM_SRC0.bits.SRPN = TICK_INTERRUPT;
	STM_SRC0.bits.SRE = 1;
}
