#include <machine/wdtcon.h>
#include <machine/intrinsics.h>
#include <tc1796b/csfr/types.h>	
#include <tc1796b/csfr/masks.h>	


namespace tc {
// 
	struct  __attribute__((aligned(16)))  UpperContext {
		union {
			PCXI_t_nonv		pcxi_typed;
			void*			pcxi;
		};
		union {
			PSW_t_nonv		psw_typed;
			long			psw;
		};
		union {
			void*			a10;
			void*			sp;
		};
		union {
			void*			a11;
			void*			ra;
		};
		long				d8;
		long				d9;
		long				d10;
		long				d11;
		void*				a12;
		void*				a13;
		void*				a14;
		void*				a15;
		long				d12;
		long				d13;
		long				d14;
		long				d15;
	};

	struct  __attribute__((aligned(16)))  LowerContext {
		union {
			PCXI_t_nonv		pcxi_typed;
			void*			pcxi;
		};
		union {
			void*			a11;
			void*			ra;
		};
		void* 				a2;
		void* 				a3;
		long 				d0;
		long 				d1;
		long 				d2;
		long 				d3;
		void* 				a4;
		void* 				a5;
		void* 				a6;
		void* 				a7;
		long 				d4;
		long 				d5;
		long 				d6;
		long 				d7;
	};

	union HalfContext {
		UpperContext uc;
		LowerContext lc;
	};

}
	PCXI_t_nonv first, second, third;
	tc::HalfContext *first_ea, *second_ea, *third_ea;
	
namespace tc {

	
	inline HalfContext* cx2ea( PCXI_t_nonv cx )  {
		unsigned int res = ( (cx.bits.PCXO << 6) ) | ( (cx.bits.PCXS << 28) );
		return (HalfContext*) res; 
	}
	
	inline void setcxea( PCXI_t_nonv& cx, HalfContext* ea ) {
		cx.bits.PCXO = (long) ea >> 6;
		cx.bits.PCXS = (long) ea >> 12;
	}
	
	PCXI_t_nonv ea2cx( HalfContext*, bool, bool, unsigned char ) __attribute__((always_inline));
	inline PCXI_t_nonv ea2cx( HalfContext* ea, bool UL, bool PIE = false, unsigned char PCPN = 0 ) {
		PCXI_t_nonv cx;
		setcxea( cx, ea);
		cx.bits.UL 		= UL,
		cx.bits.PIE 	= PIE;
		cx.bits.PCPN 	= PCPN;
		return cx;
	}
/*	
	// TODO: atomicity!
	// TODO: error handling (if FCX == LCX);
	inline PCXI_t_nonv alloc2csa() {
		
		first.reg = _mfcr(0xFE38);
		first_ea = cx2ea( first );
		second.reg = first_ea->lc.pcxi_typed.reg;
		second_ea = cx2ea( second );
		third.reg = second_ea->lc.pcxi_typed.reg;
		third_ea = cx2ea( third );
		_mtcr( 0xFE38, third.reg )
		second_ea->lc.pcxi = 0;	
		return first;
	}
	*/
	inline PCXI_t_nonv alloc2csa() {
		PCXI_t_nonv first, third;		
		_disable();
		first.reg = _mfcr($fcx);
		third.reg = cx2ea( first )->lc.pcxi_typed.reg;
		third.reg = cx2ea( third )->lc.pcxi_typed.reg;
		_mtcr( $fcx, third.reg );
		_enable();
		return first;
	}
	
};
	

extern "C" {
//	int __errno;
	void _init_args() 
	{
		asm("mov.aa %a2, %a10");
	}
	void _exit() {}
	void __main() {}
}



void kickoff(register void (*start)(void*), register void* param ) {
	// start is passed in a4, param in a5
	asm("jli %a4");
}

using namespace tc;

void settle( PCXI_t_nonv cx, void* tos, void (*start)(void*), void* param ) {
	LowerContext* lc = (LowerContext*) cx2ea( cx );
	UpperContext* uc = (UpperContext*) cx2ea( lc->pcxi_typed );
	
	lc->ra = (void*) &kickoff;
	lc->a4 = (void*) start;
	lc->a5 = param;
	uc->sp = tos;
	uc->pcxi_typed.reg			= 0;
	uc->pcxi_typed.bits.PIE 	= 1;
}

extern "C" void dispatch( register PCXI_t_nonv& from, register PCXI_t_nonv to ) {
	_svlcx();
	from.reg = _mfcr($pcxi);
	_mtcr($pcxi, to.reg );
	_rslcx();
}


// forward declarations
namespace hw { namespace hal {
	struct TCB;
}}	// namespace hw::hal




namespace hw { namespace hal {
	
struct BasicTCB {
	
	typedef void (*ThreadFunc)(TCB* me);

	PCXI_t_nonv pcxi_;			// the current context pointer

	ThreadFunc 	start_;
	void**		tos_;
};

class TCB : protected BasicTCB {
public:

	void settle ( void** tos, ThreadFunc startAddress ) {
		start_ 		= startAddress;
		tos_   		= tos;
		pcxi_.reg	= 0;
	}

	void resettle() {
		pcxi_.reg = 0;
	}

	void __attribute__((noinline)) dispatch ( register TCB& to ) {

		// save lower context (upper context has been saved implicitly by call)
		// save leaving thread context pointer
		_svlcx();													
		pcxi_.reg = _mfcr( $pcxi );									
		
		// if a thread hasn't been dispatched yet
		// we start it up from the TCB data
		if( to.pcxi_.reg == 0 ) {
			
			// we have to ensure that the following is still available 
			// after switching the stack, hence we store it in registers
			register void* tos 			= to.tos_;							
			register void (*start)() 	= (void (*)()) to.start_;
			register TCB* tcb 	= &to;
			
			// do the stack switch 
			__asm__("mov.aa %%sp, %[tos]" : : [tos] "a" (tos) : "sp" );
			// pass tcb as parameter on the stack
			__asm__("st.a [+%%sp], -4, %[tcb]": : [tcb] "a" (tcb) : "sp" ); 
			
			// Call thread function. Thereby PCXI of father is implicitly saved as part of the upper context.
			// This makes it possible that the new thread returns to its father (synchronous execution)
			// as long as the father has not been redispatched meanwhile
			start();												
			_rslcx();
			return;													
		}
		
		// otherwise simply restore the context pointer
		// and lower context (upper context restored implicitly by ret)
		_mtcr($pcxi, to.pcxi_.reg );
		_isync();						// IMPORTANT, otherwise rslcx might use the old context which ends in dispatching to itself
		_rslcx();
	}
	
};

}}	// namespace hw::hal	

	


long stack1 [ 256 ];
long stack2 [ 256 ];

//PCXI_t_nonv toc1, toc2, toc_main;
/*
void t1(void*) {
	while (true )
		dispatch( toc1, toc2 );
}

void t2(void*) {
	while (true )
		dispatch( toc2, toc1 );
}
*/

hw::hal::TCB tcb1;
hw::hal::TCB tcb2;
hw::hal::TCB tcb_main;

void __attribute__ ((stackparm)) thread1( hw::hal::TCB* me ) {
	while (true ) {
		tcb1.dispatch( tcb2 );
	}
}

void __attribute__ ((stackparm)) thread2( hw::hal::TCB* me  ) {
/*	while (true ) {
		tcb2.dispatch( tcb1 );
	}
*/
}

int main()
{
	
	tcb1.settle( (void**) &stack1[256], &thread1 );
	tcb2.settle( (void**) &stack2[256], &thread2 );
	tcb_main.dispatch( tcb1 );
	
/*
	toc1 = tc::alloc2csa();
	toc2 = tc::alloc2csa();
	
	first_ea = tc::cx2ea2( first );
	asm("nop");
	second_ea = tc::cx2ea( first );
	asm ("nop");
	settle( toc1, &stack1[256], t1 );
	settle( toc2, &stack2[256], t2 );
	
	dispatch( toc_main, toc1 );

*/
}
