<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
version="1.0">
<xsl:import href="../ComStacksCommon.xsl"/>
<xsl:output method="text"/>
<xsl:strip-space elements="*" />

<xsl:template match="/ComStacks">
#include &#34;node.h&#34;
#include &#34;can_Cfg.h&#34;
<xsl:apply-templates match="/ComStacks/Node"/>
</xsl:template>

<!-- Template for Can group on one node -->
<xsl:template match="/ComStacks/Node/Can">
namespace hw { namespace dev { namespace can {
<xsl:apply-templates select="Module"/>
};};};
</xsl:template>

<!-- Template for a Can Module -->
<xsl:template match="/ComStacks/Node/Can/Module">
namespace <xsl:value-of select="@Name"/> {

// The following is the configuration of the module <xsl:value-of select="@Name"/>
<xsl:apply-templates select="Controller" />
ControllerConfigType * DefaultCtrlConfigs[]={
<xsl:for-each select="Controller">	ControllerId<xsl:value-of select="@ControllerId"/><xsl:text>_Configs</xsl:text>
	<xsl:if test="position() != last()">,
</xsl:if>
</xsl:for-each>
};

// Structure: ControllerRef, Direction, hw filter id, hw filter mask
HwObjectConfigType HwObjectConfigs[] = {
<xsl:apply-templates select="TxHwObject | RxHwObject"/>
};

ConfigType Config = {
	HwObjectConfigs, <xsl:value-of select="count(./TxHwObject | ./RxHwObject)"/>,
	DefaultCtrlConfigs, <xsl:value-of select="count(Controller)"/>
};

};
</xsl:template>


<!-- Template for a can controller -->
<xsl:template match="/ComStacks/Node/Can/Module/Controller">
ControllerConfigType ControllerId<xsl:value-of select="@ControllerId"/>_Configs[] = {
<xsl:for-each select="Config">
	<xsl:sort select="@ConfigId" order="ascending" data-type="number"/>
	<xsl:apply-templates select="."/>
	<xsl:if test="position() != last()">,
</xsl:if>
</xsl:for-each>
};

</xsl:template>

<!-- Template for a can controller configuration -->
<xsl:template match="/ComStacks/Node/Can/Module/Controller/Config">
	<xsl:text>	{</xsl:text>
	<xsl:value-of select="@ConfigId"/>
	<xsl:text>, </xsl:text>
	<xsl:value-of select="@PropSeg"/>
	<xsl:text>, </xsl:text>
	<xsl:value-of select="@PhaseSeg1"/>
	<xsl:text>, </xsl:text>
	<xsl:value-of select="@PhaseSeg1"/>
	<xsl:text>}</xsl:text>
</xsl:template>


<!-- Template for one Hardware object -->
<xsl:template match="/ComStacks/Node/Can/Module/RxHwObject | /ComStacks/Node/Can/Module/TxHwObject">
	<xsl:text>	HW_DEV_CAN_HW_OBJECT_CONFIG_INITIALIZER(</xsl:text>
	<xsl:choose>
		<xsl:when test="name(./StandardCanHwFilter | ./ExtendedCanHwFilter) = 'StandardCanHwFilter'">
			<xsl:value-of select="./StandardCanHwFilter/@Id" /><xsl:text>, </xsl:text>
			<xsl:value-of select="./StandardCanHwFilter/@Mask" /><xsl:text>, </xsl:text>
		</xsl:when>
		<xsl:when test="name(./StandardCanHwFilter | ./ExtendedCanHwFilter) = 'ExtendedCanHwFilter'">
			<xsl:value-of select="./ExtendedCanHwFilter/@Id" />
			<xsl:text> + (1&#60;&#60;(sizeof(IdType)*8-1)), </xsl:text>
			<xsl:value-of select="./ExtendedCanHwFilter/@Mask" />
			<xsl:text> + (1&#60;&#60;(sizeof(IdType)*8-1)), </xsl:text>
		</xsl:when>
		<xsl:otherwise>
			<xsl:text>0, 0, </xsl:text>
		</xsl:otherwise>
	</xsl:choose>
	<xsl:value-of select="@ControllerRef" /><xsl:text>, </xsl:text>
	<xsl:if test="name(.) = 'RxHwObject'">RECEIVE</xsl:if>
	<xsl:if test="name(.) = 'TxHwObject'">TRANSMIT</xsl:if>

	<xsl:if test="position() != last()">),
</xsl:if>
	<xsl:if test="position() = last()">)</xsl:if>
</xsl:template>

</xsl:stylesheet>
