#!/usr/bin/perl

use warnings;
use strict;

use SystemConfiguration;

if(@ARGV != 3 || $ARGV[0] !~ /\.xml$/ || $ARGV[2] !~ /\.h$/) {
	print STDERR "Usage: $0 <config.xml> <os.config> <objects.h>\n";
	exit 1;
}

my $cfg = SystemConfiguration->new($ARGV[0], $ARGV[1]);

## Here comes the actual generation code
open (SRC, ">$ARGV[2]") || die "cannot open $ARGV[2]\n";

print SRC <<EOH;
// generated CiAO runtime
#ifndef OBJECTS_H_
#define OBJECTS_H_

#include "as/autosar_types.h"

EOH

foreach my $o( (
		[[$cfg->getTasks()], 'TaskType', 0],
		[[$cfg->getISRs()], 'ISRType', 0],
		[[$cfg->getAlarms()], 'AlarmType',0 ],
		[[$cfg->getResources()], 'ResourceType',0],
		[[grep { $_->{name} !~ 'OSDEFAULTAPPMODE' } $cfg->getAppModes()], 'AppModeType',1],
		[[$cfg->getApplications()], 'ApplicationType',0],
	) ) {
	my @objs = @{$o->[0]};
	for(0..$#objs) {
		print SRC "const $o->[1] $objs[$_]->{name} = ".($_+$o->[2]).";\n";
	}
}

my $i=1;
my %usedEvents = ();
foreach my $eve ($cfg->getEvents()) {
	my $event = $cfg->getEvent($eve);

	if ( exists $event-> {mask} ) {
		print SRC "const EventMaskType $eve = $event->{mask};\n";
		$usedEvents{$event->{mask}} = 1;
	}

}

foreach my $eve ($cfg->getEvents()) {
	my $event = $cfg->getEvent($eve);

	next if exists $event-> {mask};

	while ( exists $usedEvents{$i} ) {
		$i <<= 1;
	}

	print SRC "const EventMaskType $eve = $i;\n";
	$usedEvents{$i} = 1;
}

print SRC "#endif\n";

close(SRC);
